<?php

namespace app\components;

use Yii;
use yii\base\Component;

/**
 * HelperOrder.php
 *
 * PHP Version 5.3.0
 * 
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   SVN: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */

/**
 * HelperOrder.php
 *
 * // SUS COMENTARIOS AQUI
 *
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */
class HelperOrder extends Component {

    private $_count = 2;
    private $_idCo;
    private $_idMotivo;
    private $_fechaEntrega;
    private $_diasEntrega;
    private $_clientSiesa;

    //Constantes para la cabecera

    const NUMERO_REG_HEADER = '2';
    const TIPO_REG_HEADER = '440';
    const SUBTIPO_REG = '00';
    const VERSION_REG = '03';
    const CIA = '1';
    const CONSEC = '1';
    const CONSEC_AUTO_REG = '1';
    const ID_TIPO_DOC = 'CAJ';
    const ID_CLASE_DOCTO = '75';
    const IND_ESTADO = '1';
    const ID_CONCEPTO_HEADER = '607';
        
    const NUM_DOCTO_REFERENCIA = '';
    const REFERENCIA = '';
    const ID_CARGUE = '';
    const IND_IMPRESO = '0';
    //Constantes para los items
    const TIPO_REG_ITEM = '441';
    const ID_CONCEPTO = '607';
    const ID_ITEM = '';
    const ID_EXT1_DETALLE = '';
    const ID_EXT2_DETALLE = '';
    const CODIGO_BARRAS = '';
    const ID_UN_MOVTO = '001';
    const ID_CCOSTO_MOVTO = '';
    const ID_PROYECTO = '';
    const PRECIO_UNITARIO = '0';
    const CANT_2_PEDIDA = '000000000000000.0000';

    /**
     * Constructor de la clase
     * 
     * @return    void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S.     
     * @access    public     
     */
    public function __construct() {
        
    }

    /**
     * Funcion desctructora 
     * 
     * @return    void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S. 
     * @access    public     
     */
    public function __destruct() {
        
    }

    /**
     * Metodo para crear la cabecera del pedido
     * 
     * @param Order $model Modelo Order
     * 
     * @return boolean|\HeaderOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createHeader($model) {
        if (is_object($model)) {
            try {
                
                $error = false;
           
                $nit = $model->nit;
                $this->_idMotivo = '01';
                $this->_idCo = $model->id_co;
                $this->_clientSiesa = new ClientSiesa();
                $this->_fechaEntrega = str_replace('-', '', $model->date);

                $diasEnrega = $model->getNumDiasEntrega($model->date);                
                if (!$diasEnrega) {
                    $error = true;
                } else {
                    $this->_diasEntrega = $diasEnrega;
                }
                $solicitante = $model->getSolicitante($model->nit);
                $codigo = isset($solicitante['codigo'])? $solicitante['codigo'] : $nit;

                if ($error) return false;

                $headerOrder = new HeaderOrder();
                $headerOrder->numeroReg = self::NUMERO_REG_HEADER;
                $headerOrder->tipoReg = self::TIPO_REG_HEADER;
                $headerOrder->subTipoReg = self::SUBTIPO_REG;
                $headerOrder->versionReg = self::VERSION_REG;
                $headerOrder->cia = $model->id_cia;
                $headerOrder->consecAutoReg = self::CONSEC_AUTO_REG;
                $headerOrder->idCo = $this->_idCo; 
                $headerOrder->idTipoDocto = self::ID_TIPO_DOC; 
                $headerOrder->consecDocto = self::CONSEC;
                $headerOrder->idFecha = str_replace('-', '', date('Y-m-d')); //Formatro AAAAMMDD
                $headerOrder->idTercero = ''; 
                $headerOrder->idTerceroSolicitante = $codigo; 
                $headerOrder->fechaEntrega = $this->_fechaEntrega; //Formatro AAAAMMDD
                $headerOrder->numDiasEntrega = $diasEnrega;
                $headerOrder->idClaseDocto = self::ID_CLASE_DOCTO;
                $headerOrder->indEstado = self::IND_ESTADO;
                $headerOrder->indImpresion = self::IND_IMPRESO;
                $headerOrder->notas = 'No. Requisicion APP Chefbrger: '
                        . $model->id . ' -- ' . $model->description;
                $headerOrder->idConcepto = self::ID_CONCEPTO_HEADER;
                $headerOrder->idBodegaSalida = $model->id_cellar_out; 
                $headerOrder->idBodegaEntrada = $model->id_cellar_in; 
                $headerOrder->referencia = 'REQ: '.$model->num_req;
                $headerOrder->idUbicacionEnt = ''; //PENDIENTE
                $headerOrder->idCargue = self::ID_CARGUE;
                $headerOrder->idCargue = '';
                //$headerOrder->numDoctoReferencia = $model->num_req;
                //$headerOrder->idProyecto = '';
                return $headerOrder;
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                return false;
            }
        }
        return false;
    }

    /**
     * Metodo para crear el objeto de items
     * 
     * @param OrderProduct $model      Modelo de tipo OrderProduct
     * @param int          $countItems Contador de items
     * 
     * @return boolean|\ItemOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createItem($model, $countItems, $idBodega, $unitMesure) {

        if (is_object($model)) {
            try {
                $error = false;

                if (!$idBodega) $error = true;
                if ($error) return false;

                $this->_count = $this->_count + 1;

                $item = new ItemOrder();
                $item->numeroReg = $this->_count; //Consecutivo del item
                $item->tipoReg = self::TIPO_REG_ITEM;
                $item->subtipoReg = self::SUBTIPO_REG;
                $item->versionReg = self::VERSION_REG;
                $item->cia = self::CIA; // id de la compañia 
                $item->idCo = $this->_idCo; // f201_id_co_factura con la tabla t201
                $item->idTipoDocto = self::ID_TIPO_DOC; //t144_mc_motivo_impto_asumido - f021_id
                $item->consecDocto = self::CONSEC; //Identificador del encabezado
                $item->nroRegistro = $countItems; //Cantidad de registros - numero de items
                $item->idItem = self::ID_ITEM; //Id del item t120_mc_item columna f120_rowid;                
                $item->referenciaItem = $model->reference; 
                $item->codigoBarras = self::CODIGO_BARRAS;
                $item->idExt1Detalle = self::ID_EXT1_DETALLE;
                $item->idExt2Detalle = self::ID_EXT2_DETALLE;
                $item->idBodega = $idBodega; //SE REQUIERE LA BODEGA SE SACA DE TABLAS
                $item->idConcepto = self::ID_CONCEPTO;
                $item->idMotivo = $this->_idMotivo; //SE REQUIERE EL MOTIVO SE SACA DE TABLAS
                $item->idUnidadMedida = $unitMesure; //consulta unidad de medida
                $item->cantPedidaBase = $model->quantity; //Cantidad pedida formato 000000000000000.0000
                $item->cant2pedida = self::CANT_2_PEDIDA; //formato 000000000000000.0000
                $item->fechaEntrega = $this->_fechaEntrega; //Formatro AAAAMMDD
                $item->numDiasEntrega = $this->_diasEntrega;
                $item->idCoMovto = $this->_idCo;
                $item->campo = '';
                $item->idCcostoMovto = self::ID_CCOSTO_MOVTO;
                $item->idProyecto = self::ID_PROYECTO;
                $item->notas = ''; 
                $item->descVariable = ''; 
                $item->idUnMovto = self::ID_UN_MOVTO;
                $item->precioUnitario = self::PRECIO_UNITARIO;
                $item->idUbicacionSal = '';                
                return $item;
            } catch (Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                return false;
            }
        }
    }
    
    /**
     * Retorna el Id de centro de operaciones
     * 
     * @param Order $model Modelo Order
     * 
     * @return boolean|\HeaderOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getIdCo(){
        return $this->_idCo;
    }

}

?>
