<?php

namespace app\controllers;

use app\models\CalculatorMainItems;
use app\models\CalculatorSecondaryItems;
use app\models\CalculatorSecondaryItemsSearch;
use app\models\Param;
use Yii;
use app\models\CalculatorMainItemsSearch;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

/**
 * Class CalculateController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller in charge of managing the calculator module, with the main and secondary items
 * @package app\controllers
 */
class CalculateController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * List of References enabled for the calculator
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new CalculatorMainItemsSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        //Tipos de Referencias o Items principales
        $types = json_decode(Param::findOne(['key' => 'types_references'])->value, true);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'types' => $types
        ]);
    }

    /**
     * Creation of References for the calculator module (Main and secondary items)
     * @return array|string|\yii\web\Response
     * @throws \Throwable
     * @throws \yii\db\StaleObjectException
     */
    public function actionCreate()
    {
        $model = new CalculatorMainItems();
        // Tipos de Referencias para Referencias principales
        $types = json_decode(Param::findOne(['key' => 'types_references'])->value, true);
        // Unidades de medida para Referencias secundarios
        $measurementUnits = json_decode(Param::findOne(['key' => 'measurement_units'])->value, true);
        $post = Yii::$app->request->post();
        $errors = [];

        if (empty($post) == false) {
            // Creación de Referencia Principal
            $calculate = new CalculatorMainItems();
            $calculate->name = $post['CalculatorMainItems']['name'];
            $calculate->code_type = $post['CalculatorMainItems']['code_type'];

            if (empty($post['CalculatorMainItems']['quantity_grams']) == TRUE && empty($post['CalculatorMainItems']['quantity_liters']) == TRUE) {
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'message' => 'Se debe ingresar la cantidad en litros o gramos',
                ];
            }

            if (empty($post['CalculatorMainItems']['quantity_grams']) == FALSE) {
                $calculate->quantity_grams = $post['CalculatorMainItems']['quantity_grams'];
            }

            if (empty($post['CalculatorMainItems']['quantity_liters']) == FALSE) {
                $calculate->quantity_liters = $post['CalculatorMainItems']['quantity_liters'];
            }

            // Validación de datos
            if ($calculate->validate()) {
                $calculate->save();
                $calculateSecondary = [];
                $grams_secundary = 0;
                $lts_secundary = 0;
                $errors = [];

                // Almacenamientos de Referencias secundarias relacionada a Item principal
                foreach ($post['Calculate_secondary'] as $key => $value) {
                    $calculateSecondary[$key] = new CalculatorSecondaryItems();
                    $calculateSecondary[$key]->id_item = $calculate->id;
                    $calculateSecondary[$key]->code = $value['code'];
                    $calculateSecondary[$key]->description = $value['description'];
                    $calculateSecondary[$key]->equivalence = $value['equivalence'];
                    $calculateSecondary[$key]->code_unity = $value['code_unity'];

                    if ($calculateSecondary[$key]->code_unity == 'g' || $calculateSecondary[$key]->code_unity == 'kg') {
                        $grams_secundary += $calculateSecondary[$key]->equivalence;
                    } else {
                        $lts_secundary += $calculateSecondary[$key]->equivalence;
                    }

                    if ($calculateSecondary[$key]->validate() == FALSE) {
                        // Error de almacenamiento - Eliminación de Items creados
                        CalculatorSecondaryItems::deleteAll('id_item = ' . $calculate->id);

                        //Lectura de errores
                        foreach ($calculateSecondary[$key]->errors as $error) {
                            foreach ($error as $min_error) {
                                $errors[] = $min_error;
                            }
                        }

                        if (empty($errors) == false) {
                            $calculate->delete();
                            break;
                        }
                    }
                }

                if (empty($errors) == false) {
                    // Error con equivalencias
                    CalculatorSecondaryItems::deleteAll('id_item = ' . $calculate->id);
                    $calculate->delete();
                }

                foreach ($calculateSecondary as $value) {
                    $value->save();
                }
            } else {
                foreach ($calculate->errors as $error) {
                    foreach ($error as $min_error) {
                        $errors[] = $min_error;
                    }
                }
            }

            // Respuesta de errores por AJAX
            if (empty($errors) == FALSE) {
                $error = '<ul><li>';
                $error .= implode('</li><li>', $errors);
                $error .= '</ul>';
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'message' => $error,
                ];
            }

            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => 'SUCCESS',
                'id' => $calculate->id,
            ];
        } else {
            return $this->render('create', [
                'model' => $model,
                'types' => $types,
                'measurementUnits' => $measurementUnits
            ]);
        }
    }

    /**
     * Viewing References of the Calculator Module with Main and Secondary References
     * @param $id
     * @return string
     * @throws NotFoundHttpException
     */
    public function actionView($id)
    {
        // Referencia principal
        $searchModel = new CalculatorMainItemsSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        // Referencias Secundarias
        $searchModelSecondary = new CalculatorSecondaryItemsSearch();
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $queryParams['CalculatorSecondaryItemsSearch']['id_item'] = $id;
        $references = $searchModelSecondary->search($queryParams);
        // Tipos de Referencias Principales
        $types = json_decode(Param::findOne(['key' => 'types_references'])->value, true);
        // Unidades de Medida
        $measurementUnits = json_decode(Param::findOne(['key' => 'measurement_units'])->value, true);

        // Visualización de registro
        return $this->render('view', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'types' => $types,
            'measurementUnits' => $measurementUnits,
            'model' => $this->findModel($id),
            'references' => $references
        ]);
    }

    /**
     * Main and secondary reference update
     * @param $id
     * @return array|string|\yii\web\Response
     * @throws NotFoundHttpException
     */
    public function actionUpdate($id)
    {
        // Referencia Principal
        $model = $this->findModel($id);
        // Tipos de Referencias principales
        $types = json_decode(Param::findOne(['key' => 'types_references'])->value, true);
        // Unidades de medida para Referencias Secundarias
        $measurementUnits = json_decode(Param::findOne(['key' => 'measurement_units'])->value, true);
        $post = Yii::$app->request->post();
        // Referencias Secundarias
        $references = CalculatorSecondaryItems::find()->where(['id_item' => $id])->asArray()->all();
        $errors = [];

        if (empty($post) == false) {
            if (empty($post['CalculatorMainItems']) == false) {
                $model->load($post);

                //Validación de datos
                if ($model->validate()) {
                    if (empty($post['Calculate_secondary']) == FALSE) {
                        $model->save();
                        CalculatorSecondaryItems::deleteAll('id_item = ' . $model->id);

                        $grams_secundary = 0;
                        $lts_secundary = 0;
                        $calculateSecondary = [];
                        $errors = [];

                        //Actualización de Referencias Secundarias

                        foreach ($post['Calculate_secondary'] as $key => $value) {
                            $calculateSecondary[$key] = new CalculatorSecondaryItems();
                            $calculateSecondary[$key]->id_item = $model->id;
                            $calculateSecondary[$key]->code = $value['code'];
                            $calculateSecondary[$key]->description = $value['description'];
                            $calculateSecondary[$key]->equivalence = $value['equivalence'];
                            $calculateSecondary[$key]->code_unity = $value['code_unity'];

                            if ($calculateSecondary[$key]->code_unity == 'g' || $calculateSecondary[$key]->code_unity == 'kg') {
                                $grams_secundary += $calculateSecondary[$key]->equivalence;
                            } else {
                                $lts_secundary += $calculateSecondary[$key]->equivalence;
                            }

                            if ($calculateSecondary[$key]->validate() == FALSE) {
                                $errors = [];
                                foreach ($calculateSecondary[$key]->errors as $error) {
                                    foreach ($error as $min_error) {
                                        $errors[] = $min_error;
                                    }
                                }

                                if (empty($errors) == false) {
                                    break;
                                }
                            }

                            $calculateSecondary[$key]->save();
                        }
                    }
                } else {
                    // Array de errores
                    foreach ($model->errors as $error) {
                        foreach ($error as $min_error) {
                            $errors[] = $min_error;
                        }
                    }
                }

                // Respuesta de errores por AJAX
                if (empty($errors) == FALSE) {
                    $error = '<ul><li>';
                    $error .= implode('</li><li>', $errors);
                    $error .= '</ul>';
                    \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                    return [
                        'status' => 'ERROR',
                        'message' => $error,
                    ];
                }

                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'SUCCESS',
                    'id' => $model->id,
                ];
            }
        }

        return $this->render('update', [
            'model' => $model,
            'types' => $types,
            'measurementUnits' => $measurementUnits,
            'references' => $references
        ]);
    }

    /**
     * Get of Main Reference
     * @param $id
     * @return CalculatorMainItems|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = CalculatorMainItems::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    /**
     * Delete of Main Reference and related secondary
     * @param $id
     * @return \yii\web\Response
     * @throws \Throwable
     */
    public function actionDelete($id)
    {
        try {
            $this->findModel($id)->delete();
            CalculatorSecondaryItems::deleteAll('id_item = ' . $id);
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            return $this->redirect(['index']);
        }
    }

    /**
     * Result according to calculator by Main Reference and equivalences of secondary references
     * de Tipo Simple
     * @return array
     */
    public function actionGetQuantity()
    {
        if (empty($_POST) == false) {
            // Id Referencia Principal
            $item = $_POST['item'];
            $units = 0;
            if (empty($_POST['units']) == FALSE) {
                $units = $_POST['units'];
            }
            $grams = $_POST['grams'];
            // Unidades de medida de Referencias secundarias
            $measurementUnits = json_decode(Param::findOne(['key' => 'measurement_units'])->value, true);
            // Referencia Principal
            $item = CalculatorMainItems::findOne(['id' => $item]);

            if (empty($item) == FALSE && isset($units) == TRUE) {
                $item = $item->toArray();
                // Cálculo de item simples
                if ($item['code_type'] == CalculatorMainItems::TYPE_SIMPLE) {
                    // Referencias secundarias
                    $reference = CalculatorSecondaryItems::findOne(['id_item' => $item['id']])->toArray();

                    if (empty($reference) == FALSE) {
                        if (empty($grams) == FALSE && empty($item['quantity_grams']) == FALSE) {
                            $units += $grams / $item['quantity_grams'];
                        }

                        if ($units > 0) {
                            $total = round($units * $reference['equivalence'], 2);
                        } else {
                            $total = 0;
                        }

                        if (empty($total) == FALSE) {
                            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                            return [
                                'total' => $total,
                                'status' => 'OK',
                                'unity' => $measurementUnits[$reference['code_unity']],
                            ];
                        } else {
                            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                            return [
                                'status' => 'ERROR',
                                'message' => 'Por favor ingresar Unidades y/o Gramos para calcular la equivalencia.',
                            ];
                        }
                    } else {
                        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                        return [
                            'status' => 'ERROR',
                            'message' => 'La opción seleccionada no tiene Referencias asociadas.',
                        ];
                    }
                } else if ($item['code_type'] == CalculatorMainItems::TYPE_COMPOSED) {
                    // Referencias secundarias
                    $references = CalculatorSecondaryItems::find()->where(['id_item' => $item['id']])->asArray()->all();

                    if (empty($references) == FALSE) {
                        $totales = [];

                        if (empty($grams) == FALSE && empty($item['quantity_grams']) == FALSE) {
                            $units += round($grams / $item['quantity_grams'], 3);
                        }

                        foreach ($references as $key => $value) {
                            if ($value['code_unity'] == 'g' || $value['code_unity'] == 'kg') {
                                $quantityTotal = $item['quantity_grams'];
                                $quantityTotal = $quantityTotal/1000;
                            } else {
                                $quantityTotal = $item['quantity_liters'];
                            }

                            if ($quantityTotal > 0) {
                                $percentage = round(($value['equivalence'] * 100) / $quantityTotal, 2);
                                $quantityFull = round($quantityTotal * $units, 3);
                                $total = round((($quantityFull * $percentage) / 100), 1);

                                if ($total > 0) {
                                    $totales[$key]['code'] = $value['code'];
                                    $totales[$key]['description'] = $value['description'];
                                    $totales[$key]['total'] = $total;
                                    $totales[$key]['unity'] = $measurementUnits[$value['code_unity']];
                                }
                            } else if ($units == 0) {
                                $totales[$key]['code'] = $value['code'];
                                $totales[$key]['description'] = $value['description'];
                                $totales[$key]['total'] = 0;
                                $totales[$key]['unity'] = '';
                            }
                        }

                        if (empty($totales) == TRUE) {
                            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                            return [
                                'status' => 'ERROR',
                                'message' => 'Por favor ingresar Unidades y/o Gramos para calcular la equivalencia.',
                            ];
                        } else {
                            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                            return [
                                'status' => 'OK',
                                'response' => $totales,
                            ];
                        }
                    } else {
                        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                        return [
                            'status' => 'ERROR',
                            'message' => 'La opción seleccionada no tiene Referencias asociadas.',
                        ];
                    }
                }
            }
        }
    }

    public function actionGetField()
    {
        if (empty($_POST) == false) {
            // Id Referencia Principal
            $product = $_POST['product'];
            // Referencia Principal
            $item = CalculatorMainItems::findOne(['id' => $product]);
            $liters = FALSE;
            $grams = FALSE;

            if (empty($item->quantity_liters) == FALSE) {
                $liters = TRUE;
            }

            if (empty($item->quantity_grams) == FALSE) {
                $grams = TRUE;
            }

            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => 'OK',
                'liters' => $liters,
                'grams' => $grams,
            ];
        }
    }
}
