<?php

namespace app\controllers;

use app\models\Inventory;
use app\models\Usrpdv;
use Yii;
use app\models\Configpdv;
use app\models\ConfigpdvSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * Class CalculateController
 * @author  Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller in charge of managing the module of the PDV (Point of Sale), relating it to the Operations Center, Operation Group and Cellar
 * @package app\controllers
 */
class ConfigpdvController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Configpdv models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ConfigpdvSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * PDV visualization
     * @param $id
     * @return string
     * @throws NotFoundHttpException
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * New relationship of PDV
     * @return string|\yii\web\Response
     */
    public function actionCreate()
    {
        $model = new Configpdv();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * PDV relation update
     * @param $id
     * @return string|\yii\web\Response
     * @throws NotFoundHttpException
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Delete of PDV relation
     * @param $id
     * @return \yii\web\Response
     * @throws NotFoundHttpException
     * @throws \Throwable
     * @throws \yii\db\StaleObjectException
     */
    public function actionDelete($id)
    {
        $model = $this->findModel($id);
        //Usrpdv::deleteAll(['code_pdv' => $model->code_pdv]);
        //Inventory::deleteAll(['code_pdv' => $model->code_pdv, 'id_cellar' => $model->id_cellar]);
        $model->delete();
        return $this->redirect(['index']);
    }

    /**
     * Search PDV by ID
     * @param $id
     * @return Configpdv|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = Configpdv::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
}
