<?php

namespace app\controllers;

use app\models\Configpdv;
use app\models\InventoryType;
use app\models\Param;
use app\models\Planning;
use app\models\Requisition;
use app\models\User;
use app\models\Usrpdv;
use Yii;
use yii\helpers\HtmlPurifier;
use yii\web\Controller;
use yii\helpers\Json;
use yii\helpers\ArrayHelper;
use app\models\Generic;


/**
 * Class GenericController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller with general methods used in several modules
 * @package app\controllers
 */
class GenericController extends Controller
{
    /**
     * List of Cellar of BD SIESA
     * @param null $q
     * @param null $id
     * @return mixed
     */
    public function actionGetCellar($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los
            $model = new Configpdv();
            $cellars = $model->getCellar($q);
            if(count($cellars)>0){
                foreach ($cellars as $cellar) {
                    if(!empty($cellar['descripcion'])){
                        $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];
                    }
                }
            }
        }
        $out['results'] = $data;
        return $out;
    }

    /**
     * View multilocation of pdv
     * @return mixed
     */
    public function actionGetMultilocation() {
        $id = Yii::$app->request->get('id');
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $data = 0;

        if (!is_null($id)) {
            # Buscamos el cliente en todos los
            $model = new Configpdv();
            $cellar = $model->getMultilocation($id);

            if(!empty($cellar['multilocation'])){
                $data = $cellar['multilocation'];
            }
        }
        $out['multilocation'] = $data;
        return $out;
    }

    /**
     * Get locations and lotes of reference
     * @return mixed
     */
    public function actionGetMultilocationReference() {
        $cellar = Yii::$app->request->get('cellar');
        $reference = Yii::$app->request->get('reference');
        $description = Yii::$app->request->get('description');
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $data['lotes']  = [];
        $data['locations']  = [];

        if (!is_null($cellar) && !is_null($reference) && !is_null($description)) {
            # Buscamos el cliente en todos los
            $model = new Configpdv();
            $lotes = $model->getLotesMultilocationReference($cellar, $reference, $description);
            $locations = $model->getLocationsMultilocationReference($cellar, $reference, $description);

            if(!empty($lotes)){
                $data['lotes'] = $lotes;
            }

            if(!empty($locations)){
                $data['locations'] = $locations;
            }
        }
        $out['data'] = $data;
        return $out;
    }


    /**
     * List of Operation Centers of BD SIESA
     * @param null $q
     * @param null $id
     * @return mixed
     */
    public function actionGetlistco($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los
            $model = new Configpdv();
            $cellars = $model->getListco($q);
            if(count($cellars)>0){
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];
                }
            }
        }
        $out['results'] = $data;
        return $out;
    }

    /**
     * List of Companies of BD SIESA
     * @param null $q
     * @param null $id
     * @return mixed
     */
    public function actionGetCia($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los
            $model = new Configpdv();
            $cellars = $model->getCia($q);
            if(count($cellars)>0){
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];
                }
            }
        }
        $out['results'] = $data;
        return $out;
    }

    /**
     * PDV AJAX Response (Points of Sales) of BD SIESA
     */
    public function actionGetPdvs() {
	\Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $out = [];
        $params = $_POST['depdrop_all_params'];
        $selected = '';

        if (isset($params['configpdv-id_cia']) || isset($params['usrpdv-id_cia'])) {
            if(isset($params['configpdv-id_cia']) == true){
                $siesa = true;
                $cia = $params['configpdv-id_cia'];
            }else if(isset($params['usrpdv-id_cia']) == true){
                $siesa = false;
                $cia = $params['usrpdv-id_cia'];
                $id = $params['usrpdv-id'];

                if(empty($id) == false){
                    $selected = explode('-', Usrpdv::find()->where(['id' => $id])->one()->code_pdv);
                }
            }

            if ($cia != null) {
                if($siesa == true){
                    $model = new Configpdv();
                    $pdvs = $model->getPdvs($cia);
                    $out[0] = [
                        'id' => '01CEDI', 'name' => '01CEDI - GENÉRICO CEDI'
                    ];
                    $out[1] = [
                        'id' => '01PLANTA', 'name' => '01PLANTA - GENÉRICO PLANTA 1'
                    ];
                    $out[2] = [
                        'id' => '02PLANTA', 'name' => '02PLANTA - GENÉRICO PLANTA 2'
                    ];
                }else if($siesa == false){
                    $pdvs = Configpdv::find()->where('id_cia = '.$cia)->groupBy('name_pdv')->all();
                }

                if(count($pdvs)>0){
                    foreach ($pdvs as $value) {
                        $out[] = ['id' => isset($value['nit']) ? $value['nit'] : $value['code_pdv'],
                            'name' => strtoupper(isset($value['razon_social']) ? $value['razon_social'] : $value['code_pdv'].' - '.$value['name_pdv'])];
                    }
                }
                return ['output'=>$out, 'selected'=>$selected];
            }
        }
        return ['output'=>'', 'selected'=>''];
    }

    /**
     * AJAX response of list of CO groups related to a Company of BD SIESA
     */
    public function actionGetGroups() {
        $out = [];
        $params = $_POST['depdrop_all_params'];
        $params['type'] = '';
        $cia = '';
        $siesa = false;
        $selected = '';
        if (empty($params['configpdv-id_cia']) == false || empty($params['usrpdv-code_pdv']) == false || empty($params['inventorytype-id_cia']) == false) {
            if(empty($params['configpdv-id_cia']) == false){
                $siesa = true;
                $params['type'] = 'configpdv';
                $cia = $params['configpdv-id_cia'];
            }else if(empty($params['usrpdv-id_cia']) == false){
                $params['type'] = 'usrpdv';
                $cia = $params['usrpdv-id_cia'];
                $code_pdv = $params['usrpdv-code_pdv'];
                $id = $params['usrpdv-id'];

                if(empty($id) == false){
                    $selected = explode('-', Usrpdv::find()->where(['id' => $id])->one()->id_group_co);
                }
            }else if(empty($params['inventorytype-id_cia']) == false){
                $params['type'] = 'inventorytype';
                $cia = $params['inventorytype-id_cia'];
                $user_co = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('id_group_co')->one();

                if(empty($user_co) == false){
                    $user_co = explode("-", $user_co['id_group_co']);
                }
            }

            if (empty($cia) == false || empty($code_pdv) == false) {
                $groups = [];
                if($siesa == true && $params['type'] == 'configpdv'){
                    $model = new Configpdv();
                    $groups = $model->getGroupsCo($cia);
                    $out[0] = ['id' => '001', 'name' => '001 - GENÉRICO'];
                }else if($params['type'] == 'inventorytype') {
                    if(empty($user_co) == false){
                        $groups = Configpdv::find()->where(['id_cia' => $cia])->andWhere(['in', 'id_group_co', $user_co])->groupBy('name_group_co')->all();
                    }
                }else if($params['type'] == 'usrpdv') {
                    if(is_array($code_pdv) == true){
                        $groups = Configpdv::find()->where(['id_cia' => $cia])->andWhere(['in', 'code_pdv', $code_pdv])->groupBy('name_group_co')->all();
                    }else{
                        $groups = Configpdv::find()->where(['id_cia' => $cia, 'code_pdv' => $code_pdv])->groupBy('name_group_co')->all();
                    }
                }

                if(count($groups)>0){
                    foreach ($groups as $value) {
                        $out[] = ['id' => isset($value['codigo']) ? $value['codigo'] : $value['id_group_co'],
                            'name' => strtoupper(isset($value['descripcion']) ? $value['descripcion'] : $value['id_group_co'].' - '.$value['name_group_co'])];
                    }
                }else if(empty($out) == true){
                    echo Json::encode(['output'=>'', 'selected'=>'']);
                    return;
                }

                echo Json::encode(['output'=>$out, 'selected'=>$selected]);
                return;
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
        return;
    }

    /**
     * AJAX response of list of CO groups related to a Company of BD SIESA
     */
    public function actionGetBusinessUnit() {
        $out = [];
        $model = new Planning();
        $units = $model->getBusinessUnit(1);

        if(count($units)>0){
            foreach ($units as $value) {
                $out[] = ['id' => isset($value['codigo']) ? $value['codigo'] : $value['id_group_co'],
                    'name' => strtoupper(isset($value['descripcion']) ? $value['descripcion'] : $value['id_group_co'].' - '.$value['name_group_co'])];
            }
        }
        return ArrayHelper::map($out, 'id', 'name');
    }

    /**
     * Returns list of CO related to a Company and a CO Group
     */
    public function actionGetListCoGroup() {
        $out = [];
        $params = $_POST['depdrop_all_params'];
        $siesa = false;
        $selected = '';

        if ((isset($params['configpdv-id_group_co']) && isset($params['configpdv-id_cia'])) || ((isset($params['usrpdv-id_group_co']) == true && isset($params['usrpdv-id_cia']) == true && isset($params['usrpdv-code_pdv']) == true) && isset($params['usrpdv-id_cia']))) {
            if(empty($params['configpdv-id_group_co']) == false){
                $siesa = true;
                $group = $params['configpdv-id_group_co'];
                $cia = $params['configpdv-id_cia'];
            }else if(empty($params['usrpdv-code_pdv']) == false && empty($params['usrpdv-id_group_co']) == false && empty($params['usrpdv-id_cia']) == false){
                $cia = $params['usrpdv-id_cia'];
                $code_pdv = $params['usrpdv-code_pdv'];
                $group = $params['usrpdv-id_group_co'];

                $id = $params['usrpdv-id'];

                if(empty($id) == false){
                    $selected = explode('-', Usrpdv::find()->where(['id' => $id])->one()->id_co);
                }
            }

            if (isset($group) == true || isset($cia) == true || isset($code_pdv) == true) {
                $co = [];

                if($group == '001'){
                    $out[0] = ['id' => '001', 'name' => '001 - GENÉRICO'];
                    echo Json::encode(['output'=>$out, 'selected'=>'']);
                    return;
                }if($siesa == true){
                    $model = new Configpdv();
                    $co = $model->getListCoByGroup($group, $cia);
                }else if(empty($group) == false && empty($cia) == false && empty($code_pdv) == false){
                    if(is_array($code_pdv) == true){
                        $co = Configpdv::find()->where(['id_cia' => $cia, 'id_group_co' => $group])->andWhere(['in', 'code_pdv', $code_pdv])->groupBy('name_co')->all();
                    }else{
                        $co = Configpdv::find()->where(['id_cia' => $cia, 'id_group_co' => $group, 'code_pdv' => $code_pdv])->groupBy('name_co')->all();
                    }
                }

                if(count($co)>0){
                    foreach ($co as $value) {
                        $out[] = ['id' => isset($value['codigo']) ? $value['codigo'] : $value['id_co'],
                            'name' => strtoupper(isset($value['descripcion']) ? $value['descripcion'] : $value['id_co'].' - '.$value['name_co'])];
                    }
                }else{
                    echo Json::encode(['output'=>'', 'selected'=>'']);
                    return;
                }
                echo Json::encode(['output'=>$out, 'selected'=>$selected]);
                return;
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
        return;
    }

    /**
     * List of companies
     * @return array
     */
    public function getCiasDrop()
    {
        $cias = Configpdv::find()->groupBy('name_cia')->all();
        foreach ($cias as $value) {
            $value->name_cia = $value->id_cia . ' - ' . $value->name_cia;
        }
        return ArrayHelper::map($cias, 'id_cia', 'name_cia');
    }

    /**
     * List of companies
     * @return array
     */
    public function actionGetCiasDrop()
    {
        return $this->getCiasDrop();
    }

    /**
     * List of Users
     * @return array
     */
    public function actionGetUsersDrop(){
        $users = User::find()->where(['status'=>10])->orderBy('username')->all();
        foreach ($users as $value) {
            $value->username = strtoupper($value->username);
        }

        return ArrayHelper::map($users, 'id', 'username');
    }

    /**
     * List of Inventory Types
     * @return array
     */
    public function actionGetInventoryTypeDrop(){
        $types = InventoryType::find()->all();
        foreach ($types as $value) {
            $value->name = ucwords($value->name);
        }

        return ArrayHelper::map($types, 'id', 'name');
    }

    /**
     * List of Estados
     * @return array
     */
    public function actionGetInventoryStatusDrop(){
        return json_decode(Param::findOne(['key' => 'status_inventory'])->value, true);
    }

    /**
     * Roles of the current User
     * @param $rol
     * @return bool
     */
    public function actionGetRol($rol){
        $roles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->identity->getId());
        return array_key_exists ($rol, $roles);
    }

    /**
     * Translation months of Calendar year of the Planning module
     * @return array
     */
    public function actionGetMonths(){
        $months = [
            '01' => \Yii::t('app', 'January'),
            '02' => \Yii::t('app', 'February'),
            '03' => \Yii::t('app', 'March'),
            '04' => \Yii::t('app', 'April'),
            '05' => \Yii::t('app', 'May'),
            '06' => \Yii::t('app', 'June'),
            '07' => \Yii::t('app', 'July'),
            '08' => \Yii::t('app', 'August'),
            '09' => \Yii::t('app', 'September'),
            '10' => \Yii::t('app', 'October'),
            '11' => \Yii::t('app', 'November'),
            '12' => \Yii::t('app', 'December')
        ];
        return $months;
    }

    /**
     * Retorna la informacion del punto de venta configurado
     *
     * @param string $pdv            Nit del punto de venta
     * @param string $business_name Nombre del punto de venta
     *
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetInfoPdv(){
        $params = $_POST;
        if (isset($params['nit'])){
            $infoPdv = Configpdv::find()
                ->where(['code_pdv' => $params['nit']])
                ->groupBy('code_pdv')
                ->asArray()->one();
            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return $infoPdv;
        }
    }

    /**
     * Retorna la informacion del cliente
     *
     * @param string $cliente Nit del cliente
     *
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getClient($cliente = null)
    {
        $data = [];

        $user_pdv = ArrayHelper::map(Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('name_pdv')->all(), 'code_pdv', 'code_pdv');

        $clients = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
            ->where(['in', 'code_pdv', $user_pdv])
            ->groupBy('code_pdv')
            ->asArray()->all();

        if (count($clients) > 0) {
            foreach ($clients as $client) {
                $data[$client['code_pdv']] = $client['code_pdv'] . ' - ' . $client['name_pdv'];
            }
        }

        return $data;
    }

    /**
     * Retorna el listado de productos
     *
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetProducts() {
        if (isset($_POST['nit']) && !empty($_POST['nit'])) {
            $ancho = HtmlPurifier::process($_POST['ancho']);
            $pedido = HtmlPurifier::process($_POST['pedido']);
            $cia = HtmlPurifier::process($_POST['cia']);
            $nit = HtmlPurifier::process($_POST['nit']);
            $model = new Requisition();
            $listProduct = $model->getProducts($nit, $cia);

            if($_POST['type'] == 'requisition'){
                return $this->renderAjax('/requisition/_formProducts', [
                    'model' => $model,
                    'listProduct' => $listProduct,
                    'ancho' => $ancho,
                    'pedido' => $pedido,
                ]);
            }else{
                return $this->renderAjax('/in-out/_formProducts', [
                    'model' => $model,
                    'listProduct' => $listProduct,
                    'ancho' => $ancho,
                    'pedido' => $pedido,
                ]);
            }
        }
    }
}
