<?php

namespace app\controllers;

use app\models\CalculatorMainItems;
use app\models\Configpdv;
use app\models\Generic;
use app\models\InOut;
use app\models\InOutDetail;
use app\models\Inventory;
use app\models\InventorySearch;
use app\models\InventoryType;
use app\models\InventoryTypeReferences;
use app\models\Param;
use app\models\Planning;
use app\models\UserSiesa;
use app\models\Usrpdv;
use phpDocumentor\Reflection\Types\Object_;
use Yii;
use yii\data\ActiveDataProvider;
use yii\db\Query;
use yii\helpers\ArrayHelper;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\InventoryDetailSearch;
use app\models\InventoryDetail;


/**
 * Class InventoryController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller in charge of the inventory module management
 * @package app\controllers
 */
class InventoryController extends Controller
{
    public $select_inventory_details = [
        'inventory_detail.id as id_inventory_detail',
        'inventory_detail.counting_1',
        'inventory_detail.counting_2',
        'inventory_detail.counting_2',
        'inventory_detail.assemble_1',
        'inventory_detail.assemble_2',
        'inventory_detail.notes',
        'inventory_detail.difference_amount',
        'inventory_detail.difference_price',
        'inventory_detail.reason',
        'inventory_detail.status',
        'inventory_detail.id_cellar',
        'inventory_detail.cellar',
        'inventory_detail.current_inventory',
        'inventory_detail.unit_price',
        'inventory_detail.is_created',
        'inventory_type_references.*'
    ];

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * List of Inventories
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new InventorySearch();
        // Grupos de Centro de Operaciones habilitados al Usuario actual
        $groups_user = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('id_group_co')->one();
        if (empty($groups_user) == false) {
            $groups_user = explode('-', $groups_user['id_group_co']);
        }

        // Puntos de Ventas habilitados al Usuario actual
        $pdv_user_code = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('code_pdv')->one();
        if (!empty($pdv_user_code)) {
            $pdv_user_code = explode('-', $pdv_user_code['code_pdv']);
            // Tipos de Inventarios relacionados a los Grupos de CO habiltiados al Usuario actual
            $inventory_type = ArrayHelper::map(InventoryType::find()
                ->where(['in', 'id_group_co', $groups_user])->all(), 'id', 'name');
            // Puntos de Ventas relacionados

            // a los Grupos de CO habiltiados al Usuario actual
            $pdv_list = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
                ->where(['in', 'code_pdv', $pdv_user_code])
                ->groupBy('code_pdv')
                ->asArray()->all();
            $pdv_list = ArrayHelper::map($pdv_list, 'code_pdv', 'name_pdv');
            // Planeación del día para los puntos de ventas habilitados al Usuario
            $planning = Planning::find()
                ->select(['planning.*', 'CONCAT(config_pdv.name_pdv," - ",config_pdv.name_cellar_in," ( ", planning.date, " )") AS name_pdv'])
                ->innerJoin('config_pdv', 'config_pdv.code_pdv = planning.code_pdv')
                ->where(['in', 'planning.code_pdv', $pdv_user_code])
                ->andWhere(['in', 'date', [date('Y-m-d'), date('Y-m-d',strtotime('-1 day'))]])
                ->asArray()->all();
        }else{
            $pdv_user_code = null;
            $inventory_type = null;
            $planning = null;
            $pdv_list = null;
        }

        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $queryParams['InventorySearch']['code_pdv'] = $pdv_user_code;
        $dataProvider = $searchModel->search($queryParams);

        // Visualización de los Inventarios habilitados
        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'inventory_type_list' => $inventory_type,
            'planning' => $planning,
            'pdv_list' => $pdv_list
        ]);
    }

    /**
     * Viewing Complete Information of an Inventory
     * @param $id
     * @return string|\yii\web\Response
     * @throws NotFoundHttpException
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionView($id)
    {
        // Rol administrador
        $admin = Yii::$app->runAction('generic/get-rol', ['rol' => 'Administrador']);
        // Rol administrador Punto de Venta
        $admin_pdv = Yii::$app->runAction('generic/get-rol', ['rol' => 'Administrador de PVD']);
        // Rol Planeador
        $planeador = Yii::$app->runAction('generic/get-rol', ['rol' => 'Planeador de Inventarios']);

        // Puntos de Ventas habilitados al Usuario actual
        $pdv_user_code = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('code_pdv')->one();

        if (empty($pdv_user_code) == false) {
            $pdv_user_code = explode('-', $pdv_user_code['code_pdv']);
        }

        // Puntos de Ventas relacionados
        // a los Grupos de CO habiltiados al Usuario actual
        $pdv_list = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
            ->where(['in', 'code_pdv', $pdv_user_code])
            ->groupBy('code_pdv')
            ->asArray()->all();
        $pdv_list = ArrayHelper::map($pdv_list, 'code_pdv', 'name_pdv');

        // Rol administrador de PDV
        $id_user =  Yii::$app->user->identity->getId();
        $searchModel = new InventoryDetailSearch();
        // Inventario
        $model = $this->findModel($id);
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $queryParams['InventoryDetailSearch']['id_inventory'] = $id;
        $dataProvider = $searchModel->search($queryParams);
        $reasons = json_decode(Param::findOne(['key' => 'reasons_inventory'])->value, true);

        if(empty($model) == false){
            if(array_key_exists($model->code_pdv, $pdv_list) == FALSE){
                return $this->redirect(['index']);
            }

            // Bodega asociadas a los PDV del Usuario actual
            $cellar = ArrayHelper::map(Configpdv::find()->where(['name_pdv' => $model->name_pdv])->asArray()->all(), 'id_cellar', 'id_cellar');

            //Yii::$app->session->setFlash('success', 'Validación realizada correctamente.');
            //Sesiones
            $msn_success = Yii::$app->session->get('msn-success');
            $msn_error = Yii::$app->session->get('msn-error');

            if(empty($msn_success) == false){
                Yii::$app->session->setFlash('success', $msn_success);
                Yii::$app->session->remove('msn-success');
            }

            if(empty($msn_error) == false){
                Yii::$app->session->setFlash('error', $msn_error);
                Yii::$app->session->remove('msn-error');
            }

            if(empty($cellar) == false){
                $subQuery = InventoryDetail::find()
                    ->select('id_reference')
                    ->andwhere(['inventory_detail.is_created' => '1', 'inventory_detail.id_inventory' => $id])
                    ->all();
                if($admin == true || $planeador == true || ($admin_pdv == true && $model->created_by == \Yii::$app->user->getIdentity()->username)){
                    // Detalles del Inventario
                    $details = (new Query())
                        ->from('inventory_type_references')
                        ->select($this->select_inventory_details)
                        ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'1\'')
                        ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                        ->andwhere(['inventory_type_references.id_inventory_type' => $model->id_inventory_type, 'inventory_type_references.selected' => 'on']);
                    $details_all = $details->all();
                }else{
                    // Detalles del Inventario
                    $details = (new Query())
                        ->from('inventory_type_references')
                        ->select($this->select_inventory_details)
                        ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_usuario = \''.$id_user.'\' AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'0\'')
                        ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                        ->andwhere(['inventory_type_references.id_inventory_type' => $model->id_inventory_type, 'inventory_type_references.selected' => 'on']);
                    $details_all = $details->all();
                }

                if(empty($details_all) == false){
                    // Creación de referencias
                    foreach ($details->orderBy('inventory_type_references.description')->all() as $value){
                        if(empty($value['id_inventory_detail']) == true && $model->created_by == \Yii::$app->user->identity->username){
                            $model_detail = new InventoryDetail();
                            $model_detail->status = InventoryDetail::STATUS_IN_PROCESS;
                            $model_detail->id_inventory = $id;
                            $model_detail->id_reference = $value['id'];
                            $model_detail->id_usuario = $id_user;
                            $model_detail->counting_1 = 0;
                            $model_detail->is_created = '1';
                            $model_detail->save();
                        }
                    }
                }

                $details = new ActiveDataProvider([
                    'query' => $details,
                ]);

                // Visulización del Inventario y detalles del mismo
                return $this->render('view', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
                    'model' => $this->findModel($id),
                    'details' => $details,
                    'admin' => $admin,
                    'reasons' => $reasons
                ]);
            }
        }
        return $this->redirect(['index']);
    }

    /**
     * Creation of new Inventory
     * @param $type
     * @param $pdv
     * @return \yii\web\Response
     */
    public function actionCreate($type = null, $pdv = null, $planning = null)
    {
        $model = new Inventory();
        if((empty($type) == false && empty($pdv) == false) || empty($planning) == false){
            $business_unit = '';

            // Puntos de Ventas
            if(empty($pdv) == false){
                $pdv = Configpdv::findOne(['code_pdv' => $pdv]);
            }else {
                // Planificación día actual
                $planning = Planning::findOne(['id' => $planning]);

                if(empty($planning) == false){
                    // Unidades de negocio
                    $business_units = Yii::$app->runAction('generic/get-business-unit', []);

                    $pdv = Configpdv::findOne(['code_pdv' => $planning->code_pdv]);
                    $type = $planning->id_inventory_type;

                    if(empty($business_units) == false){
                        foreach ($business_units as $key => $value){
                            if($key == $planning->id_business_unit){
                                $business_unit = $value;
                            }
                        }
                    }
                }else{
                    return $this->redirect(['index']);
                }
            }

            $cellars = ArrayHelper::map(Configpdv::find()->where(['code_pdv' => $planning->code_pdv])->all(),'id_cellar_in', 'id_cellar_in');
            $refences_code = [];
            $currencies_price = [];
            //Referencias relacionadas al Inventario
            $references = InventoryTypeReferences::find()
                ->select($this->select_inventory_details)
                ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id')
                ->where(['id_inventory_type' => $type])
                ->andWhere(['selected' => 'on'])
                ->asArray()->all();

            if(empty($references) == true){
                Yii::$app->session->setFlash('error', 'No Existen referencias programadas en las Bodegas relaciondas al Punto de Venta.');
                return $this->redirect(['index']);
                return $this->redirect(['index']);
            }

            if(!empty($type) && !empty($pdv)){
                $inventory = Inventory::find()
                    ->where(['code_pdv' => $pdv->code_pdv, 'id_inventory_type' => $type, 'created' => date('Y-m-d')])
                    ->andWhere('code_status != \''.Inventory::STATUS_FINAL.'\'')
                    ->asArray()->all();
                if(!empty($inventory)){
                    Yii::$app->session->setFlash('error', 'Ya existe un Inventario para el Punto de Venta el día de hoy.');
                    return $this->redirect(['index']);
                }
            }

            $pdv = Configpdv::findOne(['code_pdv' => $planning->code_pdv]);

            $estados = json_decode(Param::findOne(['key' => 'status_inventory'])->value, true);
            // Creación del Inventario
            $inventory_model = new Inventory();
            $inventory_model->id_inventory_type = $type;
            $inventory_model->code_pdv = $pdv->code_pdv;
            $inventory_model->name_pdv = $pdv->name_pdv;
            $inventory_model->business_unit = $business_unit;
            $inventory_model->consecutive = $this->getConsecutive();
            $inventory_model->code_status = Inventory::STATUS_PENDIENTE_1;
            $inventory_model->code_name = $estados[Inventory::STATUS_PENDIENTE_1];
            $inventory_model->save();
            $references_without_price = [];
            $references_save = [];

            // Creación de Detalles de Inventarios
            foreach ($references as $detail){
                $refences_code[] = $detail['reference'];
            }

            $refences_code_save = $refences_code;

            if(empty($refences_code) == false){
                $currencies_price = $model->getInfoReferencesInventory($pdv->id_cia, $refences_code, $cellars);
            }

            foreach ($references as $detail){
                $save = false;
                $model_detail = new InventoryDetail();
                $model_detail->status = InventoryDetail::STATUS_IN_PROCESS;
                $model_detail->id_inventory = $inventory_model->id;
                $model_detail->id_reference = $detail['id'];
                $model_detail->id_usuario = Yii::$app->user->identity->getId();
                $model_detail->counting_1 = 0;
                $model_detail->is_created = '1';

                if(empty($currencies_price) == false){
                    foreach ($currencies_price as $price){
                        $error = false;
                        if(strcmp(trim($detail['reference']), trim($price['reference'])) === 0 && in_array($detail['reference'], $refences_code_save)){
                            $id_reference = array_keys($refences_code_save, $detail['reference']);
                            if(isset($price['price'])){
                                $model_detail->unit_price = round((float)$price['price'], 6);
                                if($model_detail->unit_price <= 0){
                                    $error = true;
                                }
                            }else{
                                $error = true;
                            }

                            if(isset($price['current_inventory']) == true){
                                $model_detail->current_inventory = round((float)$price['current_inventory'], 3);
                            }

                            if(isset($price['cellar_name']) == true){
                                $model_detail->cellar = $price['cellar_name'];
                            }else{
                                $error = true;
                            }

                            if(isset($price['cellar_id']) == true){
                                $model_detail->id_cellar = $price['cellar_id'];
                            }else{
                                $error = true;
                            }

                            if($error == false){
                                unset($refences_code_save[$id_reference[0]]);
                                $model_detail->save();
                                $save = true;
                                break;
                            }
                        }
                    }
                    if($save == false){
                        $references_without_price[$detail['id']] = $detail['reference'];
                    }
                }
            }

            $refences_code_save = $references_without_price;

            if(empty($references_without_price) == false){
                $price_references = $model->getLastPriceInventory($references_without_price);

                if(empty($price_references) == false){
                    foreach ($price_references as $price){
                        $error = false;
                        if(array_search($price['reference'], $references_without_price) != false && in_array($price['reference'], $refences_code_save)){
                            $id_reference = array_keys($references_without_price, $price['reference']);
                            $model_detail = new InventoryDetail();
                            $model_detail->status = InventoryDetail::STATUS_IN_PROCESS;
                            $model_detail->id_inventory = $inventory_model->id;
                            $model_detail->id_reference = $id_reference[0];
                            $model_detail->id_usuario = Yii::$app->user->identity->getId();
                            $model_detail->counting_1 = 0;
                            $model_detail->is_created = '1';

                            if(isset($price['price']) == true){
                                $model_detail->unit_price = round((float)$price['price'], 2);
                                if($model_detail->unit_price <= 0){
                                    $error = true;
                                }
                            }else{
                                $error = true;
                            }

                            $model_detail->current_inventory = 0;
                            $model_detail->cellar = $pdv->name_cellar_in;
                            $model_detail->id_cellar = $pdv->id_cellar_in;

                            if($error == false){
                                unset($refences_code_save[$id_reference[0]]);
                                $model_detail->save();
                                continue;
                            }
                        }
                    }
                }
            }

            // Visualización del Inventario y detalles relacionados
            return $this->redirect(['view', 'id' => $inventory_model->id]);
        }

        return $this->redirect(['index']);
    }

    /**
     * Inventory update and details of it
     * @param $id
     * @param null $validate
     * @return string|\yii\web\Response
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionUpdate($id, $type = null)
    {
        $inventory_model = Inventory::findOne(['id' => $id]);

        if(empty($inventory_model) == false && empty($type) == false) {
            $users_siesa = ArrayHelper::map(UserSiesa::find()->where(['status' => '10'])->all(), 'id', 'name');
            // Rol administrador
            $admin = Yii::$app->runAction('generic/get-rol', ['rol' => 'Administrador']);
            // Rol administrador Punto de Venta
            $admin_pdv = Yii::$app->runAction('generic/get-rol', ['rol' => 'Administrador de PVD']);

            $id_user = Yii::$app->user->identity->getId();
            // Lista de PDV relacionados al Usuario actual
            $pdv_list = ArrayHelper::map(Usrpdv::find()->where(['id_user' => $id_user])->groupBy('name_pdv')->all(), 'name_pdv', 'name_pdv');
            // Lista de Bodegas relacionadas a los PDV del Usuario actual
            $cellar = ArrayHelper::map(Configpdv::find()->where(['name_pdv' => $inventory_model->name_pdv])->asArray()->all(), 'id_cellar', 'id_cellar');
            $roles = \Yii::$app->authManager->getRolesByUser(Yii::$app->user->identity->getId());
            $planeador = array_key_exists ('Planeador de Inventarios', $roles);
            $subQuery = InventoryDetail::find()
                ->select('id_reference')
                ->andwhere(['inventory_detail.is_created' => '1', 'inventory_detail.id_inventory' => $inventory_model->id, 'inventory_detail.status' => InventoryDetail::STATUS_IN_PROCESS])
                ->all();
            // Puntos de Ventas habilitados al Usuario actual
            $pdv_user_code = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('code_pdv')->one();
            if (empty($pdv_user_code) == false) {
                $pdv_user_code = explode('-', $pdv_user_code['code_pdv']);
            }

            // Puntos de Ventas relacionados
            // a los Grupos de CO habiltiados al Usuario actual
            $pdv_list = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
                ->where(['in', 'code_pdv', $pdv_user_code])
                ->groupBy('code_pdv')
                ->asArray()->all();
            $pdv_list = ArrayHelper::map($pdv_list, 'code_pdv', 'name_pdv');

            if(array_key_exists($inventory_model->code_pdv, $pdv_list) == FALSE){
                return $this->redirect(['index']);
            }

            $multilocation = Configpdv::findOne(['code_pdv' => $inventory_model->code_pdv]);

            if(empty($multilocation)){
                $multilocation->multi_ubicacion = 0;
            }

            if($planeador == true){
                //Referencias relacionda al inventario
                $references = InventoryTypeReferences::find()
                    ->select($this->select_inventory_details)
                    ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'1\'')
                    ->andwhere(['inventory_type_references.id_inventory_type' => $inventory_model->id_inventory_type, 'inventory_type_references.selected' => 'on'])
                    ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                    ->asArray()->all();
            }else{
                switch ($type){
                    case 'count':
                        $number_count = $inventory_model->code_status;
                        //Referencias relacionda al inventario
                        $references = InventoryTypeReferences::find()
                            ->select($this->select_inventory_details)
                            ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_usuario = \''.$id_user.'\' AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'0\'')
                            ->andwhere(['inventory_type_references.id_inventory_type' => $inventory_model->id_inventory_type, 'inventory_type_references.selected' => 'on'])
                            ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                            ->asArray()->orderBy('inventory_type_references.description')->all();
                        break;
                    case 'validate':
                        if($admin == true || ($admin_pdv == true && $inventory_model->created_by == \Yii::$app->user->getIdentity()->username)){
                            //Referencias relacionda al inventario
                            $references = InventoryTypeReferences::find()
                                ->select($this->select_inventory_details)
                                ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'1\'')
                                ->andwhere(['inventory_type_references.id_inventory_type' => $inventory_model->id_inventory_type, 'inventory_type_references.selected' => 'on'])
                                ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                                ->asArray()->orderBy('inventory_type_references.description')->all();
                        }
                        break;
                    default:
                        if($admin == true || $planeador == true){
                            //Referencias relacionda al inventario
                            $references = InventoryTypeReferences::find()
                                ->select($this->select_inventory_details)
                                ->leftJoin('inventory_detail', 'inventory_detail.id_reference=inventory_type_references.id AND inventory_detail.id_inventory = \''.$id.'\' AND inventory_detail.is_created = \'1\'')
                                ->andwhere(['inventory_type_references.id_inventory_type' => $inventory_model->id_inventory_type, 'inventory_type_references.selected' => 'on'])
                                ->andwhere(['in', 'inventory_type_references.id', ArrayHelper::map($subQuery, 'id_reference', 'id_reference')])
                                ->asArray()->orderBy('inventory_type_references.description')->all();
                        }
                        break;
                }
            }

            if(empty($references) == false) {
                $inventoryDetail = InventoryDetail::find()
                    ->where(['id_inventory' => $inventory_model->id])
                    ->andWhere(['id_usuario' => $id_user])->asArray()->all();
                $post = Yii::$app->request->post();

                if (empty($post) == false) {
                    if (($admin == true || ($admin_pdv == true && $inventory_model->created_by == \Yii::$app->user->identity->username)) && $post['type'] == 'validate') {
                        if($inventory_model->code_status == Inventory::STATUS_PENDIENTE_1){
                            $inventory_exits = InventoryDetail::find()->where(['id_inventory' => $inventory_model->id, 'is_created' => '0'])->andWhere('counting_1 IS NOT NULL')->all();
                        }else if($inventory_model->code_status == Inventory::STATUS_PENDIENTE_2){
                            $inventory_exits = InventoryDetail::find()->where(['id_inventory' => $inventory_model->id, 'is_created' => '0'])->andWhere('counting_2 IS NOT NULL')->all();
                        }else if($inventory_model->code_status == Inventory::STATUS_DESCUADRE){
                            $inventory_exits = true;
                        }

                        if(empty($inventory_exits) == true){
                            Yii::$app->session->remove('msn-success');
                            Yii::$app->session->set('msn-error', 'No existen conteos para validar.');
                            return $this->redirect(['view', 'id' => $inventory_model->id]);
                        }else{
                            //Realizar validación
                            $inventory_details = $post['Inventory_details'];
                            return $this->validateInventoryDetails($inventory_details, $inventory_model);
                        }
                    } else if($type == 'count'){
                        //Almacenamiento de detalles de Invetario
                        $inventory_details = $post['Inventory_details'];
                        $this->saveInventoryDetails($inventory_details, $id, $inventory_model->code_status);

                        if ($inventory_model->code_status === Inventory::STATUS_FINAL) {
                            return $this->redirect(['index']);
                        } else {
                            if($post['type_request'] == 'ajax'){
                                return true;
                            }else{
                                return $this->redirect(['view', 'id' => $id]);
                            }
                        }
                    }
                }
                // Referencias para la Calculadora de tipo simple y compuest
                $products_simple = ArrayHelper::map(CalculatorMainItems::find()->where(['code_type' => CalculatorMainItems::TYPE_SIMPLE])->asArray()->all(), 'id', 'name');
                $products_composed = ArrayHelper::map(CalculatorMainItems::find()->where(['code_type' => CalculatorMainItems::TYPE_COMPOSED])->asArray()->all(), 'id', 'name');

                // Vista de actualización
                return $this->render('update', [
                    'model' => $inventory_model,
                    'multilocation' => $multilocation,
                    'references' => $references,
                    'inventoryDetail' => $inventoryDetail,
                    'pdv_list' => $pdv_list,
                    'products_simple' => $products_simple,
                    'products_composed' => $products_composed,
                    'type' => $type,
                    'users_siesa' => $users_siesa
                ]);
            }else{
                return $this->redirect(['index']);
            }
        }else{
            return $this->redirect(['index']);
        }
    }

    /**
     * Elimination of Inventory and Associated Details
     * @param $id
     * @return \yii\web\Response
     * @throws \Throwable
     */
    public function actionDelete($id)
    {
        try{
            InventoryDetail::deleteAll('id_inventory = '.$id);
            $this->findModel($id)->delete();
            return $this->redirect(['index']);
        }catch (\Exception $e){
            return $this->redirect(['index']);
        }
    }

    /**
     * Inventory Search
     * @param $id
     * @return Inventory|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = Inventory::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    /**
     * Generation of Consecutive for new Inventories
     * @return string
     */
    public function getConsecutive(){
        $inventory = Inventory::find()
            ->where("consecutive LIKE '%".date('Y')."%'")
            ->orderBy(['id' => SORT_DESC])
            ->limit('1')
            ->asArray()->all();

        if(empty($inventory) == false) {
            $consecitive = explode('-', $inventory[0]['consecutive']);
            if(empty($consecitive[0]) == false){
                return ($consecitive[0] + 1) .'-'. date('Y');
            }
        }
        return '1-'.date('Y');
    }

    /**
     * Storage of Inventory Details
     * @param $inventory_details
     * @param $id_inventory
     */
    public function saveInventoryDetails($inventory_details, $id_inventory, $code_status)
    {

        foreach ($inventory_details as $detail) {
            //Creación de Detalles relacionados a un Inventario
            $model_detail = InventoryDetail::findOne(['id_inventory' => $id_inventory, 'id_reference' => $detail['id_reference'], 'id_usuario' => Yii::$app->user->identity->getId(), 'is_created' => '0']);

            if (empty($model_detail) == true) {
                $model_detail = new InventoryDetail();
                $model_detail->status = InventoryDetail::STATUS_IN_PROCESS;
                if ($code_status == Inventory::STATUS_PENDIENTE_2) {
                    $model_detail->counting_1 = 0;
                    $model_detail->assemble_1 = 0;
                }
            }
            $model_detail->id_inventory = $id_inventory;
            $model_detail->id_reference = $detail['id_reference'];
            $model_detail->id_usuario = Yii::$app->user->identity->getId();
            $model_detail->is_created = '0';

            // Conteo 1
            if (isset($detail['counting_1']) == true) {
                $model_detail->counting_1 = $detail['counting_1'];
            }

            // Ensamble 1
            if (isset($detail['assemble_1']) == true) {
                $model_detail->assemble_1 = $detail['assemble_1'];
            }

            // Conteo
            if (isset($detail['counting_2']) == true) {
                $model_detail->counting_2 = $detail['counting_2'];
            }

            // Ensamble 2
            if (isset($detail['assemble_2']) == true) {
                $model_detail->assemble_2 = $detail['assemble_2'];
            }

            // Diferencia en cantidad
            if (isset($detail['difference_amount']) == true) {
                $model_detail->difference_amount = $detail['difference_amount'];
            }

            // Diferencia en precio
            if (isset($detail['difference_price']) == true) {
                $model_detail->difference_price = $detail['difference_price'];
            }

            // Razón descudre
            if (empty($detail['reason']) == false) {
                $model_detail->reason = $detail['reason'];
            }

            // Motivo descuadre
            if (empty($detail['notes']) == false) {
                $model_detail->notes = $detail['notes'];
            }

            $model_detail->save();
            $this->calculateSumCount($id_inventory, $detail['id_reference']);
        }
    }

    /**
     * Calculation of total sums for Inventory Counts by reference
     * @param $id_inventory
     * @param $id_reference
     * @return bool
     */
    public function calculateSumCount($id_inventory, $id_reference){
        if(empty($id_inventory) == false && empty($id_reference) == false){
            // Suma total de conteo 1 de referencia de un Inventario
            $counting_1 = InventoryDetail::find()->where(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '0'])->asArray()->sum('counting_1');
            // Suma total de conteo 2 de referencia de un Inventario
            $counting_2 = InventoryDetail::find()->where(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '0'])->asArray()->sum('counting_2');
            // Suma total de ensamble 1 de referencia de un Inventario
            $assemble_1 = InventoryDetail::find()->where(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '0'])->asArray()->sum('assemble_1');
            // Suma total de ensamble 2 de referencia de un Inventario
            $assemble_2 = InventoryDetail::find()->where(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '0'])->asArray()->sum('assemble_2');

            if((isset($counting_1) && $counting_1 >= 0) || empty($assemble_1) == false){
                // Actualización de conteos en Detalle creador del Inventario
                $reference_create = InventoryDetail::findOne(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '1']);
                $reference_create->counting_1 = $counting_1;
                $reference_create->assemble_1 = $assemble_1;
                $reference_create->save();
            }

            if((isset($counting_2) && $counting_2 >= 0) || empty($assemble_2) == false){
                // Actualización de conteos en Detalle creador del Inventario
                $reference_create = InventoryDetail::findOne(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '1']);
                $reference_create->counting_2 = $counting_2;
                $reference_create->assemble_2 = $assemble_2;
                $reference_create->save();
            }
            return true;
        }
    }

    /**
     * Inventory Validation
     * @param $inventory_details
     * @param $inventory_model
     * @return \yii\web\Response
     */
    public function validateInventoryDetails($inventory_details, $inventory_model){
        $estados = json_decode(Param::findOne(['key' => 'status_inventory'])->value, true);
        $finalizado = true;
        $model_generic = new Generic();

        foreach ($inventory_details as $detail) {
            $inventoryDetail = InventoryDetail::findOne(['id_inventory' => $inventory_model->id, 'id_reference' => $detail['id_reference'], 'is_created' => '1']);
            if(empty($inventoryDetail) == true){
                $inventoryDetail = new InventoryDetail();
                $inventoryDetail->status = InventoryDetail::STATUS_IN_PROCESS;
                $inventoryDetail->id_inventory = $inventory_model->id;
                $inventoryDetail->id_reference = $detail['id_reference'];
                $inventoryDetail->id_usuario = Yii::$app->user->identity->getId();
                $inventoryDetail->counting_1 = 0;
                $inventoryDetail->is_created = '1';
                $inventoryDetail->save();
            }
            $status = true;

            //Calculo de diferencias
            $current_inventory = $inventoryDetail->current_inventory;
            $unit_price = $inventoryDetail->unit_price;
            $rango_perdida = $inventoryDetail->inventoryReference->margin_loss;
            $pendiente_cont_2 = false;

            if($inventory_model->code_status == Inventory::STATUS_DESCUADRE){
                // Datos de descuadre
                if(empty($detail['reason']) == false){
                    $inventoryDetail->reason = $detail['reason'];
                }

                if(empty($detail['notes']) == false){
                    $inventoryDetail->notes = $detail['notes'];
                }
            }else{
                if($inventoryDetail->status == InventoryDetail::STATUS_IN_PROCESS){
                    $difference_price = 0;
                    $difference_amount = 0;

                    // Actualización de conteo 1 con diferencias de precio y cantidad
                    if(isset($detail['counting_1']) == true && $detail['counting_1'] != ''){
                        $counting_1 = $detail['counting_1'];
                        if(isset($detail['assemble_1']) == true && $detail['assemble_1'] != ''){
                            $counting_1 += $detail['assemble_1'];
                        }
                        $difference_amount = $counting_1-$current_inventory;
                    }

                    if($inventory_model->code_status == Inventory::STATUS_PENDIENTE_2 && isset($detail['counting_2']) == true && $detail['counting_2'] != ''){
                        $counting_2 = $detail['counting_2'];
                        if(isset($detail['assemble_2']) == true && $detail['assemble_2'] != ''){
                            $counting_2 += $detail['assemble_2'];
                        }
                         $difference_amount = $counting_2-$current_inventory;
                    }

                    // Diferencia en cantidad
                    if($difference_amount != 0){
                        if($rango_perdida != 0){
                            if($rango_perdida < 0){
                                $rango_perdida = $rango_perdida*-1;
                            }

                            if($difference_amount < 0){
                                $difference_amount = $difference_amount*-1;
                            }

                            $rango_perdida_real = ($current_inventory*$rango_perdida)/100;

                            if($difference_amount > $rango_perdida_real){
                                $status = false;
                                $finalizado = false;
                            }
                        }else{
                            $status = false;
                            $finalizado = false;
                        }

                        if($status == true){
                            $inventoryDetail->status = InventoryDetail::STATUS_FINISHED;
                        }
                    }else if($difference_amount == 0){
                        $inventoryDetail->difference_amount = 0;
                        $inventoryDetail->difference_price = 0;
                        $inventoryDetail->status = InventoryDetail::STATUS_FINISHED;
                    }
                    // Actualización
                    InventoryDetail::updateAll(['status' => $inventoryDetail->status, 'difference_amount' => $difference_amount, 'difference_price' => $difference_price], ['id_inventory' => $inventory_model->id, 'id_reference' => $detail['id_reference']]);
                }
            }
            $inventoryDetail->save();
        }

        //Si el Inventario es finalizado
        if($finalizado == false){
            if($inventory_model->code_status == Inventory::STATUS_PENDIENTE_1){
                $inventory_model->code_status = Inventory::STATUS_PENDIENTE_2;
                $inventory_model->code_name = $estados[Inventory::STATUS_PENDIENTE_2];
            }else if($inventory_model->code_status == Inventory::STATUS_PENDIENTE_2){
                $inventory_model->code_status = Inventory::STATUS_DESCUADRE;
                $inventory_model->code_name = $estados[Inventory::STATUS_DESCUADRE];
            }
            Yii::$app->session->remove('msn-error');
            Yii::$app->session->set('msn-success', 'Validación realizada correctamente.');
            $inventory_model->save();
            return $this->redirect(['view', 'id' => $inventory_model->id]);
        }else{
            $response = true;
            // Si quedo con descuadre se asigna el estado "Pendiente de ajuste"
            if($inventory_model->code_status == Inventory::STATUS_DESCUADRE) {
                $users_planeadores = $model_generic->getUserRolesByPdv($inventory_model->code_pdv, 'Planeador de Inventarios');

                if(!empty($users_planeadores)){
                    //Correo de entradas
                    foreach ($users_planeadores as $user){
                        try{
                            Yii::$app->mailer->compose(['html' => '@app/mail/inventoryadjustment-html', 'text' => 'inSiesa-text'], ['user' => $user])
                                ->setFrom([Yii::$app->params['adminEmail'] => Yii::$app->name])
                                ->setTo($user['email'])
                                ->setSubject('Ajuste Pendientes - ' . Yii::$app->name)
                                ->send();
                        }catch (\Exception $exception){
                            echo $exception->getMessage();
                            die();
                            continue;
                        }
                        continue;
                    }
                }
                $response = false;
                $inventory_model->code_status = Inventory::STATUS_PENDIENTE_AJUSTE;
                $inventory_model->code_name = $estados[Inventory::STATUS_PENDIENTE_AJUSTE];
                $inventory_model->save();
            }

            if($response == true){
                $inventory_model->code_status = 'finalizado';
                $inventory_model->code_name = $estados['finalizado'];
                $inventory_model->save();

                Yii::$app->session->set('msn-success', 'Inventario Finalizado con exito.');
                return $this->redirect(['view', 'id' => $inventory_model->id]);
            }
            Yii::$app->session->set('msn-error', 'El Inventario no se Finalizó, aún falta realizar los ajustes.');
            return $this->redirect(['view', 'id' => $inventory_model->id]);
        }
    }

    /**
     * Function to adjust data in the BD SIESA
     * @param $inventory_model
     */
    public function actionSaveAdjust(){
        $post = Yii::$app->request->post();
        $model_generic = new Generic();
        $response = true;
        $user_siesa = new Object_();
        $user_siesa->usuario = [];
        $user_siesa->clave = [];

        if (empty($post) == false) {
            $inventory_model = Inventory::findOne(['id' => $post['Inventory_id']]);

            if (!empty($post['user_siesa_input'])) {
                if ($post['user_siesa_input'] != 0) {
                    $user = UserSiesa::findOne(['id' => $post['user_siesa_input']]);
                    $user_siesa->usuario = trim($user->username);
                    $user_siesa->clave = trim($user->password);
                } else {
                    $user_siesa->usuario = 'jhon_mosquera';
                    $user_siesa->clave = '2359710';
                }
            } else {
                $user_siesa->usuario = 'jhon_mosquera';
                $user_siesa->clave = '2359710';
            }

            if(!empty($post['Inventory_details'])) {
                $inventory_details = $post['Inventory_details'];
                $estados = json_decode(Param::findOne(['key' => 'status_inventory'])->value, true);
                $ids_in = [];
                $ids_out = [];

                // Información de Inventario
                $inventory_info = Inventory::find()
                    ->select('inventory.*, config_pdv.*')
                    ->innerJoin('config_pdv', 'inventory.code_pdv = config_pdv.code_pdv')
                    ->where(['inventory.id' => $inventory_model->id])
                    ->groupBy('inventory.code_pdv')->asArray()->one();

                $bussines = explode('-', $inventory_info['business_unit']);

                $cont_in = 0;
                $cont_out = 0;
                $input_val = false;
                $output_val = false;
                //Se crean las entradas al Inventarios (Diferencia Positiva con respecto al valor actual)
                $input = new InOut();
                $input->id_user_siesa = $post['user_siesa_input'];
                $input->id_co = $inventory_info['id_co'];
                $input->name_co = $inventory_info['name_co'];
                $input->id_cia = $inventory_info['id_cia'];
                $input->name_cia = $inventory_info['name_cia'];
                $input->id_cia = $inventory_info['id_cia'];
                $input->name_cia = $inventory_info['name_cia'];
                if(empty($bussines) == false){
                    $input->id_business_unit = trim($bussines[0]);
                    $input->name_business_unit = trim($bussines[1]);
                }
                $input->nit = $inventory_info['code_pdv'];
                $input->business_name = $inventory_info['name_pdv'];
                $input->id_cellar_in = '';
                $input->name_cellar_in = '';
                $input->id_cellar_out = $inventory_info['id_cellar_in'];
                $input->name_cellar_out = $inventory_info['name_cellar_in'];
                $input->description = 'Entrada - Descuadre Inventario ID =' . $inventory_model->id;
                $input->type = InOut::TYPE_IN;
                $input->type_created = InOut::TYPE_CREATED_SYSTEM;
                $input->status = InOut::STATUS_PENDIENTE;
                $input->date = date('Y-m-d H:i:s');
                $input->save();

                //Se crean las salidas al Inventarios (Diferencia Negativa con respecto al valor actual)
                $output = new InOut();
                $output->id_user_siesa = $post['user_siesa_input'];
                $output->id_co = $inventory_info['id_co'];
                $output->name_co = $inventory_info['name_co'];
                $output->id_cia = $inventory_info['id_cia'];
                $output->name_cia = $inventory_info['name_cia'];
                if(empty($bussines) == false){
                    $output->id_business_unit = trim($bussines[0]);
                    $output->name_business_unit = trim($bussines[1]);
                }
                $output->nit = $inventory_info['code_pdv'];
                $output->business_name = $inventory_info['name_pdv'];
                $output->id_cellar_in = '';
                $output->name_cellar_in = '';
                $output->id_cellar_out = $inventory_info['id_cellar_in'];
                $output->name_cellar_out = $inventory_info['name_cellar_in'];
                $output->description = 'Salida - Descuadre Inventario ID = ' . $inventory_model->id;
                $output->type = InOut::TYPE_OUT;
                $output->type_created = InOut::TYPE_CREATED_SYSTEM;
                $output->status = InOut::STATUS_PENDIENTE;
                $output->date = date('Y-m-d H:i:s');
                $output->save();

                foreach ($inventory_details as $detail){
                    if (!empty($detail['adjustable'])) {
                        $inventoryDetail = InventoryDetail::findOne(['id' => $detail['id_inventory_detail']]);

                        if(!empty($inventoryDetail)){
                            if ($detail['adjustable'] == 'on') {
                                if(!empty($post['id_adj'])){
                                    if(in_array($detail['id_inventory_detail'], $post['id_adj'])) {
                                        foreach ($post['id_adj'] as $key => $value){
                                            $reference = InventoryTypeReferences::findOne(['id' => $inventoryDetail->id_reference]);
                                            $key_value = array_keys($post['qty_adj_reference'], $reference->reference);

                                            if(!empty($key_value) && array_search($key, $key_value) !== false){
                                                $inOutDetail = new InOutDetail();
                                                $inOutDetail->reference = $reference->reference;
                                                $inOutDetail->description = $reference->description;
                                                $inOutDetail->adjusted_cost = $inventoryDetail->unit_price;

                                                if(empty($inventoryDetail->assemble_2) == false){
                                                    $inOutDetail->quantity_count = $inventoryDetail->counting_2+$inventoryDetail->assemble_2;
                                                }else{
                                                    $inOutDetail->quantity_count = $inventoryDetail->counting_2;
                                                }

                                                //Se calcula la diferencia positiva o negativa
                                                if ($inventoryDetail->difference_amount > 0) {
                                                    $inOutDetail->quantity = $post['qty_adj'][$key];
                                                    if(!empty($post['select_lotes'][$key])){
                                                        $inOutDetail->lote = $post['select_lotes'][$key];
                                                    }else{
                                                        $inOutDetail->lote = '';
                                                    }
                                                    $inOutDetail->location = $post['select_locations'][$key];
                                                    $inOutDetail->in_out_id = $input['id'];
                                                    $input_val = true;
                                                    $cont_in++;
                                                }elseif ($inventoryDetail->difference_amount < 0){
                                                    $inOutDetail->quantity = abs($post['qty_adj'][$key]);
                                                    if(!empty($post['select_lotes'][$key])){
                                                        $inOutDetail->lote = $post['select_lotes'][$key];
                                                    }else{
                                                        $inOutDetail->lote = '';
                                                    }
                                                    $inOutDetail->location = $post['select_locations'][$key];
                                                    $inOutDetail->in_out_id = $output->id;
                                                    $output_val = true;
                                                    $cont_out++;
                                                }
                                                $inOutDetail->unid_exist = $inventoryDetail->current_inventory;
                                                $inOutDetail->category = $reference->id_category;
                                                if (!$inOutDetail->save()) {
                                                    Yii::$app->session->set('msn-error', 'Error al guardar las Entradas y Salidas.');
                                                    return $this->redirect(['view', 'id' => $inventory_model->id]);
                                                }
                                                $inventoryDetail->adjustable = 'on';
                                            }
                                        }
                                    }
                                }else{
                                    $reference = InventoryTypeReferences::findOne(['id' => $inventoryDetail->id_reference]);
                                    $inOutDetail = new InOutDetail();
                                    $inOutDetail->reference = $reference->reference;
                                    $inOutDetail->description = $reference->description;
                                    $inOutDetail->adjusted_cost = $inventoryDetail->unit_price;

                                    if(empty($inventoryDetail->assemble_2) == false){
                                        $inOutDetail->quantity_count = $inventoryDetail->counting_2+$inventoryDetail->assemble_2;
                                    }else{
                                        $inOutDetail->quantity_count = $inventoryDetail->counting_2;
                                    }

                                    //Se calcula la diferencia positiva o negativa
                                    if ($inventoryDetail->difference_amount > 0) {
                                        $inOutDetail->quantity = $inventoryDetail->difference_amount;
                                        $inOutDetail->in_out_id = $input['id'];
                                        $input_val = true;
                                        $cont_in++;
                                    }elseif ($inventoryDetail->difference_amount < 0){
                                        $inOutDetail->quantity = abs($inventoryDetail->difference_amount);
                                        $inOutDetail->in_out_id = $output->id;
                                        $output_val = true;
                                        $cont_out++;
                                    }
                                    $inOutDetail->unid_exist = $inventoryDetail->current_inventory;
                                    $inOutDetail->category = $reference->id_category;
                                    if (!$inOutDetail->save()) {
                                        Yii::$app->session->set('msn-error', 'Error al guardar las Entradas y Salidas.');
                                        return $this->redirect(['view', 'id' => $inventory_model->id]);
                                    }
                                    $inventoryDetail->adjustable = 'on';
                                }
                            }
                            $inventoryDetail->save();
                            if ($inventoryDetail->difference_amount > 0) {
                                $ids_in[] = $inventoryDetail->id;
                            }else{
                                $ids_out[] = $inventoryDetail->id;
                            }
                        }
                    }
                }

                $response_in = true;
                $response_out = true;


                if($input_val == false){
                    $input->delete();
                }else{
                    if($input->sendEntryERP($input, $user_siesa) == true){
                        $input->status = InOut::STATUS_PROCEDED;
                        $input->save();
                        //Finalizan solo las entradas
                        InventoryDetail::updateAll(
                            ['status' => InventoryDetail::STATUS_FINISHED],
                            ['in', 'id', $ids_in]
                        );
                    }else{
                        $response_in = false;
                    }
                }

                if($output_val == false){
                    $output->delete();
                }else{
                    if($output->sendEntryERP($output, $user_siesa) == true){
                        $output->status = InOut::STATUS_PROCEDED;
                        $output->save();
                        //Finalizan solo las salidas
                        InventoryDetail::updateAll(
                            ['status' => InventoryDetail::STATUS_FINISHED],
                            ['in', 'id', $ids_out]
                        );
                    }else{
                        $response_out = false;
                    }
                }

                if($response_in == true && $response_out == true){
                    $inventory_model->code_status = 'finalizado';
                    $inventory_model->code_name = $estados['finalizado'];
                    $inventory_model->save();

                    Yii::$app->session->set('msn-success', 'Inventario Finalizado con exito.');
                    return $this->redirect(['view', 'id' => $inventory_model->id]);
                }
                Yii::$app->session->set('msn-error', 'El Inventario no se Finalizó, error al realizar el Ajuste.');
                return $this->redirect(['view', 'id' => $inventory_model->id]);
            }else{
                Yii::$app->session->set('msn-error', 'Error al realizar el ajuste.');
                return $this->redirect(['view', 'id' => $inventory_model->id]);
            }
        }
    }

    /**
     * User References Count
     * @return array
     */
    public function actionGetUsersCount(){
        $id_inventory = Yii::$app->request->get('id_inventory');
        $id_reference = Yii::$app->request->get('id_reference');
        $type = Yii::$app->request->get('type');
        $count = Yii::$app->request->get('count');
        $data = [];

        if(empty($id_inventory) == false && empty($id_reference) == false && empty($count) == false){
            if($count == '1'){
                if($type == 'assemble'){
                    $where = 'assemble_1 IS NOT NULL';
                }else{
                    $where = 'counting_1 IS NOT NULL';
                }
            }else{
                if($type == 'assemble'){
                    $where = 'assemble_2 IS NOT NULL';
                }else{
                    $where = 'counting_2 IS NOT NULL';
                }
            }

            $model = InventoryDetail::find()->where(['id_inventory' => $id_inventory, 'id_reference' => $id_reference, 'is_created' => '0'])->andWhere($where)->asArray()->all();
            if(empty($model) == false){
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'OK',
                    'counts' => $model,
                ];
            }else{
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'No existen conteos.',
                ];
            }
        }
    }

    /**
     * AJAX response of list of POS by Type of Inventory
     * @return array
     */
    public function actionGetPdvsGroup(){
        $type = Yii::$app->request->get('type');
        $data = [];

        if(empty($type) == false){
            $model = InventoryType::find()
                ->select('usr_pdv.*')
                ->where(['inventory_type.id' => $type, 'usr_pdv.id_user' => Yii::$app->user->identity->getId()])
                ->innerJoin('usr_pdv', 'inventory_type.id_group_co = usr_pdv.id_group_co')
                ->asArray()->all();

            if(empty($model) == false){
                // Select de PDV habilitados para el Usuario
                $html = '<select id="id-pdv" class="form-control" name="">';
                foreach ($model as $value){
                    $html .= '<option value="'.$value['code_pdv'].'">'.$value['name_pdv'].'</option>';
                }
                $html .= '</select>';

                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'OK',
                    'html' => $html,
                ];
            }else{
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'No existen Puntos de Venta relacionado al Tipo selecciondo.',
                ];
            }
        }
    }
}

