<?php

namespace app\controllers;

use app\models\Configpdv;
use app\models\Generic;
use app\models\Inventory;
use app\models\InventoryDetail;
use app\models\InventoryTypeReferences;
use app\models\InventoryTypeReferencesSearch;
use app\models\InventoryTypeSearch;
use app\models\Planning;
use app\models\Usrpdv;
use Yii;
use app\models\InventoryType;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * Class InventoryTypeController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller for the management of Inventory Types
 * @package app\controllers
 */
class InventoryTypeController extends Controller
{
    /**
     * @inheritdoc && !empty($reference['reference'])
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all InventoryType models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new InventoryTypeSearch();
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $user_co = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('id_group_co')->one();
        if(empty($user_co) == false){
            $user_co = explode("-", $user_co['id_group_co']);
        }else{
            $user_co = null;
        }

        $queryParams['InventoryTypeSearch']['id_group_co'] = $user_co;
        $dataProvider = $searchModel->search($queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Display of Inventory Type
     * @param $id
     * @return string
     * @throws NotFoundHttpException
     */
    public function actionView($id)
    {
        $searchModel = new InventoryTypeReferencesSearch();
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $queryParams['InventoryTypeReferencesSearch']['id_inventory_type'] = $id;
        $dataProvider = $searchModel->search($queryParams);
        $model = $this->findModel($id);

        return $this->render('view', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model' => $model,
        ]);
    }

    /**
     * Creation of a new Inventory Type with its related references
     * @return string|\yii\web\Response
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionCreate()
    {
        $model = new InventoryType();
        // Compañias en BD
        $cias = Yii::$app->runAction('generic/get-cias-drop', []);
        $post = Yii::$app->request->post();
        $selected = false;

        if (empty($post) == false) {
            if(empty($post['Inventory_type_references']) == false){
                // Almacenamiento de Tipo de Inventario
                if(empty($post['InventoryType']) == false){
                    $model->load($post);

                    if ($model->validate() == false) {
                        return $this->render('create', [
                            'model' => $model,
                            'cias' => $cias
                        ]);
                    }else{
                        $model->save();
                        $inventory_type_references = $post['Inventory_type_references'];

                        // Creación de Referencia relacionadas al Tipo de Inventario
                        foreach ($inventory_type_references as $reference){
                            if(empty($reference['selected']) == false){
                                if(!empty($reference['name_category']) && !empty($reference['reference'])){
                                    $selected = true;
                                    // Cantidad actual en Inventario solo mientras pruebas
                                    $inventory_type_references_model = new InventoryTypeReferences();
                                    $inventory_type_references_model->id_category = $reference['id_category'];
                                    $inventory_type_references_model->name_category = $reference['name_category'];
                                    $inventory_type_references_model->reference = $reference['reference'];
                                    $inventory_type_references_model->description = $reference['description'];

                                    if(isset($reference['measured_unit']) == true){
                                        $inventory_type_references_model->measured_unit = $reference['measured_unit'];
                                    }

                                    if(isset($reference['unit_price']) == true){
                                        $inventory_type_references_model->unit_price = $reference['unit_price'];
                                    }

                                    if(isset($reference['id_provider']) == true){
                                        $inventory_type_references_model->id_provider = $reference['id_provider'];
                                    }

                                    if(isset($reference['name_provider']) == true){
                                        $inventory_type_references_model->name_provider = $reference['name_provider'];
                                    }

                                    if (isset($reference['margin_loss']) == true) {
                                        $inventory_type_references_model->margin_loss = $reference['margin_loss'];
                                    }else{
                                        $inventory_type_references_model->margin_loss = 0;
                                    }

                                    if(empty($reference['selected']) == false){
                                        $inventory_type_references_model->selected = $reference['selected'];
                                    }

                                    if (empty($reference['calculable']) == false) {
                                        $inventory_type_references_model->calculable = $reference['calculable'];
                                    }else{
                                        $inventory_type_references_model->calculable = NULL;
                                    }

                                    $inventory_type_references_model->id_inventory_type = $model->id;
                                    $inventory_type_references_model->save();
                                }
                            }
                        }

                        if($selected == true){
                            return $this->redirect(['view', 'id' => $model->id]);
                        }else{
                            $model->remove();
                            Yii::$app->session->setFlash('error', 'Debe existir mínimo una Referencia habilitada.');
                            return $this->render('create', [
                                'model' => $model,
                                'cias' => $cias
                            ]);
                        }
                    }
                }
            }

        }

        return $this->render('create', [
            'model' => $model,
            'cias' => $cias
        ]);
    }

    public function actionValidateType()
    {
        $post = Yii::$app->request->post();

        if (empty($post) == false) {
            // Almacenamiento de Tipo de Inventario
            if(empty($post['name']) == false){
                if(empty($post['id_type_inventory']) == false){
                    $model = InventoryType::findOne(['name' => $post['name'], 'name_group_co' => $post['group'], 'id' => !$post['id_type_inventory']]);
                }else{
                    $model = InventoryType::findOne(['name' => $post['name'], 'name_group_co' => $post['group']]);
                }

                if (empty($model) == false) {
                    \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                    return [
                        'status' => 'ERROR',
                        'message' => 'El nombre '.$post['name'].' ya está asignado a un Tipo de Inventario.'
                    ];
                }else{
                    \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                    return [
                        'status' => 'SUCCESS',
                    ];
                }
            }
        }
    }

    /**
     * Inventory Type Update and Related References
     * @param $id
     * @return string|\yii\web\Response
     * @throws NotFoundHttpException
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        // Compañías en BD
        $cias = Yii::$app->runAction('generic/get-cias-drop', []);
        $post = Yii::$app->request->post();

        if (empty($post) == false) {
            if (empty($post['InventoryType']) == false) {
                $model->load($post);

                if ($model->validate()) {
                    $model->save();
                } else {
                    return $this->render('create', [
                        'model' => $model,
                        'cias' => $cias
                    ]);
                }
            }

            //InventoryTypeReferences::deleteAll('id_inventory_type = '.$model->id);
            $ids_update_references = [];
            $ids_update_references_calculables = [];
            $ids_update_categories = [];

            if (empty($post['Inventory_type_references']) == false) {
                $inventory_type_references = $post['Inventory_type_references'];

                 // Actualización de Tipos de Referencias
                foreach ($inventory_type_references as $reference) {
                    if(empty($reference['selected']) == false) {
                        if(empty($reference['id']) == false){
                            $inventory_type_references_model = InventoryTypeReferences::findOne(['id' => $reference['id']]);
                        }else{
                            //Se crea la Referencia
                            $inventory_type_references_model = new InventoryTypeReferences();
                        }

                        if(!empty($reference['name_category']) && !empty($reference['reference'])){
                            $inventory_type_references_model->id_category = $reference['id_category'];
                            $inventory_type_references_model->name_category = $reference['name_category'];
                            $inventory_type_references_model->reference = $reference['reference'];
                            $inventory_type_references_model->description = $reference['description'];

                            if (isset($reference['measured_unit']) == true) {
                                $inventory_type_references_model->measured_unit = $reference['measured_unit'];
                            }

                            if (isset($reference['margin_loss']) == true) {
                                $inventory_type_references_model->margin_loss = $reference['margin_loss'];
                            }else{
                                $inventory_type_references_model->margin_loss = 0;
                            }

                            if (empty($reference['calculable']) == false) {
                                $inventory_type_references_model->calculable = $reference['calculable'];
                            }else{
                                $inventory_type_references_model->calculable = NULL;
                            }

                            $inventory_type_references_model->selected = $reference['selected'];
                            $inventory_type_references_model->id_inventory_type = $model->id;
                            $inventory_type_references_model->save();
                            $ids_update_categories[$inventory_type_references_model->id_category] = $inventory_type_references_model->id_category;
                            $ids_update_references[] = $inventory_type_references_model->id;
                        }
                    }
                }

                if(empty($ids_update_categories) == false && empty($ids_update_references) == false){
                    $ids_update_references = implode("','", $ids_update_references);

                    foreach ($ids_update_categories as $category){
                        InventoryTypeReferences::deleteAll('id not in (\''.$ids_update_references.'\') AND id_inventory_type = '.$model->id.' AND id_category = '.$category);
                    }
                }
            }
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
            'cias' => $cias
        ]);
    }

    /**
     * Delete of Inventory Type, Related Inventories and Related Planning
     * @param $id
     * @return \yii\web\Response
     * @throws \Throwable
     */
    public function actionDelete($id)
    {
        try{
            $inventories = Inventory::find()->where(['id_inventory_type' => $id])->all();

            if(empty($inventories) == false){
                foreach ($inventories as $value){
                    $details = InventoryDetail::find()->where(['id_inventory' => $value->id])->all();

                    if(empty($details) == false){
                        foreach ($details as $detail){
                            $detail->delete();
                        }
                    }
                    $value->delete();
                }
            }

            Planning::deleteAll('id_inventory_type = '.$id);
            InventoryTypeReferences::deleteAll('id_inventory_type = '.$id);
            $this->findModel($id)->delete();
        }catch (\Exception $e){
            echo $e->getMessage();
        }
        return $this->redirect(['index']);
    }

    /**
     * Search of Inventory Type
     * @param $id
     * @return InventoryType|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = InventoryType::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    public function actionGetReferences(){
        // Id compañia
        $id_cia = Yii::$app->request->get('id_cia');
        // Tipo de Inventario
        $id_type_inventory = Yii::$app->request->get('id_type_inventory');
        // Tipo de Inventario
        $category = Yii::$app->request->get('category');
        // Contador de referencias
        $int_references = Yii::$app->request->get('int_references');
        $references_type_inventory_calculable = null;
        $data = [];

        if(empty($id_cia) == false){
            $model = new InventoryType();
            $references_type_inventory = [];
            $references_margin  = [];
            if(empty($id_type_inventory) == false){
                $references_type_inventory_calculable = InventoryTypeReferences::find()
                    ->select('reference')
                    ->where(['id_inventory_type' => $id_type_inventory, 'id_category' => $category, 'calculable' => 'on'])
                    ->asArray()->indexBy('id')->column();
                $references_type_inventory = InventoryTypeReferences::find()
                    ->select('reference')
                    ->where(['id_inventory_type' => $id_type_inventory, 'id_category' => $category])
                    ->asArray()->indexBy('id')->column();
                $references_margin = InventoryTypeReferences::find()
                    ->select('margin_loss')
                    ->where(['id_inventory_type' => $id_type_inventory, 'id_category' => $category])
                    ->asArray()->indexBy('id')->column();
            }

            $references_siesa = $model->getReferencesByCategory($id_cia, $category);

            if(empty($references_siesa) == false){
                return $this->renderAjax('_formReferences', [
                    'model' => $model,
                    'references_siesa' => $references_siesa,
                    'references_type_inventory' => $references_type_inventory,
                    'references_type_inventory_calculable' => $references_type_inventory_calculable,
                    'references_margin' => $references_margin,
                    'int' => $int_references
                ]);

            }else{
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'La Categoría seleccionada no tiene Referencias.',
                ];
            }
        }
    }

    public function actionGetCategories(){
        // Id compañia
        $id_cia = Yii::$app->request->get('id_cia');
        // Tipo de Inventario
        $id_type_inventory = Yii::$app->request->get('id_type_inventory');
        $data = [];

        if(empty($id_cia) == false){
            $model = new InventoryType();
            // Referencias de SIESA
            $categories = $model->getCategoriesByCia();

            if(empty($categories) == false){
                return $this->renderAjax('_formCategories', [
                    'model' => $model,
                    'categories' => $categories
                ]);
            }else{
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'La Compañia seleccionada no tiene Categorías registadas.',
                ];
            }
        }
    }

    /**
     * Método para actualizar categoría, proveedor y precio de referencias
     * @param $id
     * 
     */
    public function actionUpdateSiesa($id){
        $references = InventoryTypeReferences::find()
            ->where(['id_inventory_type' => $id])
            ->all();
        
        if(empty($references) == false){
            $model = new InventoryTypeReferences();
            foreach ($references as $reference){
                $info = $model->getInfoReference($reference);
                if(empty($info) == false){
                    if(isset($info['id_category']) == true){
                        $reference->id_category = $info['id_category'];
                    }
                    if(isset($info['name_category']) == true){
                        $reference->name_category = $info['name_category'];
                    }

                    if(isset($info['measured_unit']) == true){
                        $reference->measured_unit = $info['measured_unit'];
                    }
                    $reference->save();
                }else{
                    $reference->delete();
                }
            }
        }
        Yii::$app->session->setFlash('success', 'Actualización con SIESA realizada correctamente');
        return $this->redirect(['index']);
    }
}
