<?php

namespace app\controllers;

use app\models\Configpdv;
use app\models\InventoryType;
use app\models\Planning;
use app\models\Usrpdv;
use Yii;
use yii\helpers\ArrayHelper;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Json;
use yii\filters\AccessControl;
use \DateTime;

/**
 * InventoryController implements the CRUD actions for Inventory model.
 */

/**
 * Class InventoryController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller for the management calendar events in the planning of Inventory Types by Point of Sale
 * @package app\controllers
 */
class PlanningController extends Controller
{

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Initial view of Sales Points and months of the year
     * @return string
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionIndex()
    {
        $user_pdv = Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('name_pdv')->one();

        if(!empty($user_pdv)){
            $user_pdv = explode("-", $user_pdv['code_pdv']);
            $pdv_list = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
                ->where(['in', 'code_pdv', $user_pdv])
                ->groupBy('code_pdv')
                ->asArray()->all();

            $pdv_list = ArrayHelper::map($pdv_list, 'code_pdv', 'name_pdv');
        }else{
            $pdv_list = [];
        }
        $months = Yii::$app->runAction('generic/get-months', []);
        return $this->render('index', [
            'pdv_list' => $pdv_list,
            'months' => $months
        ]);
    }

    /**
     * Returns monthly events related to a PDV
     * @return array|string
     */
    public function actionGetEventsMonth(){
        // PDV
        $pdv = Yii::$app->request->get('pdv');
        // Mes
        $month = Yii::$app->request->get('month');
        // Unidades de negocio
        $business_units = Yii::$app->runAction('generic/get-business-unit', []);

        if(empty($pdv) == FALSE && empty($month) == FALSE && empty($business_units) == FALSE){
            $model = new Planning();
            $events = array();
            $last_day = date("d",(mktime(0,0,0,$month+1,1, date('Y'))-1));
            $date_start = date('Y-m-01', strtotime(date('Y-'.($month)).' -1 month'));
            $date_end = date('Y-m-t', strtotime(date('Y-'.($month)).' +1 month'));
            // Grupo de CO de PDV
            $group_pdv = Configpdv::find()->select('id_group_co')
                ->where(['code_pdv' => $pdv])
                ->groupBy('id_group_co')
                ->asArray()->all();

            // Grupos hablitados para el Usuario actual
            $group_list = Usrpdv::find()->select('id_group_co')
                ->where(['id_user' => Yii::$app->user->identity->getId()])
                ->andWhere(['like', 'id_group_co', ArrayHelper::map($group_pdv, 'id_group_co', 'id_group_co')])
                ->groupBy('id_group_co')
                ->asArray()->one();

            if(empty($group_list) == false){
                $group_list = explode("-", $group_list['id_group_co']);
            }

            // Lista de Tipos de Inventario habilitados para el Usuario actua
            $type_list = ArrayHelper::map(InventoryType::find()
                ->where(['in', 'id_group_co', $group_list])
                ->all(), 'id', 'name');

            try{
                // Calendario con planeación de PDV por mes
                $planning_list = Planning::find()->where(['code_pdv' => $pdv])->andWhere(['between', 'date', $date_start, $date_end ])->all();

                if(empty($planning_list) == FALSE){
                    foreach ($planning_list as $value){
                        // Creación de Eventos de Calendario
                        if(empty($value->inventoryType) == FALSE){
                            $Event = new \yii2fullcalendar\models\Event();
                            $Event->id = $value->id;
                            $Event->start = $value->date;
                            $Event->title = $value->inventoryType->name.' - '.$value->id_business_unit;
                            $events[] = $Event;
                        }
                    }
                }

                // Respuesta AJAX de calendario
                return $this->renderAjax('_planning', [
                    'events' => $events,
                    'month' => $month,
                    'model' => $model,
                    'type_list' => $type_list,
                    'business_units' => $business_units,
                ]);

            }catch (\Exception $e){
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'Error de conexión.',
                ];
            }
        }else{
            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => 'ERROR',
                'error' => 'Seleccione el mes y el Punto de Venta.',
            ];
        }
    }

    /**
     * Creation of a new event in Calendar by PDV
     * @return array
     */
    public function actionCreate(){
        $date = Yii::$app->request->get('date');
        // Tipo de Inventario
        $id_inventory_type = Yii::$app->request->get('id_inventory_type');
        // Unidad de Negocio
        $id_business_unit = Yii::$app->request->get('id_business_unit');
        // Código PDV
        $code_pdv = Yii::$app->request->get('code_pdv');

        if(empty($date) == FALSE && empty($id_inventory_type) == FALSE && empty($code_pdv) == FALSE && empty($id_business_unit) == FALSE){
            // Tipo de Inventario
            $inventory_type = InventoryType::find()->all();
            $inventory_type_list = [];

            if(empty($inventory_type) == false){
                foreach ($inventory_type as $type){
                    $inventory_type_list[$type->id] = $type->name;
                }
            }

            // Nombre PDV
            $pdv = Configpdv::findOne(['code_pdv' => $code_pdv]);

            // Validación planación por día de PDV
            $planning = Planning::find()->where([
                'date'=> $date,
                'name_pdv'=> $pdv->name_pdv,
                'code_pdv'=> $code_pdv,
            ])->all();

            if(empty($planning) == false){
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'message' => 'Debe existir solo una planeación por día.'
                ];
            }

            $planning = new Planning();
            $planning->setAttributes([
                'date'=> $date,
                'id_inventory_type'=> $id_inventory_type,
                'id_business_unit'=> $id_business_unit,
                'name_pdv'=> $pdv->name_pdv,
                'code_pdv'=> $code_pdv,
            ]);
            $planning->save();

            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => 'OK',
                'id' => $planning->id,
                'name' => $planning->inventoryType->name.' - '.$id_business_unit,
                'start' => $date,
            ];
        }
    }

    /**
     * Delete of Event with AJAX Response
     * @return array
     * @throws NotFoundHttpException
     * @throws \Throwable
     * @throws \yii\db\StaleObjectException
     */
    public function actionDelete(){
        $id = Yii::$app->request->post('id');

        if(empty($id) == false){
            $this->findModel($id)->delete();

            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => true
            ];
        }
    }

    /**
     * Event Search
     * @param $id
     * @return Planning|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = Planning::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    /**
     * List of all events
     * @return array
     */
    public function actionGetAll()
    {
        $planning_list = Planning::find()->all();
        $result = [];

        foreach ($planning_list as $planning){
            $result[]['title'] = $planning->inventoryType->name;
            $result[]['start'] = $planning->date;
        }
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        return [
            'result' => $planning_list
        ];
    }

    /**
     * Duplicate events from one month to one or more POS
     * @return array
     * @throws \Throwable
     */
    public function actionGetDuplicateEvents(){
        $pdv = Yii::$app->request->get('pdv');
        // Mes
        $month = Yii::$app->request->get('month');
        // Puntos de Ventas
        $pdvs = Yii::$app->request->get('pdvs');

        if(empty($pdv) == FALSE && empty($month) == FALSE && empty($pdvs) == FALSE){
            $pdvs = explode(',', $pdvs);
            $last_day = date("d",(mktime(0,0,0,$month+1,1, date('Y'))-1));
            $date_start = date('Y-'.($month).'-01');
            $date_end = date('Y-'.($month).'-'.$last_day);

            try{
                foreach ($pdvs as $value_pvd){
                    if($pdv != $value_pvd){
                        $name_pdv = Usrpdv::findOne(['code_pdv' => $value_pvd]);
                        $planning_list = Planning::find()->where(['code_pdv' => $pdv])->andWhere(['between', 'date', $date_start, $date_end ])->asArray()->all();

                        if(empty($planning_list) == FALSE){
                            $events = Planning::find()->where(['code_pdv' => $value_pvd])->andWhere(['between', 'date', $date_start, $date_end])->all();

                            if(empty($events) == FALSE){
                                foreach ($events as $value){
                                    $value->delete();
                                }
                            }
                            foreach ($planning_list as $event){
                                $planning = new Planning();
                                $planning->code_pdv = $value_pvd;
                                $planning->name_pdv = $name_pdv->name_pdv;
                                $planning->id_business_unit = $event['id_business_unit'];
                                $planning->id_inventory_type = $event['id_inventory_type'];
                                $planning->date = $event['date'];
                                $planning->save();
                            }
                        }else{
                            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                            return [
                                'status' => 'ERROR',
                                'error' => 'No existen eventos para duplicar.',
                            ];
                        }
                    }
                }

                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'SUCCESS',
                    'message' => 'Eventos duplicados correctamente.',
                ];
            }catch (\Exception $e){
                \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
                return [
                    'status' => 'ERROR',
                    'error' => 'Error de conexión.',
                ];
            }
        }else{
            \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            return [
                'status' => 'ERROR',
                'error' => 'Seleccione los Puntos de Venta.',
            ];
        }
    }

}
