<?php

namespace app\controllers;

use app\models\InOut;
use app\models\InventoryDetail;
use app\models\Param;
use app\models\ParamSearch;
use app\models\User;
use mdm\admin\components\UserStatus;
use Yii;
use yii\filters\AccessControl;
use yii\helpers\ArrayHelper;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use yii\filters\VerbFilter;
use app\models\LoginForm;
use app\models\ContactForm;
use app\models\PasswordResetRequest;
use mdm\admin\models\form\ResetPassword;
use yii\base\InvalidParamException;
use yii\web\BadRequestHttpException;

class UserController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['post'],
                    'logout' => ['post'],
                    'activate' => ['post'],
                ],
            ],
        ];
    }

    public function actionIndex()
    {
        return $this->redirect(['admin/user/index']);
    }

    
    /**
     * Signup new user
     * @return string
     */
    public function actionSignup()
    {
        $model = new User();
        $post = Yii::$app->request->post();

        if(empty($post) == FALSE){
            $model->username = $post['User']['username'];
            $model->email = $post['User']['email'];
            $model->status = ArrayHelper::getValue(Yii::$app->params, 'user.defaultStatus', UserStatus::ACTIVE);
            $model->setPassword($post['User']['password']);
            $model->generateAuthKey();
            if ($model->save()) {
                return $this->redirect(['/user/view', 'id' => $model->id]);
            }
        }

        return $this->render('signup', ['model' => $model]);
    }

    /**
     * Request reset password
     * @return string
     */
    public function actionRequestPasswordReset()
    {
        $model = new PasswordResetRequest();
        if ($model->load(Yii::$app->getRequest()->post()) && $model->validate()) {

            if ($model->sendEmail()) {
                Yii::$app->getSession()->setFlash('success', 'Revise su correo electrónico para obtener más instrucciones.');

                return $this->redirect(['/admin/user/login']);
            } else {
                Yii::$app->getSession()->setFlash('error', 'Lo sentimos, no podemos restablecer la contraseña para el correo electrónico proporcionado.');
            }
        }

        return $this->render('requestPasswordResetToken', [
            'model' => $model,
        ]);
    }

    /**
     * Reset password
     * @return string
     */
    public function actionResetPassword($token)
    {
        try {
            $model = new ResetPassword($token);
        } catch (InvalidParamException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if ($model->load(Yii::$app->getRequest()->post()) && $model->validate() && $model->resetPassword()) {
            Yii::$app->getSession()->setFlash('success', 'Se guardó una nueva contraseña.');

            return $this->redirect(['/admin/user/login']);
        }

        return $this->render('resetPassword', [
            'model' => $model,
        ]);
    }

    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    protected function findModel($id)
    {
        if (($model = User::findOne($id)) !== null) {
            return $model;
        }
        InventoryDetail::deleteAll('id_usuario = '.$id);
        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    public function actionUpdate($id = null)
    {
        if(empty($id) == true){
            $id = Yii::$app->user->identity->getId();
        }
        $model = $this->findModel($id);
        $states = json_decode(Param::findOne(['key' => 'states_users'])->value, true);
        $post = Yii::$app->request->post();

        if(empty($post) == FALSE){
            $model->username = $post['User']['username'];
            $model->email = $post['User']['email'];
            $model->status = $post['User']['status'];

            if(empty($post['User']['password']) == false){
                $model->setPassword($post['User']['password']);
                $model->generateAuthKey();
            }

            if ($model->save()) {
                return $this->redirect(['/user/view', 'id' => $model->id]);
            }
        } else {
            return $this->render('update', [
                'model' => $model,
                'states' => $states
            ]);
        }
    }

    /**
     * Displays a InOut model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id)
        ]);
    }
}
