<?php

namespace app\controllers;

use app\models\Param;
use app\models\UserSiesa;
use app\models\UserSiesaSearch;
use yii\web\Controller;
use yii\filters\VerbFilter;
use yii\web\NotFoundHttpException;
use Yii;
/**
 * Class InOutController
 * @author Ing. Maryeiny Losada Romero
 * Ingeneo S.A.S
 * Controller to manage the Inputs and Outputs of Products in SIESA
 * @package app\controllers
 */
class UserSiesaController extends Controller
{
    
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * List of Users
     * @return string
     * @throws \yii\base\InvalidRouteException
     * @throws \yii\console\Exception
     */
    public function actionIndex()
    {
        $searchModel = new UserSiesaSearch();
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $dataProvider = $searchModel->search($queryParams);
        $states = json_decode(Param::findOne(['key' => 'states_users'])->value, true);

        // Visualización de los Inventarios habilitados
        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'states' => $states
        ]);
    }

    /**
     * Delete of Inputs and Outputs
     * @return array
     * @throws NotFoundHttpException
     * @throws \Throwable
     * @throws \yii\db\StaleObjectException
     */
    public function actionDelete($id){
        try{
            $model = $this->findModel($id);
            $model->delete();
        }catch (\Exception $e){
            Yii::$app->session->setFlash('error', 'Error al Eliminar el Usuario.');
        }

        return $this->redirect(['index']);
    }

    /**
     * Displays a InOut model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        $states = json_decode(Param::findOne(['key' => 'states_users'])->value, true);

        return $this->render('view', [
            'model' => $this->findModel($id),
            'states' => $states
        ]);
    }

    /**
     * Event Search
     * @param $id
     * @return Planning|null
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = UserSiesa::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    /**
     * Creates a new Entry model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new UserSiesa();
        $states = json_decode(Param::findOne(['key' => 'states_users'])->value, true);

        if ($model->load(Yii::$app->request->post())) {
            $model->save();
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('create', [
                'model' => $model,
                'states' => $states
            ]);
        }
    }

    /**
     * Updates an existing Entry model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $states = json_decode(Param::findOne(['key' => 'states_users'])->value, true);

        if ($model->load(Yii::$app->request->post())) {
            $model->save();
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('update', [
                'model' => $model,
                'states' => $states
            ]);
        }
    }
}
