<?php

namespace app\controllers;

use app\models\Configpdv;
use Yii;
use app\models\Usrpdv;
use app\models\UsrpdvSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Json;


/**
 * UsrpdvController implements the CRUD actions for Usrpdv model.
 */
class UsrpdvController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Usrpdv models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new UsrpdvSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $users = Yii::$app->runAction('generic/get-users-drop', []);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'users' => $users
        ]);
    }

    /**
     * Displays a single Usrpdv model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Usrpdv model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Usrpdv();
        $users = Yii::$app->runAction('generic/get-users-drop', []);
        $cias = Yii::$app->runAction('generic/get-cias-drop', []);

        if ($model->load(Yii::$app->request->post()) ) {
            $model->code_pdv = implode('-', $model->code_pdv);
            $model->id_group_co = implode('-', $model->id_group_co);
            $model->id_co = implode('-', $model->id_co);
            $model->validate();
            $model->save();

            if(empty($model->errors) == false){
                return $this->render('create', [
                    'model' => $model,
                    'users' => $users,
                    'cias' => $cias
                ]);
            }
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
            'users' => $users,
            'cias' => $cias
        ]);
    }

    /**
     * Updates an existing Usrpdv model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $users = Yii::$app->runAction('generic/get-users-drop', []);
        $cias = Yii::$app->runAction('generic/get-cias-drop', []);

        if ($model->load(Yii::$app->request->post()) ) {
            $model->code_pdv = implode('-', $model->code_pdv);
            $model->id_group_co = implode('-', $model->id_group_co);
            $model->id_co = implode('-', $model->id_co);
            $model->validate();
            $model->save();

            if(empty($model->errors) == false){
                return $this->render('create', [
                    'model' => $model,
                    'users' => $users,
                    'cias' => $cias
                ]);
            }
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
            'users' => $users,
            'cias' => $cias
        ]);
    }

    /**
     * Deletes an existing Usrpdv model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Usrpdv model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Usrpdv the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Usrpdv::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
}
