<?php
namespace app\components;

use Yii;
use yii\base\Component;
use yii\base\InvalidConfig\Exception;
use yii\helpers\ArrayHelper;
use yii\helpers\HtmlPurifier;
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of IntegradorERP
 *
 * @author categoric2016&
 */
class IntegradorERP extends Component{
    
    public $response;
    private $_listERP;
    private $_classERP;
    
    public function __construct()
    {
        $this->_listERP = Yii::$app->params['listERPS'];
    }
    
    /**
     * Metodo para definir el orden de las clases que interactuan en 
     * el patros de cadena de responsabilidad
     * 
     * @return array/Object
     * 
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    private function _getClassParents()
    {        
        //Obtenemos las clases habilitadas para el modelo de responsabilidad                
        if(count($this->_listERP) > 0){
            $beforeClass = '';
            //Creamos un array de objetos para cada una de las clases y 
            //definimos la estructura de responsabilidades
            foreach($this->_listERP as $key => $value){
                $nameClass = 'app\components\\'.$value;            
                if($key === 0){                    
                    $beforeClass = new $nameClass();                    
                } else{
                    $beforeClass = new $nameClass($beforeClass);                    
                }
                $this->_classERP[] = $beforeClass;
            }
        }

        return $this->_classERP;
    }
    
    /**
     * Integradoe de ERP retorna el listado de clientes
     * @return type
     */
    public function getListClientes(){
        $this->_getClassParents();
        $this->response = [];
        if(count($this->_classERP) > 0){
            $endClass = end($this->_classERP);
            $this->response = $endClass->getListClientes();            
        }
        return $this->response;
    }
    
    /**
     * Integrador retorna el la informacion del cliente
     * 
     * @return array
     */
    public function getCliente($cliente){
        $stringCliente = HtmlPurifier::process($cliente);
        $this->_getClassParents();
        $this->response = [];
        if(count($this->_classERP) > 0){
            $endClass = end($this->_classERP);
            $this->response = $endClass->getCliente($stringCliente);            
        }
        return $this->response;
    }
    
    /**
     * Integrador retorna las sucursales asociadas al nit
     * 
     * @return array
     */
    public function getSucursales($nit){
        $stringNit = HtmlPurifier::process($nit);
        $this->_getClassParents();
        $this->response = [];
        if(count($this->_classERP) > 0){
            $endClass = end($this->_classERP);
            $this->response = $endClass->getSucursales($stringNit);            
        }
        return $this->response;
    }
    
    /**
     * Retorna los productos de cada ERP
     * 
     * @return array
     */
    public function getProductos($nit, $codSucursal){
        $prfNit = HtmlPurifier::process($nit);
        $prfCodSucursal = HtmlPurifier::process($codSucursal);
        $this->_getClassParents();
        $this->response = [];
        if(count($this->_classERP) > 0){
            $endClass = end($this->_classERP);
            $this->response = $endClass->getProductos($prfNit, $prfCodSucursal);            
        }
        return $this->response;
    }
    
    /**
     * Guarda los pedidos en cada ERP
     * @param type $model
     * @return type
     */
    public function guardarPedido($pedido){
        $this->_getClassParents();
        $this->response = [];
        if(count($this->_classERP) > 0){
            $endClass = end($this->_classERP);
            $this->response = $endClass->guardarPedido($pedido);            
        }
        return $this->response;
    }    
    
}
