<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;

/**
 * This is the model class for table "calculator_main_items".
 *
 * @property int $id
 * @property string $name
 * @property string $code_type
 * @property string $quantity_liters
 * @property string $quantity_grams
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 */
class CalculatorMainItems extends ActiveRecord
{
    const TYPE_SIMPLE = 'simple';
    const TYPE_COMPOSED = 'composed';

    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'calculator_main_items';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            ['name', 'unique'],
            [['name', 'code_type'], 'required'],
            [['created', 'modified'], 'safe'],
            [['quantity_liters', 'quantity_grams'], 'number'],
            [['name'], 'string', 'max' => 250],
            [['code_type', 'created_by', 'modified_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'name' => Yii::t('app', 'Reference'),
            'code_type' => Yii::t('app', 'Type'),
            'quantity_liters' => Yii::t('app', 'Liters'),
            'quantity_grams' => Yii::t('app', 'Grams'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' =>Yii::t('app', 'Modified By'),
        ];
    }
}
