<?php

namespace app\models;

use Yii;
use app\components\ClientSiesa;
use yii\db\ActiveRecord;
use yii\behaviors\BlameableBehavior;
use yii\db\Expression;
use app\components\IntegradorERP;
use app\components\ws_in_out\FactoryInOut;
use app\components\WebServices;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "in_out_inventory".
 *
 * @property int $id
 * @property string $type
 * @property string $id_user_siesa
 * @property string $type_created
 * @property string $id_co
 * @property string $name_co
 * @property string $id_cia
 * @property string $name_cia
 * @property string $id_business_unit
 * @property string $name_business_unit
 * @property string $nit
 * @property string $business_name
 * @property string $id_cellar_in
 * @property string $name_cellar_in
 * @property string $id_cellar_out
 * @property string $name_cellar_out
 * @property string $number
 * @property string $description
 * @property string $status
 * @property string $date
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property InOutDetail[] $inOutInventoryDetails
 */
class InOut extends ActiveRecord
{
    const TYPE_IN = 'in';
    const TYPE_OUT = 'out';
    const STATUS_PROCEDED = 'processed';
    const STATUS_PENDIENTE = 'pendiente';
    const STATUS_ALLOW = 'autorizado';
    const STATUS_PARCIAL = 'parcial';
    const STATUS_DELETE = 'borrado';
    const TYPE_CREATED_SYSTEM = 'Sistema';
    const TYPE_CREATED_ADMID = 'Administrador';
    
    public $id_pdv;
    public $cellar_in;
    public $cia;
    
    
    /**
     * Aplica los valores previos del modelo al activerecord
     * @return array
     */
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ? Yii::$app->user->identity->username
                            : '',
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'in_out';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['nit', 'business_name', 'status', 'date', 'id_cellar_out', 
                'id_co', 'name_co', 'id_cia',
                'name_cia', 'id_business_unit'], 'required'],
            [['description', 'type', 'type_created'], 'string'],
            [['date', 'created', 'modified', 'id_user_siesa'], 'safe'],
            [['id_co', 'id_cia', 'nit', 'id_cellar_in', 'id_cellar_out', 'number', 'status', 'modified_by'], 'string', 'max' => 45],
            [['name_co', 'name_cia', 'business_name', 'name_cellar_in', 'name_cellar_out', 'name_business_unit'], 'string', 'max' => 250],
            [['created_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_user_siesa' => Yii::t('app', 'User SIESA'),
            'type' => Yii::t('app', 'Type'),
            'type_created' => Yii::t('app', 'Type Created'),
            'id_co' => Yii::t('app', 'Id Co'),
            'name_co' => Yii::t('app', 'Name Co'),
            'cia' => Yii::t('app', 'Cia'),
            'id_cia' => Yii::t('app', 'Id Cia'),
            'name_cia' => Yii::t('app', 'Name Cia'),
            'id_business_unit' => Yii::t('app', 'Id Business Unit'),
            'name_business_unit' => Yii::t('app', 'Name Business Unit'),
            'nit' => Yii::t('app', 'Nit'),
            'business_name' => Yii::t('app', 'Business Name'),
            'cellar_in' => Yii::t('app', 'Cellar In'),
            'id_cellar_in' => Yii::t('app', 'Id Cellar In'),
            'name_cellar_in' => Yii::t('app', 'Name Cellar In'),
            'id_cellar_out' => Yii::t('app', 'Id Cellar Out'),
            'name_cellar_out' => Yii::t('app', 'Name Cellar Out'),
            'number' => Yii::t('app', 'Number'),
            'description' => Yii::t('app', 'Description'),
            'status' => Yii::t('app', 'Status'),
            'date' => Yii::t('app', 'Date InOut'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInOutInventoryDetails()
    {
        return $this->hasMany(InOutDetail::className(), ['in_out_id' => 'id']);
    }
    
    /**
     * Retorna el listado de clientes
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getLisClients(){
        $integradorERP = new IntegradorERP();
        return $integradorERP->getListClientes();
    }
    
    /**
     * Retorna la informacion del cliente
     * 
     * @param string $cliente Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getClient($cliente = null){
        $data = [];
        $user_pdv = ArrayHelper::map(Usrpdv::find()->where(['id_user' => Yii::$app->user->identity->getId()])->groupBy('name_pdv')->all(), 'code_pdv', 'code_pdv');

        $clients = Configpdv::find()->select(['code_pdv', 'CONCAT(name_pdv," - ",name_cellar_in) AS name_pdv'])
            ->where(['in', 'code_pdv', $user_pdv])
            ->groupBy('code_pdv')
            ->asArray()->all();
        if(count($clients)>0){                   
            foreach ($clients as $client) {
                $data[$client['code_pdv']] = $client['code_pdv'].' - '.$client['name_pdv'];    
            }                 
        } 
        return $data;
    }
    
    /**
     * Obtiene las sucursales a partir del nit
     * 
     * @param string $nit Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getBranches($nit){
        $integradorERP = new ClientSiesa();        
        return $integradorERP->getSucursales($nit);
    }
    
    /**
     * Retorna los productos asociados a un nit y sucursal
     * 
     * @param string $nit         Nit de la sucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getProducts($nit, $cia){
        $siesa = new ClientSiesa;
        return $siesa->getProductos($nit, $cia);
    }
    
    /**
     * Envia el pedido a los diferente ERP
     * 
     * @param Pedidos $model
     * 
     * @return boolean
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function sendEntryERP($model, $user_siesa = null){
        try {
            if (!empty($model) && is_object($model)) {
                $factoryOrder = new FactoryInOut();
                //Pendiente Cliente SIESA
                $arrayXml = $factoryOrder->createEntry($model->id, $model, $user_siesa);

                // Creación del archivo almacenamiento
                if($model->type == InOut::TYPE_IN){
                    $name = 'entrada-inventario-'.$model->id.'-'.time().'.txt';
                }else{
                    $name = 'salida-inventario-'.$model->id.'-'.time().'.txt';
                }

                if(empty( $arrayXml[0]) == false){
                    $txt= fopen(Yii::getAlias('@app').'/web/xml/'.$name, 'a') or die ('Problemas al crear el archivo');
                    fwrite($txt, $arrayXml[0]);
                    fclose($txt);
                }
                $envioSiesa = true;
                if (count($arrayXml) > 0) {
                    foreach ($arrayXml as $xml) {
                        if (!empty($xml)) {
                            $webServices = new WebServices();
                            $method = 'ImportarXML';
                            $url = \Yii::$app->params['paramsWs']['urlWs'];
                            $params = array('pvstrDatos' => $xml, 'printTipoError' => 0);
                            if ($webServices->executeMethodWs($method, $url,
                                            $params)) {
                                $response = $webServices->response;
                                if (isset($response->printTipoError)
                                        && $response->printTipoError === 0) {                                    
                                    $envioSiesa = true;
                                } else {
                                    $error = $response->ImportarXMLResult->any;                                  
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $xml . ']}', true) . '}',
                                            'webservices');
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $error . ']}', true) . '}',
                                            'webservices');
                                    $envioSiesa = false;
                                }
                            }
                        }
                    }
                }

            }            
        } catch (\Exception $exc) {
            $envioSiesa = false;
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r('{Error ['. $exc->getMessage()
                    . ']}', true) . '}', 'webservices');
        }
        return $envioSiesa;
    }
    
    /**
     * Metodo para calcular el numero de dias 
     * entre la fecha de entrega y la fecha actual del sistema
     * 
     * @param string $fecha Fecha de entrega
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   SVN: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getNumDiasEntrega($fecha)
    {
        if(!empty($fecha) && is_string($fecha)) {
            $segundos = strtotime($fecha) - strtotime(date('Y-m-d'));
            return abs(intval($segundos / 60 / 60 / 24));
        }
        return false;
    }
    
    /**
     * Retorna el 
     * 
     * @param string $fecha Fecha de entrega
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   SVN: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getSolicitante($nit)
    {
        $siesa = new ClientSiesa;
        return $siesa->getCodeApplicant($nit);
    }
    
    /**
     * Retorna la informacion de las bodegas
     * 
     * @param string $cellar Bodega
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getCellar($cellar){
        $siesa = new ClientSiesa;
        return $siesa->getCellar($cellar);
    }
    

}
