<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\behaviors\BlameableBehavior;
use yii\db\Expression;

/**
 * This is the model class for table "entry_detail".
 *
 * @property int $id
 * @property string $reference
 * @property string $description
 * @property double $quantity
 * @property double $quantity_count
 * @property double $unid_exist
 * @property string $category
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 * @property int $in_out_id
 *
 * @property InOutInventory $in-out
 */
class InOutInventoryDetail extends ActiveRecord
{
    /**
     * Aplica los valores previos del modelo al activerecord
     * @return array
     */
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ? Yii::$app->user->identity->username
                            : '',
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'in_out_detail';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['reference', 'description', 'quantity', 'in_out_id', 'quantity_count'], 'required'],
            [['description'], 'string'],
            [['quantity', 'unid_exist', 'quantity_count'], 'number'],
            [['created', 'modified'], 'safe'],
            [['in_out_id'], 'integer'],
            [['reference'], 'string', 'max' => 250],
            [['category'], 'string', 'max' => 45],
            [['created_by', 'modified_by'], 'string', 'max' => 150],
            [['in_out_id'], 'exist', 'skipOnError' => true, 'targetClass' => InOutInventory::className(), 'targetAttribute' => ['in_out_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'reference' => Yii::t('app', 'Reference'),
            'description' => Yii::t('app', 'Description'),
            'quantity' => Yii::t('app', 'Quantity'),
            'quantity_count' => Yii::t('app', 'Quantity Count'),
            'unid_exist' => Yii::t('app', 'Unid Exist'),
            'category' => Yii::t('app', 'Category'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
            'in_out_id' => Yii::t('app', 'InOutInventory ID'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInOutInventory()
    {
        return $this->hasOne(InOutInventory::className(), ['id' => 'in_out_id']);
    }
}
