<?php

namespace app\models;

use app\components\ClientSiesa;
use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;

/**
 * This is the model class for table "inventory".
 *
 * @property int $id
 * @property int $id_inventory_type
 * @property string $code_status
 * @property string $code_name
 * @property string $code_pdv
 * @property string $name_pdv
 * @property string $consecutive
 * @property string $business_unit
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property InventoryType $inventoryType
 */
class Inventory extends ActiveRecord
{
    const STATUS_FINAL = 'finalizado';
    const STATUS_PENDIENTE_1 = 'pendiente_cont_1';
    const STATUS_PENDIENTE_2 = 'pendiente_cont_2';
    const STATUS_DESCUADRE = 'descuadre';
    const STATUS_PENDIENTE_AJUSTE = 'pendiente_ajuste';

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'inventory';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id_inventory_type', 'code_status', 'code_name', 'code_pdv', 'consecutive'], 'required'],
            [['id_inventory_type'], 'integer'],
            [['created', 'modified'], 'safe'],
            [['code_status', 'name_pdv', 'code_name', 'created_by', 'modified_by', 'business_unit'], 'string', 'max' => 150],
            [['consecutive'], 'string', 'max' => 10],
            [['id_inventory_type'], 'exist', 'skipOnError' => true, 'targetClass' => InventoryType::className(), 'targetAttribute' => ['id_inventory_type' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_inventory_type' => Yii::t('app', 'Create Inventory Type'),
            'code_status' => Yii::t('app', 'Code Status'),
            'code_name' => Yii::t('app', 'Code Name'),
            'name_pdv' => Yii::t('app', 'Name Pdv'),
            'code_pdv' => Yii::t('app', 'Code Pdv'),
            'consecutive' => Yii::t('app', 'Consecutive'),
            'business_unit' => Yii::t('app', 'Business Unit'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryType()
    {
        return $this->hasOne(InventoryType::className(), ['id' => 'id_inventory_type']);
    }

    /**
     * @return Todos los conteos de los auxiliares
     */
    public function getUsersCount($id_inventory, $id_reference, $count){

    }

    /**
     *
     * @return array|bool|null
     */
    public function getInfoReferencesInventory($id_cia, $references, $cellars){
        $siesa = new ClientSiesa();

        try{
            $exception_initial = json_decode(Param::findOne(['key' => 'exception_references_initial_digit'])->value, true);
            $plan_category = Param::findOne(['key' => 'plan_category'])->value;

            return $siesa->getInfoReferencesInventory($id_cia, $references, $cellars, $exception_initial, $plan_category);
        }catch (\Exception $e){
            return false;
        }
    }

    /**
     *
     * @return array|bool|null
     */
    public function getLastPriceInventory($references){
        $siesa = new ClientSiesa();

        try{
            return $siesa->getLastPriceInventory($references);
        }catch (\Exception $e){
            return false;
        }
    }

}
