<?php

namespace app\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use app\models\Inventory;

/**
 * InventorySearch represents the model behind the search form of `app\models\Inventory`.
 *
* @property InventoryType $inventoryType
 */
class InventorySearch extends Inventory
{
    public $id_cia;
    public $id_group;
    public $name_inventory;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'id_inventory_type', 'id_cia'], 'integer'],
            [['code_name', 'name_pdv', 'code_pdv', 'consecutive', 'created_by', 'modified_by', 'id_group'], 'safe'],
            [['id_inventory_type'], 'exist', 'skipOnError' => true, 'targetClass' => InventoryType::className(), 'targetAttribute' => ['id_inventory_type' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = Inventory::find();
        $roles = \Yii::$app->authManager->getRolesByUser(Yii::$app->user->identity->getId());
        $auxiliar = array_key_exists ('Auxiliar de Inventario', $roles);
        $planeador = array_key_exists ('Planeador de Inventarios', $roles);
        $admin = array_key_exists ('Administrador', $roles);
        $admin_pdv = array_key_exists ('Administrador de PVD', $roles);
        
        $this->load($params);

        if(empty($this->id_cia) == false){
            if(empty($this->id_group) == false){
                $query->innerJoin('inventory_type', "inventory.id_inventory_type=inventory_type.id")->where(['inventory_type.id_group_co' => $this->id_group])->andWhere(['inventory_type.id_cia' => $this->id_cia]);
            }else{
                $query->innerJoin('inventory_type', "inventory.id_inventory_type=inventory_type.id")->where(['inventory_type.id_cia' => $this->id_cia]);
            }
        }else if(empty($this->id_group) == false){
            $query->innerJoin('inventory_type', "inventory.id_inventory_type=inventory_type.id")->where(['inventory_type.id_group_co' => $this->id_group]);
        }

        if($auxiliar == true && $admin == false && $admin_pdv == false){
            $query->andWhere('inventory.code_status = "pendiente_cont_1" || inventory.code_status = "pendiente_cont_2"');
        }

        if($planeador == true && $admin == false){
            $query->andWhere('inventory.code_status = "pendiente_ajuste"');
        }

        if(!empty($this->code_pdv)){
            $query->andWhere(['in', 'code_pdv', $this->code_pdv]);
        }else{
            $query->andWhere('code_pdv IS NULL');
        }

        // add conditions that should always apply here
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'pagination' => [
                'pageSize' => 20,
            ],
        ]);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $query->andFilterWhere(['like', 'consecutive', trim($this->consecutive)])
            ->andFilterWhere(['in', 'code_pdv', $this->code_pdv])
            ->andFilterWhere(['like', 'id_inventory_type', trim($this->id_inventory_type)])
            ->andFilterWhere(['like', 'name_pdv', trim($this->name_pdv)])
            ->andFilterWhere(['like', 'code_status', trim($this->code_name)])
            ->andFilterWhere(['like', 'created_by', trim($this->created_by)])
            ->andFilterWhere(['like', 'modified_by', trim($this->modified_by)]);
        return $dataProvider;
    }
}
