<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;
use app\components\ClientSiesa;

/**
 * This is the model class for table "inventory_type".
 *
 * @property int $id
 * @property string $id_cia
 * @property string $name_cia
 * @property string $id_group_co
 * @property string $name_group_co
 * @property string $name
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 */
class InventoryType extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'inventory_type';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name', 'id_group_co'], 'unique', 'targetAttribute' => ['name', 'id_group_co']],
            [['id_cia', 'name_cia', 'name_group_co', 'name', 'id_group_co'], 'required'],
            [['created', 'modified'], 'safe'],
            [['id_cia', 'id_group_co'], 'string', 'max' => 45],
            [['name_cia'], 'string', 'max' => 250],
            [['name_group_co', 'name', 'created_by', 'modified_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_cia' => Yii::t('app', 'Id Cia'),
            'name_cia' => Yii::t('app', 'Name Cia'),
            'id_group_co' => Yii::t('app', 'Id Group Co'),
            'name_group_co' => Yii::t('app', 'Name Group Co'),
            'name' => Yii::t('app', 'Name'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By')
        ];
    }

    public function getReferences($id_cia, $cellars){
        $siesa = new ClientSiesa();

        try{
            return $siesa->getReferences($id_cia, $cellars);
        }catch (\Exception $e){
            return false;
        }
    }

    public function getCategoriesByCia(){
        $siesa = new ClientSiesa();

        try{
            $plan_category = Param::findOne(['key' => 'plan_category'])->value;
            return $siesa->getCategories($plan_category);
        }catch (\Exception $e){
            return false;
        }
    }

    public function getReferencesByCategory($id_cia, $category){
        $siesa = new ClientSiesa();

        try{
            $exception_initial = json_decode(Param::findOne(['key' => 'exception_references_initial_digit'])->value, true);
            $plan_category = Param::findOne(['key' => 'plan_category'])->value;
	
            return $siesa->getReferencesByCategory($id_cia, $category, $exception_initial, $plan_category);
        }catch (\Exception $e){
            return false;
        }
    }

}
