<?php

namespace app\models;

use Codeception\PHPUnit\ResultPrinter\Report;
use Yii;
use yii\data\ActiveDataProvider;
use kartik\grid\GridView;
use yii\db\Query;
use yii\helpers\ArrayHelper;
use kartik\select2\Select2;

/**
 * This is the model class for table "reportes".
 *
 * @property int $id
 * @property string $report
 * @property string $addParameters
 * @property string $defaultParameters
 * @property string $ejex
 * @property string $ejey
 * @property string $type
 * @property int $cia_id
 * @property int $user_id
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property User $user
 */
class ReportsDynamic extends BeforeModel
{
    public $id;
    public $cia;
    public $group_co;
    public $pdv;
    public $types_inventory;
    public $inventory;
    public $status;
    public $reason;
    public $reference;
    public $date;
    public $counting_1;
    public $counting_2;
    public $assemble_1;
    public $assemble_2;
    public $month;
    public $year;
    public $id_cellar;
    public $cellar;
    public $unit_price;
    public $variation;

    public $arrayKeys;

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'reports';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'cia', 'group_co' , 'pdv', 'types_inventory', 'inventory',
                'status', 'reference', 'date', 'counting_1',
                'counting_2', 'assemble_1', 'assemble_2', 'month', 'year', 'cellar', 'id_cellar', 'unit_price', 'reason', 'variation'], 'safe']];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'report' => Yii::t('app', 'Report'),
            'addParameters' => Yii::t('app', 'Add Parameters'),
            'defaultParameters' => Yii::t('app', 'Default Parameters'),
            'ejex' => Yii::t('app', 'Ejex'),
            'ejey' => Yii::t('app', 'Ejey'),
            'type' => Yii::t('app', 'Tipo'),
            'cia_id' => Yii::t('app', 'Cia'),
            'user_id' => Yii::t('app', 'User'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * Listado de los parametros por defecto sin espacios
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listDefaultParameters()
    {
        $list = 'group_co,pdv,types_inventory,inventory,status,reference,counting_1,counting_2,assemble_1,assemble_2,date';
        return str_replace(' ', '', $list);
    }

    /**
     * Listado de los parametros adicionales sin espacios
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParametersAdd()
    {
        $list = 'cia,month,year,cellar,id_cellar,unit_price,reason,variation';
        return str_replace(' ', '', $list);
    }

    /**
     * Retorna el listado a partir de un array y un string dado
     *
     * @param string $string String separado por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTempList($string)
    {
        return $this->_getDataFromList($string, $this->listParams());
    }

    /**
     * Retorna el array de los parametros que iran el en select de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelect($params)
    {
        return $this->_getDataFromList($params, $this->_paramsSelect());
    }

    /**
     * Retorna el array de los parametros que iran el en select de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelectGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsSelectGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupBy($params)
    {
        return $this->_getDataFromList($params, $this->_paramsGroupBy());
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupByGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsGroupByGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en group de la consulta
     * de ventas
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListOrderByGraph($params)
    {
        return $this->_getDataFromList($params, $this->_paramsOrderByGraph($params));
    }

    /**
     * Retorna el array de los parametros que iran el en grid de la vista
     *
     * @param string $params Parametros separados por comas
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListGrid($params)
    {
        return $this->_getDataFromList($params, $this->listParamsGrid());
    }

    /**
     * Funcion que busca los datos de un string en un array
     *
     * @param string $string Datos separados por comas
     * @param array $list Array donde se buscaran los datos
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getDataFromList($string, $list)
    {
        $array = explode(',', $string);
        $arrayResul = [];
        if (count($array) > 0) {
            foreach ($array as $value) {
                $trimValue = trim($value);
                if (isset($list[$trimValue])) {
                    $arrayResul[$trimValue] = $list[$trimValue];
                }
            }
        }
        return $arrayResul;
    }

    /**
     * Lisrados de parametros configurables en el reporte
     *
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParams()
    {
        return [
            'year' => ['content' => Yii::t('app', 'Year')],
            'month' => ['content' => Yii::t('app', 'Month')],
            'cia' => ['content' => Yii::t('app', 'Cia')],
            'group_co' => ['content' => Yii::t('app', 'Name Group Co')],
            'pdv' => ['content' => Yii::t('app', 'PDV')],
            'types_inventory' => ['content' => Yii::t('app', 'Name Type Inventario')],
            'inventory' => ['content' => Yii::t('app', 'Inventory ID')],
            'status' => ['content' => Yii::t('app', 'Status')],
            'reference' => ['content' => Yii::t('app', 'Reference')],
            'reason' => ['content' => Yii::t('app', 'Reason of Carelessness')],
            'counting_1' => ['content' => Yii::t('app', 'Counting 1')],
            'counting_2' => ['content' => Yii::t('app', 'Counting 2')],
            'assemble_1' => ['content' => Yii::t('app', 'Ensamble 1')],
            'assemble_2' => ['content' => Yii::t('app', 'Ensamble 2')],
            'date' => ['content' => Yii::t('app', 'Date')],
            'id_cellar' => ['content' => Yii::t('app', 'Id Cellar')],
            'cellar' => ['content' => Yii::t('app', 'Cellar')],
            'unit_price' => ['content' => Yii::t('app', 'Adjusted Cost')],
            'variation' => ['content' => Yii::t('app', 'Variation')],
        ];
    }

    /**
     * Metodo que retorna el listado de las columnas para el grid
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsGrid()
    {
        # Traemos todos los meses para el filtro -------------------------------
        $id_user = Yii::$app->user->identity->getId();
        $pdv_user_code = ArrayHelper::map(Usrpdv::find()
            ->where(['id_user' => $id_user])
            ->groupBy('name_pdv')->all(), 'code_pdv', 'code_pdv');
        // Grupos de Centro de Operaciones habilitados al Usuario actual
        $groups_user = ArrayHelper::map(Usrpdv::find()
            ->where(['id_user' => $id_user])
            ->groupBy('id_group_co')->all(), 'id_group_co', 'id_group_co');

        $status = json_decode(Param::findOne(['key' => 'status_inventory'])->value, true);
        $months = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio',
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        $types_inventory =  ArrayHelper::map(
            InventoryType::find()->select('name')
                ->orderby('name')->groupBy('name')->asArray()->all(), 'name', 'name');
        $pdvs =  ArrayHelper::map(
            Configpdv::find()->select(['code_pdv', 'name_pdv'])->andWhere(['in', 'code_pdv', $pdv_user_code])
                ->orderby('name_pdv')->groupBy('name_pdv')->asArray()->all(), 'name_pdv', 'name_pdv');
        $cias =  ArrayHelper::map(
            Configpdv::find()->select(['id_cia', 'name_cia'])
                ->orderby('name_cia')->groupBy('name_cia')->asArray()->all(), 'name_cia', 'name_cia');
        $groups_co =  ArrayHelper::map(
            Configpdv::find()->select(['id_group_co', 'name_group_co'])->where(['in', 'id_group_co', $groups_user])
                ->orderby('name_group_co')->groupBy('name_group_co')->asArray()->all(), 'name_group_co', 'name_group_co');
        $reasons = json_decode(Param::findOne(['key' => 'reasons_inventory'])->value, true);

        return [
            'year' => [
                'attribute' => 'year',
                'header' => Yii::t('app', 'Year'),
                'value' => 'year',
                'group'=>true,
                'groupedRow'=>false,
                'groupFooter'=>function () {
                    return [
                        'mergeColumns'=>[['month', 'inventory', 'counting_1', 'counting_2', 'assemble_1', 'assemble_2']],
                        'content'=>[
                            'year' => Yii::t('app', 'Total Year'),
                            'inventory'=>  GridView::F_COUNT,
                            'counting_1' => GridView::F_SUM,
                            'counting_2' => GridView::F_SUM,
                            'assemble_1' => GridView::F_SUM,
                            'assemble_2' => GridView::F_SUM
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'inventory' => ['format' => 'number', 'decimals' => 0],
                            'counting_1' => ['format' => 'number', 'decimals' => 0],
                            'counting_2' => ['format' => 'number', 'decimals' => 0],
                            'assemble_1' => ['format' => 'number', 'decimals' => 0],
                            'assemble_2' => ['format' => 'number', 'decimals' => 0],
                        ],
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                },
            ],
            'month' => [
                'attribute' => 'month',
                'filter' => $months,
                'filterType'=>  GridView::FILTER_SELECT2,
                'width' => '120px',
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true],
                ],
                'value' => function($model) use($months){
                    return isset($months[$model['month']-1]) ? $months[$model['month']-1] : null;
                },
                'group'=>true,
                'groupedRow'=>false,
                'subGroupOf'=>'year', // supplier column index is the parent group,
                'groupFooter'=>function ($model, $key, $index, $widget) { // Closure method
                    return [
                        'mergeColumns'=>[['year', 'inventory', 'counting_1', 'counting_2', 'assemble_1', 'assemble_2']],
                        'content'=>[
                            'month' => Yii::t('app', 'Total Month'),
                            'inventory'=>  GridView::F_COUNT,
                            'counting_1' => GridView::F_SUM,
                            'counting_2' => GridView::F_SUM,
                            'assemble_1' => GridView::F_SUM,
                            'assemble_2' => GridView::F_SUM
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'inventory' => ['format' => 'number', 'decimals' => 0],
                            'counting_1' => ['format' => 'number', 'decimals' => 0],
                            'counting_2' => ['format' => 'number', 'decimals' => 0],
                            'assemble_1' => ['format' => 'number', 'decimals' => 0],
                            'assemble_2' => ['format' => 'number', 'decimals' => 0],
                        ],
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                }
            ],
            'cia' => [
                'attribute' => 'cia',
                'header' => Yii::t('app', 'Cia'),
                'filter'=>$cias,
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
            ],
            'group_co' => [
                'attribute' => 'group_co',
                'header' => Yii::t('app', 'Name Group Co'),
                'filter'=>$groups_co,
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
            ],
            'pdv' => [
                'attribute' => 'pdv',
                'header' => Yii::t('app', 'PDV'),
                'filter'=>$pdvs,
                'width' => '120px',
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
            ],
            'types_inventory' => [
                'attribute' => 'types_inventory',
                'width' => '120px',
                'header' => Yii::t('app', 'Name Type Inventario'),
                'filter'=>$types_inventory,
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
            ],
            'inventory' => [
                'attribute' => 'inventory',
                'header' => Yii::t('app', 'Inventory ID'),
            ],
            'id_cellar' => [
                'attribute' => 'id_cellar',
                'header' => Yii::t('app', 'Id Cellar'),
            ],
            'cellar' => [
                'attribute' => 'cellar',
                'header' => Yii::t('app', 'Cellar'),
            ],
            'unit_price' => [
                'attribute' => 'unit_price',
                'header' => Yii::t('app', 'Adjusted Cost'),
            ],
            'status' => [
                'attribute' => 'status',
                'header' => Yii::t('app', 'Status'),
                'filter'=>$status,
                'width' => '120px',
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
            ],
            'reference' => [
                'attribute' => 'reference',
                'header' => Yii::t('app', 'Reference'),
            ],
            'variation' => [
                'attribute' => 'variation',
                'header' => Yii::t('app', 'Variation'),
                'format'=>['decimal',3]
            ],
            'reason' => [
                'attribute' => 'reason',
                'header' => Yii::t('app', 'Reason of Carelessness'),
                'filter'=>$reasons,
                'width' => '120px',
                'filterType'=>  GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
                'value' => function($model) use ($reasons){
                    if(empty($reasons[$model['reason']]) == false){
                        return $reasons[$model['reason']];
                    }
                }
            ],
            'counting_1' => [
                'attribute' => 'counting_1',
                'header' => Yii::t('app', 'Counting 1'),
                'format'=>['decimal',3]
            ],
            'counting_2' => [
                'attribute' => 'counting_2',
                'header' => Yii::t('app', 'Counting 2'),
                'value' => function($model){
                    return empty($model['counting_2']) == FALSE || $model['counting_2'] === 0 ? $model['counting_2'] : '';
                },
                'contentOptions' => function($model) {
                    return (empty($model['counting_2']) == TRUE) ? ['class' => 'null-reporting'] : [];
                },
                'format'=>['decimal',3]
            ],
            'assemble_1' => [
                'attribute' => 'assemble_1',
                'header' => Yii::t('app', 'Ensamble 1'),
                'value' => function($model){
                    return empty($model['assemble_1']) == FALSE || $model['assemble_1'] === 0 ? $model['assemble_1'] : '';
                },
                'contentOptions' => function($model) {
                    return empty($model['assemble_1']) == TRUE ? ['class' => 'null-reporting'] : [];
                }
            ],
            'assemble_2' => [
                'attribute' => 'assemble_2',
                'header' => Yii::t('app', 'Ensamble 2'),
                'value' => function($model){
                    return empty($model['assemble_2']) == FALSE || $model['assemble_2'] === 0 ? $model['assemble_2'] : '';
                },
                'contentOptions' => function($model) {
                    return empty($model['assemble_2']) == TRUE ? ['class' => 'null-reporting'] : [];
                }
            ],
            'date' => [
                'attribute' => 'date',
                'header' => Yii::t('app', 'Date'),
                'filterType' => GridView::FILTER_DATE,
                'filterWidgetOptions' => [
                    'options' => ['readonly'=>true],
                    'pluginOptions' => [
                        'allowClear' => true,
                        'locale'=>[
                            'format' => 'YYYY-MM-DD',
                            'separator'=>' - ',
                        ],
                    ],
                ],
                'width' => '200px',
            ]
        ];
    }

    private function _paramsSelectGraph() {
        $selectMonth = 'MONTH(inventory.created) as month';
        $selectYear = 'YEAR(inventory.created) as year';

        # ----------------------------------------------------------------------
        return [
            'month' => $selectMonth,
            'year' => $selectYear,
            'cia' => 'inventory_type.name_cia as cia',
            'group_co' => 'inventory_type.name_group_co as group_co',
            'pdv' => 'inventory.name_pdv as pdv',
            'types_inventory' => 'inventory_type.name as types_inventory',
            'inventory' => 'COUNT(inventory.consecutive) as inventory',
            'status' => 'inventory.code_name as status',
            'reference' => 'inventory_type_references.description as reference',
            'counting_1' => 'SUM(inventory_detail.counting_1) as counting_1',
            'counting_2' => 'SUM(inventory_detail.counting_2) as counting_2',
            'assemble_1' => 'SUM(inventory_detail.assemble_1) as assemble_1',
            'assemble_2' => 'SUM(inventory_detail.assemble_2) as assemble_2',
            'date' => 'inventory.created as date',
            'id_cellar' => 'inventory_detail.id_cellar as id_cellar',
            'cellar' => 'inventory_detail.cellar as cellar',
            'unit_price' => 'inventory_detail.unit_price as unit_price',
            'reason' => 'inventory_detail.reason as reason',
            'variation' => '(CASE WHEN (inventory_detail.counting_2 IS NOT NULL AND inventory_detail.current_inventory > 0) THEN ((inventory_detail.current_inventory-inventory_detail.counting_2)/inventory_detail.current_inventory)*100 WHEN (inventory_detail.counting_1 IS NOT NULL AND inventory_detail.current_inventory > 0) THEN ((inventory_detail.current_inventory-inventory_detail.counting_1)/inventory_detail.current_inventory)*100 END) as `variation`, '
        ];
    }

    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelect() {
        $selectMonth = 'MONTH(inventory.created) as month';
        $selectYear = 'YEAR(inventory.created) as year';

        # ----------------------------------------------------------------------
        return [
            'month' => $selectMonth,
            'year' => $selectYear,
            'cia' => 'inventory_type.name_cia as cia',
            'group_co' => 'inventory_type.name_group_co as group_co',
            'pdv' => 'inventory.name_pdv as pdv',
            'types_inventory' => 'inventory_type.name as types_inventory',
            'inventory' => 'inventory.consecutive as inventory',
            'status' => 'inventory.code_name as status',
            'reference' => 'inventory_type_references.description as reference',
            'reason' => 'inventory_detail.reason',
            'counting_1' => 'inventory_detail.counting_1',
            'counting_2' => 'inventory_detail.counting_2',
            'assemble_1' => 'inventory_detail.assemble_1',
            'assemble_2' => 'inventory_detail.assemble_2',
            'date' => 'inventory.created as date',
            'id_cellar' => 'inventory_detail.id_cellar as id_cellar',
            'cellar' => 'inventory_detail.cellar as cellar',
            'unit_price' => 'inventory_detail.unit_price as unit_price',
            'variation' => '(CASE WHEN (inventory_detail.counting_2 IS NOT NULL AND inventory_detail.current_inventory > 0) THEN ((inventory_detail.current_inventory-inventory_detail.counting_2)/inventory_detail.current_inventory)*100 WHEN (inventory_detail.counting_1 IS NOT NULL AND inventory_detail.current_inventory > 0) THEN ((inventory_detail.current_inventory-inventory_detail.counting_1)/inventory_detail.current_inventory)*100 END) as `variation`, '
        ];
    }

    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     *
     * @return array
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelectGrafica($params) {
        return [
            'cia' => 'inventory_type.id_cia',
            'group_co' => 'inventory_type.id_group_co',
            'pdv' => 'inventory.code_pdv',
            'types_inventory' => 'inventory_type.name',
            'inventory' => 'inventory.consecutive',
            'status' => 'inventory.code_status',
            'reference' => 'inventory_type_references.reference',
            'counting_1' => 'inventory_detail.counting_1',
            'counting_2' => 'inventory_detail.counting_2',
            'assemble_1' => 'inventory_detail.assemble_1',
            'assemble_2' => 'inventory_detail.assemble_2',
            'date' => 'inventory.created',
            'reason' => 'inventory.reason'
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupBy() {
        $selectMonth = 'MONTH(inventory.created)';
        $selectYear = 'YEAR(inventory.created)';

        return [
            'year' => $selectYear,
            'month' => $selectMonth,
            'cia' => 'inventory_type.id_cia',
            'group_co' => 'inventory_type.id_group_co',
            'pdv' => 'inventory.code_pdv',
            'types_inventory' => 'inventory_type.name',
            'inventory' => 'inventory.id',
            'status' => 'inventory.code_status',
            'reference' => 'inventory_type_references.reference',
            'reason' => 'inventory_detail.reason',
            'counting_1' => 'inventory_detail.counting_1',
            'counting_2' => 'inventory_detail.counting_2',
            'assemble_1' => 'inventory_detail.assemble_1',
            'assemble_2' => 'inventory_detail.assemble_2',
            'id_cellar' => 'inventory_detail.id_cellar',
            'cellar' => 'inventory_detail.cellar',
            'unit_price' => 'inventory_detail.unit_price',
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupByGraph($params) {
        $selectMonth = 'MONTH(inventory.created)';
        $selectYear = 'YEAR(inventory.created)';

        return [
            'year' => $selectYear,
            'month' => $selectMonth,
            'cia' => 'inventory_type.id_cia',
            'group_co' => 'inventory_type.id_group_co',
            'pdv' => 'inventory.code_pdv',
            'types_inventory' => 'inventory_type.name',
            'inventory' => 'inventory.id',
            'status' => 'inventory.code_status',
            'reference' => 'inventory_type_references.reference',
            'id_cellar' => 'inventory_detail.id_cellar',
            'cellar' => 'inventory_detail.cellar',
            'unit_price' => 'inventory_detail.unit_price',
            'reason' => 'inventory.reason'
        ];
    }

    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     *
     * @return type
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsOrderByGraph($params) {
        $selectMonth = 'MONTH(inventory.created)';
        $selectYear = 'YEAR(inventory.created)';

        return [
            'year' => $selectYear,
            'month' => $selectMonth,
            'cia' => 'inventory_type.id_cia',
            'group_co' => 'inventory_type.id_group_co',
            'pdv' => 'inventory.code_pdv',
            'types_inventory' => 'inventory_type.name',
            'inventory' => 'inventory.id',
            'status' => 'inventory.code_status',
            'reference' => 'inventory_type_references.reference',
            'counting_1' => 'inventory_detail.counting_1',
            'counting_2' => 'inventory_detail.counting_2',
            'assemble_1' => 'inventory_detail.assemble_1',
            'assemble_2' => 'inventory_detail.assemble_2',
            'id_cellar' => 'inventory_detail.id_cellar',
            'cellar' => 'inventory_detail.cellar',
            'unit_price' => 'inventory_detail.unit_price',
            'reason' => 'inventory.reason'
        ];
    }

    /**
     * Genera los  resultados dinamicos para el reporte
     *
     * @param type $valueListDefecto
     * @param type $params
     * @param type $preview
     *
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function report($valueListDefaultParameters, $params, $preview = false)
    {
        $select = $this->_getListSelect($valueListDefaultParameters);
        $groupBy = $this->_getListGroupBy($valueListDefaultParameters);

        $id_user = Yii::$app->user->identity->getId();
        // Puntos de Ventas habilitados al Usuario actual
        $pdv_user_code = Usrpdv::find()
            ->where(['id_user' => $id_user])
            ->groupBy('name_pdv')->one();

        if(empty($pdv_user_code) == false){
            $pdv_user_code = explode('-', $pdv_user_code->code_pdv);
        }
        # ----------------------------------------------------------------------
        $query = Inventory::find();
        $query->select(implode(',', $select));
        $query->innerJoin('inventory_type', 'inventory.id_inventory_type = inventory_type.id');
        $query->innerJoin('inventory_detail', 'inventory_detail.id_inventory = inventory.id');
        $query->innerJoin('inventory_type_references', 'inventory_type_references.id = inventory_detail.id_reference');
        $query->andWhere(['inventory_detail.is_created' => '1']);
        $query->andWhere(['in', 'inventory.code_pdv', $pdv_user_code]);
        $query->groupBy(implode(',', $groupBy));
        //$query->orderBy(implode(',', $groupBy));

        if ($preview) {
            $query->limit(10);
        }

        $this->load($params);

        $query->andFilterWhere(['like', 'inventory.consecutive', $this->inventory])
            ->andFilterWhere(['like', 'inventory_type_references.description', $this->reference])
            ->andFilterWhere(['like', 'inventory_detail.reason', $this->reason])
            ->andFilterWhere(['like', 'inventory_detail.counting_1', $this->counting_1])
            ->andFilterWhere(['like', 'inventory_detail.counting_2', $this->counting_2])
            ->andFilterWhere(['like', 'inventory_detail.assemble_1', $this->assemble_1])
            ->andFilterWhere(['like', 'inventory_detail.assemble_2', $this->assemble_2])
            ->andFilterWhere(['=', 'inventory.created', $this->date]);
        #-----------------------------------------------------------------------
        if (is_array($this->cia)) {
            $query->andFilterWhere(['in', 'inventory_type.name_cia', $this->cia]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name_cia', $this->cia]);
        }

        if (is_array($this->group_co)) {
            $query->andFilterWhere(['in', 'inventory_type.name_group_co', $this->group_co]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name_group_co', $this->group_co]);
        }

        if (is_array($this->pdv)) {
            $query->andFilterWhere(['in', 'inventory.name_pdv', $this->pdv]);
        } else {
            $query->andFilterWhere(['like', 'inventory.name_pdv', $this->pdv]);
        }

        if (is_array($this->types_inventory)) {
            $query->andFilterWhere(['in', 'inventory_type.name', $this->types_inventory]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name', $this->types_inventory]);
        }

        if (is_array($this->status)) {
            $query->andFilterWhere(['in', 'inventory.code_status', $this->status]);
        } else {
            $query->andFilterWhere(['like', 'inventory.code_status', $this->status]);
        }

        if (empty($this->month) == FALSE) {
            if (is_array($this->month) && count($this->month) > 0) {
                $tempMonth = [];
                foreach ($this->month as $value) {
                    $tempMonth[] = $value + 1;
                }
                $query->andFilterWhere(['in', 'MONTH(inventory.created)', $tempMonth]);
            }
        }

        if (empty($this->year) == FALSE) {
            $tempYear = explode(',', $this->year);
            $query->andFilterWhere(['in', 'YEAR(inventory.created)', $tempYear]);
        }


        # ----------------------------------------------------------------------
        try{
            $arrayResult = $query->asArray()->all();
        }catch (\Exception $e){
            return new \yii\data\ArrayDataProvider([
                'allModels' => [],
                'sort' => [],
                'pagination' => []
            ]);
        }

        # Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';
        $pagination = ['pageSize' => 15];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false) {
                if ($value == $exportAll) {
                    $pagination = false;
                }
            }
        }
        # ----------------------------------------------------------------------
        $sort = [ 'attributes' =>explode(',', $valueListDefaultParameters)];
        # ----------------------------------------------------------------------
        return new \yii\data\ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort' => $sort,
            'pagination' => $pagination
        ]);
    }

    /**
     * Genera los datos para el grafico dinamico
     *
     * @param type $dataEjex
     * @param type $dataEjey
     * @param type $params
     *
     * @return \yii\data\ArrayDataProvider
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reportGraph($dataEjex, $dataEjey, $params, $type) {
        $valueListDefect = $dataEjex.','.$dataEjey;
        $listParams = $this->listParams();
        $select = $this->_getListSelectGraph($valueListDefect);
        $groupBy = $this->_getListGroupByGraph($valueListDefect);
        $id_user = Yii::$app->user->identity->getId();
        $pdv_user_code = Usrpdv::find()
            ->where(['id_user' => $id_user])
            ->groupBy('name_pdv')->one();

        if(empty($pdv_user_code) == false){
            $pdv_user_code = explode('-', $pdv_user_code->code_pdv);
        }
        #----------------------------------------------------------------------
        $query = Inventory::find();
        $query->select(implode(',', $select));
        $query->innerJoin('inventory_type', 'inventory.id_inventory_type = inventory_type.id');
        $query->innerJoin('inventory_detail', 'inventory_detail.id_inventory = inventory.id');
        $query->innerJoin('inventory_type_references', 'inventory_type_references.id = inventory_detail.id_reference');
        $query->andWhere(['inventory_detail.is_created' => '1']);
        $query->andWhere(['in', 'inventory.code_pdv', $pdv_user_code]);
        $query->groupBy(implode(',', $groupBy));
        $this->load($params);
        $nombre = 'Grfica';
        if(isset($params['id'])){
            $this->id = $params['id'];
            $report = Reports::findOne($this->id);
            $nombre  = $report->name;
        }
        $query->andFilterWhere(['like', 'inventory.consecutive', $this->inventory])
            ->andFilterWhere(['like', 'inventory_type_references.description', $this->reference])
            ->andFilterWhere(['like', 'inventory_detail.counting_1', $this->counting_1])
            ->andFilterWhere(['like', 'inventory_detail.counting_2', $this->counting_2])
            ->andFilterWhere(['like', 'inventory_detail.assemble_1', $this->assemble_1])
            ->andFilterWhere(['like', 'inventory_detail.assemble_2', $this->assemble_2])
            ->andFilterWhere(['=', 'inventory.created', $this->date])
            ->andFilterWhere(['=', 'inventory_detail.cellar', $this->cellar])
            ->andFilterWhere(['=', 'inventory_detail.id_cellar', $this->id_cellar])
            ->andFilterWhere(['=', 'inventory_detail.unit_price', $this->unit_price]);

        #-----------------------------------------------------------------------
        if (is_array($this->cia)) {
            $query->andFilterWhere(['in', 'inventory_type.name_cia', $this->cia]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name_cia', $this->cia]);
        }

        if (is_array($this->group_co)) {
            $query->andFilterWhere(['in', 'inventory_type.name_group_co', $this->group_co]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name_group_co', $this->group_co]);
        }

        if (is_array($this->pdv)) {
            $query->andFilterWhere(['in', 'inventory.name_pdv', $this->pdv]);
        } else {
            $query->andFilterWhere(['like', 'inventory.name_pdv', $this->pdv]);
        }

        if (is_array($this->types_inventory)) {
            $query->andFilterWhere(['in', 'inventory_type.name', $this->types_inventory]);
        } else {
            $query->andFilterWhere(['like', 'inventory_type.name', $this->types_inventory]);
        }

        if (is_array($this->status)) {
            $query->andFilterWhere(['in', 'inventory.code_name', $this->status]);
        } else {
            $query->andFilterWhere(['like', 'inventory.code_name', $this->status]);
        }

        if (empty($this->month) == FALSE) {
            if (is_array($this->month) && count($this->month) > 0) {
                $tempMonth = [];
                foreach ($this->month as $value) {
                    $tempMonth[] = $value + 1;
                }
                $query->andFilterWhere(['in', 'MONTH(inventory.created)', $tempMonth]);
            }
        }

        if (empty($this->year) == FALSE) {
            $tempYear = explode(',', $this->year);
            $query->andFilterWhere(['in', 'YEAR(inventory.created)', $tempYear]);
        }
        # ----------------------------------------------------------------------
        try{
            $result = $query->asArray()->all();
        }catch (\Exception $e){
            return new \yii\data\ArrayDataProvider([
                'allModels' => [],
                'sort' => [],
                'pagination' => []
            ]);
        }
        $groupEjex = explode(',', $dataEjex);
        $keyInit = $groupEjex[0];
        unset($groupEjex[0]);
        if(count($groupEjex)>0){
            $arrayResult = ArrayHelper::index($result, $keyInit, $groupEjex);
        }else{
            $arrayResult = ArrayHelper::index($result, $keyInit);
        }
        #-----------------------------------------------------------------------
        $data = [];
        $data['x'] = [];
        $data['y'] = [];
        $categories = [];

        if(count($arrayResult)>0){
            $resultsDataY = [];
            $tempEjeY = explode(',', $dataEjey);
            $this->_recursiveWalk($categories, $arrayResult, $resultsDataY, $tempEjeY);
            #-------------------------------------------------------------------
            foreach ($resultsDataY as $key => $value) {
                switch ($key) {
                    case ('inventory'):
                        $name = 'Cant('.$listParams[$key]['content'].')';
                        break;
                    case ('counting_1'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('counting_2'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('assemble_1'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    case ('assemble_2'):
                        $name = 'Suma('.$listParams[$key]['content'].')';
                        break;
                    default:
                        $name = 'Cant('.$listParams[$key]['content'].')';
                        break;
                }
                $data['y'][] = [
                    'type' => $type,
                    'name'=>$name,
                    'data'=> array_column($value, $key)];
            }
        }
        #-----------------------------------------------------------------------
        $response = [
            'lang' => [
                'downloadJPEG' => 'Descargar en JPG',
                'downloadPDF' => 'Descargar en PDF',
                'downloadPNG' => 'Descargar en PNG',
                'downloadSVG' => 'Descargar en SVG',
                'printChart' => 'Imprimir',
                'resetZoom' => 'Reestablecer',
                'resetZoomTitle' => 'Reestablecer 1:1',
            ],
            'chart' => [
                'renderTo' => 'chart-graficas',
                'height' => '800',
                'zoomType' => 'xy',
                'responsive'=>[
                    'rules'=>[
                        'condition'=>[
                            //'maxWidth'=>1200
                        ],
                        'chartOptions'=>[
                            'legend'=>['enabled'=>false]
                        ]
                    ]
                ]
            ],
            'exporting' => [
                //'scale' => '4',
                //'width' => '900',
                'sourceWidth' => '1000',
                //'printMaxWidth' => '1000',
            ],
            'id' => 'chart-g',
            'title' => ['text' => $nombre],
            'plotOptions' => [
                'column' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                    'center' => ['50%', '50%'],
                    'enableMouseTracking' => false,
                ],
                'area' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                ],
                'line' => [
                    'dataLabels' => [
                        'enabled'=>true,
                        'format'=>'{point.y:,.0f}',
                    ],
                ],
            ],
            'xAxis' => [
                //'title' => ['text' => $listParams[$dataEjex]['content']],
                'labels' => ['rotation' => '-90'],
                'categories' => $categories,
            ],
            'legend'=>[
                    'layout' => 'vertical',
                    'align' => 'left',
                    'verticalAlign' => 'middle'
            ],
            'yAxis' => [
               'title' => ['text' => '']
            ],
            'series' => $data['y']
        ];
        return json_encode($response, JSON_NUMERIC_CHECK);
    }

    /**
     * Funcion recursiva para extraer los datos
     *
     * @param array $categories
     * @param array $datos
     * @param array $resultsDataY
     * @param array $tempEjeY
     *
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _recursiveWalk(&$categories, $datos, &$resultsDataY, $tempEjeY){
        $objAux = [];
        $i = 0;
        foreach($datos as $key => $dato){
            if(isset($dato[$tempEjeY[0]])){
                $categories[] = $key;
                foreach ($tempEjeY as $value) {
                    $resultsDataY[$value][][$value] = $dato[$value];
                }
            } else {
                $objAuxJson = [];
                $this->_recursiveWalk($objAuxJson, $dato, $resultsDataY, $tempEjeY);
                $objAux['name'] = $key;
                $objAux['categories'] = $objAuxJson;
                $categories[] = $objAux;
            }
            if(++$i == 5) {break;}
        }
    }
}
