<?php

namespace app\models;

use Yii;

use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;

/**
 * This is the model class for table "calculator_secondary_items".
 *
 * @property int $id
 * @property int $id_item
 * @property int $code
 * @property int $description
 * @property double $equivalence
 * @property string $code_unity
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 */
class CalculatorSecondaryItems extends ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'calculator_secondary_items';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id_item', 'code', 'description', 'equivalence', 'code_unity'], 'required'],
            [['id_item'], 'integer'],
            [['created', 'modified', 'equivalence'], 'safe'],
            [['description'], 'string', 'max' => 250],
            [['code_unity', 'created_by', 'modified_by', 'code'], 'string', 'max' => 150],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_item' => Yii::t('app', 'Id Item'),
            'code' => Yii::t('app', 'Code'),
            'description' => Yii::t('app', 'Description'),
            'equivalence' => Yii::t('app', 'Equivalence'),
            'code_unity' => Yii::t('app', 'Code Unity'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By')
        ];
    }
}
