<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;
use app\components\ClientSiesa;

/**
 * This is the model class for table "config_pdv".
 *
 * @property int $id
 * @property string $id_cia
 * @property string $name_cia
 * @property string $code_pdv
 * @property string $name_pdv
 * @property string $id_group_co
 * @property string $name_group_co
 * @property string $multi_ubicacion
 * @property string $id_co
 * @property string $name_co
 * @property string $id_cellar_in
 * @property string $name_cellar_in
 * @property string $id_cellar_out
 * @property string $name_cellar_out
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property UsrPdv[] $usrPdvs
 */
class Configpdv extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'config_pdv';
    }
    
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                Yii::$app->user->identity->username : '',
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['code_pdv', 'name_pdv', 'id_co', 'id_cellar_in', 'id_cia', 'name_cia', 'id_group_co'], 'required'],
            [['created', 'modified'], 'safe'],
            [['code_pdv', 'id_group_co', 'id_co', 'id_cellar_in', 'id_cellar_out', 'multi_ubicacion'], 'string', 'max' => 45],
            [['name_pdv', 'name_cellar_in', 'name_cellar_out'], 'string', 'max' => 250],
            [['name_co', 'name_group_co', 'created_by', 'modified_by'], 'string', 'max' => 150],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_cia' => Yii::t('app', 'Id Cia'),
            'name_cia' => Yii::t('app', 'Name Cia'),
            'code_pdv' => Yii::t('app', 'Code Pdv'),
            'name_pdv' => Yii::t('app', 'Name Pdv'),
            'multi_ubicacion' => Yii::t('app', 'Multi-location'),
            'id_co' => Yii::t('app', 'Id Co'),
            'name_co' => Yii::t('app', 'Name Co'),
            'id_group_co' => Yii::t('app', 'Id Group Co'),
            'name_group_co' => Yii::t('app', 'Name Group Co'),
            'id_cellar_in' => Yii::t('app', 'Id Cellar In'),
            'name_cellar_in' => Yii::t('app', 'Name Cellar In'),
            'id_cellar_out' => Yii::t('app', 'Id Cellar Out'),
            'name_cellar_out' => Yii::t('app', 'Name Cellar Out'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsrPdvs()
    {
        return $this->hasMany(UsrPdv::className(), ['config_pdv_id' => 'id']);
    }
    
    /**
     * Retorna la informacion del centro de operacion
     * 
     * @param string $co Centro de operacion
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getListco($co){
        $siesa = new ClientSiesa;
        return $siesa->getListco($co);
    }
    
    /**
     * Retorna la informacion de las bodegas
     * 
     * @param string $cellar Bodega
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getCellar($cellar){
        $siesa = new ClientSiesa;
        return $siesa->getCellar($cellar);
    }

    public function getMultilocation($cellar){
        $siesa = new ClientSiesa;
        return $siesa->getMultilocation($cellar);
    }

    public function getMultilocationReference($cellar, $reference, $description){
        $siesa = new ClientSiesa;
        return $siesa->getMultilocationReference($cellar, $reference, $description);
    }

    public function getLocationsMultilocationReference($cellar, $reference, $description){
        $siesa = new ClientSiesa;
        return $siesa->getLocationsMultilocationReference($cellar, $reference, $description);
    }

    public function getLotesMultilocationReference($cellar, $reference, $description){
        $siesa = new ClientSiesa;
        return $siesa->getLotesMultilocationReference($cellar, $reference, $description);
    }

    /**
     * Retorna la informacion de las emptresas
     * 
     * @param string $cia empresa
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getCia($cia){
        $siesa = new ClientSiesa;
        return $siesa->getCia($cia);
    }
    
    /**
     * Retorna la informacion de las emptresas
     * 
     * @param string $cellar Bodega
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getPdvs($cia){
        $siesa = new ClientSiesa;
        return $siesa->getPdvs($cia);
    }

    /**
     * Retorna los grupos de Centros de Costos relacionas a una compañia
     * @param $cia
     * @return mixed
     */
    public function getGroupsCo($cia){
        $siesa = new ClientSiesa;



        return $siesa->getGroupsCo($cia);
    }

    /**
     * Retorna los Centros de Costos relacionados a un grupo
     * @param $group
     * @return mixed
     */
    public function getListCoByGroup($group, $cia){
        $siesa = new ClientSiesa;
        return $siesa->getListCoByGroup($group, $cia);
    }
}
