<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;

/**
 * This is the model class for table "inventory_detail".
 *
 * @property int $id
 * @property int $id_inventory
 * @property int $id_reference
 * @property int $id_usuario
 * @property double $counting_1
 * @property double $counting_2
 * @property double $assemble_1
 * @property double $assemble_2
 * @property string $notes
 * @property string $status
 * @property string $adjustable
 * @property int $difference_amount
 * @property string $difference_price
 * @property string $id_cellar
 * @property string $cellar
 * @property string $current_inventory
 * @property string $unit_price
 * @property string $reason
 * @property string $is_created
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property Inventory $inventory
 * @property User $user
 * @property InventoryTypeReferences $inventoryReference
 */
class InventoryDetail extends ActiveRecord
{
    const STATUS_IN_PROCESS = 'in_process';
    const STATUS_FINISHED = 'finished';

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'inventory_detail';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id_inventory', 'id_reference', 'id_usuario'], 'required'],
            [['id_inventory', 'id_reference', 'id_usuario', 'is_created'], 'integer'],
            [['counting_1', 'counting_2', 'assemble_1', 'assemble_2', 'difference_price', 'difference_amount', 'unit_price'], 'number'],
            [['created', 'modified'], 'safe'],
            [['notes'], 'string', 'max' => 500],
            [['current_inventory'], 'number'],
            [['created_by', 'modified_by', 'cellar'], 'string', 'max' => 150],
            [['reason', 'status', 'adjustable', 'id_cellar'], 'string', 'max' => 250],
            [['id_inventory'], 'exist', 'skipOnError' => true, 'targetClass' => Inventory::className(), 'targetAttribute' => ['id_inventory' => 'id']],
            [['id_reference'], 'exist', 'skipOnError' => true, 'targetClass' => InventoryTypeReferences::className(), 'targetAttribute' => ['id_reference' => 'id']],
            [['id_usuario'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['id_usuario' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_cellar' => Yii::t('app', 'Cellar'),
            'cellar' => Yii::t('app', 'Cellar'),
            'status' => Yii::t('app', 'Status'),
            'adjustable' => Yii::t('app', 'Ajustable'),
            'id_inventory' => Yii::t('app', 'Inventory ID'),
            'id_reference' => Yii::t('app', 'Inventory Type Reference'),
            'id_usuario' => Yii::t('app', 'Usuario ID'),
            'counting_1' => Yii::t('app', 'Counting 1'),
            'counting_2' => Yii::t('app', 'Counting 2'),
            'assemble_1' => Yii::t('app', 'Assemble 1'),
            'assemble_2' => Yii::t('app', 'Assemble 2'),
            'notes' => Yii::t('app', 'Notes'),
            'is_created' => Yii::t('app', 'Created'),
            'difference_amount' => Yii::t('app', 'Difference Amount'),
            'difference_price' => Yii::t('app', 'Difference Price'),
            'current_inventory' => Yii::t('app', 'Current Inventory'),
            'unit_price' => Yii::t('app', 'Unit Price'),
            'reason' => Yii::t('app', 'Reason'),
            'inventory_id' => Yii::t('app', 'Inventory ID'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventory()
    {
        return $this->hasOne(Inventory::className(), ['id' => 'id_inventory']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryReference()
    {
        return $this->hasOne(InventoryTypeReferences::className(), ['id' => 'id_reference']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(User::className(), ['id' => 'id_usuario']);
    }
}
