<?php

namespace app\models;

use app\components\ClientSiesa;
use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;

/**
 * This is the model class for table "inventory_type_references".
 *
 * @property int $id
 * @property int $id_inventory_type
 * @property string $reference
 * @property string $id_category
 * @property string $description
 * @property string $name_category
 * @property string $measured_unit
 * @property double $margin_loss
 * @property string $selected
 * @property string $calculable
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property InventoryType $inventoryType
 */
class InventoryTypeReferences extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'inventory_type_references';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id_inventory_type', 'reference', 'id_category', 'description', 'name_category', 'margin_loss'], 'required'],
            [['id_inventory_type'], 'integer'],
            [['selected'], 'string'],
            [['calculable'], 'string'],
            [['created', 'modified'], 'safe'],
            [['reference', 'measured_unit'], 'string', 'max' => 45],
            [['id_category', 'description', 'name_category'], 'string', 'max' => 250],
            [['created_by', 'modified_by'], 'string', 'max' => 150],
            [['id_inventory_type'], 'exist', 'skipOnError' => true, 'targetClass' => InventoryType::className(), 'targetAttribute' => ['id_inventory_type' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id_inventory_type' => Yii::t('app', 'Inventory Type ID'),
            'reference' => Yii::t('app', 'Referencee'),
            'id_category' => Yii::t('app', 'Id Category'),
            'description' => Yii::t('app', 'Description'),
            'name_category' => Yii::t('app', 'Name Category'),
            'measured_unit' => Yii::t('app', 'Measured Unit'),
            'margin_loss' => Yii::t('app', 'Margin Loss'),
            'selected' => Yii::t('app', 'Selected'),
            'calculable' => Yii::t('app', 'Calculable'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),

        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryType()
    {
        return $this->hasOne(InventoryType::className(), ['id' => 'id_inventory_type']);
    }

    /**
     *
     * @return array|bool|null
     */
    public function getInfoReference($reference){
        $siesa = new ClientSiesa();

        try{
            $plan_category = Param::findOne(['key' => 'plan_category'])->value;

            return $siesa->getInfoReference($reference, $plan_category);
        }catch (\Exception $e){
            return false;
        }
    }
}
