<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;
use app\components\ClientSiesa;

/**
 * This is the model class for table "planning".
 *
 * @property int $id
 * @property int $id_inventory_type
 * @property string $code_pdv
 * @property string $name_pdv
 * @property string $id_business_unit
 * @property string $date
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property InventoryType $inventoryType
 */
class Planning extends ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'planning';
    }

    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ?
                    Yii::$app->user->identity->username : ''
            ]
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id_inventory_type', 'code_pdv', 'name_pdv', 'date', 'id_business_unit'], 'required'],
            [['id_inventory_type'], 'integer'],
            [['date', 'created', 'modified'], 'safe'],
            [['code_pdv'], 'string', 'max' => 25],
            [['name_pdv'], 'string', 'max' => 250],
            [['created_by', 'modified_by'], 'string', 'max' => 150],
            [['id_inventory_type'], 'exist', 'skipOnError' => true, 'targetClass' => InventoryType::className(), 'targetAttribute' => ['id_inventory_type' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_inventory_type' => Yii::t('app', 'Name Type Inventario'),
            'code_pdv' => Yii::t('app', 'Code Pdv'),
            'name_pdv' => Yii::t('app', 'Name Pdv'),
            'date' => Yii::t('app', 'Date'),
            'id_business_unit' => Yii::t('app', 'Id Business Unit'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By')
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryType()
    {
        return $this->hasOne(InventoryType::className(), ['id' => 'id_inventory_type']);
    }

    /**
     * Retorna Unidades de Negocio de una compañia
     * @param $cia
     * @return mixed
     */
    public function getBusinessUnit($cia){
        $siesa = new ClientSiesa;
        return $siesa->getBusinessUnit($cia);
    }

}
