<?php

use yii\helpers\Html;
use yii\widgets\DetailView;
use kartik\grid\GridView;
use kartik\export\ExportMenu;

/* @var $this yii\web\View */
/* @var $model app\models\CalculatorMainItems */

$this->title = $model->name;
$this->params['breadcrumbs'][] = ['label' => Yii::t('app', 'References'), 'url' => ['index']];
$this->params['breadcrumbs'][] = $this->title;
?>

<div class="calculate-view box box-solid">
    <div class="box-header with-border">
        <?= Html::a(Yii::t('app', 'Update'), ['update', 'id' => $model->id], ['class' => 'btn btn-primary']) ?>
        <?= Html::a(Yii::t('app', 'Delete'), ['delete', 'id' => $model->id], [
            'class' => 'btn btn-danger',
            'data' => [
                'confirm' => Yii::t('app', 'Are you sure you want to delete this item?'),
                'method' => 'post',
            ],
        ]) ?>
        <?= Html::a(Yii::t('app', 'Cancel'), ['index', 'id' => $model->id], ['class' => 'btn btn-default']) ?>
    </div>
    <div class="box-body">
        <?= DetailView::widget([
            'model' => $model,
            'attributes' => [
                'name',
                [
                    'attribute'=>'code_type',
                    'label' => Yii::t('app', 'Type'),
                    'filter'=>$types,
                    'value' => function($data) use ($types){
                        return $types[$data->code_type];
                    }
                ],
                'quantity_liters',
                'quantity_grams',
                'created_by',
                'created',
                'modified_by',
                'modified'
            ]
        ]) ?>
    </div>
</div>

<div class="calculate-secondary-view box box-solid">
    <div class="box-header with-border">
        <h4><?= Yii::t('app', 'Equivalences') ?></h4>
    </div>
    <div class="box-body">
        <div class="calculate-secondary-index">
            <small>
                <?= GridView::widget([
                    'dataProvider' => $references,
                    'responsive' => true,
                    'responsiveWrap' => false,
                    'resizableColumns' => true,
                    'floatHeader' => false,
                    'floatHeaderOptions' => ['scrollingTop' => '0'],
                    'columns' => [
                        ['class' => 'yii\grid\SerialColumn'],
                        'code',
                        'description',
                        'equivalence',
                        [
                            'attribute'=>'code_unity',
                            'label' => Yii::t('app', 'Name Unity'),
                            'filter'=>$measurementUnits,
                            'value' => function($data) use ($measurementUnits){
                                return $measurementUnits[$data->code_unity];
                            }
                        ],
                        'created_by',
                        'created',
                        'modified_by',
                        'modified',
                    ],
                ]); ?>
            </small>
        </div>
    </div>
</div>
