<?php

use yii\helpers\Html;
use yii\widgets\DetailView;
use kartik\grid\GridView;
use kartik\export\ExportMenu;
use yii\helpers\ArrayHelper;
use app\models\InventoryType;
use app\models\Inventory;


/* @var $this yii\web\View */
/* @var $model app\models\Inventory */

$roles = \Yii::$app->authManager->getRolesByUser(Yii::$app->user->identity->getId());
$admin = array_key_exists ('Administrador', $roles);
$admin_pdv = array_key_exists ('Administrador de PVD', $roles);
$auxiliar = array_key_exists ('Auxiliar de Inventario', $roles);
$planeador = array_key_exists ('Planeador de Inventarios', $roles);

$this->title =  Yii::t('app', 'Inventory').' # '.$model->consecutive;
$this->params['breadcrumbs'][] = ['label' => Yii::t('app', 'Inventories'), 'url' => ['index']];
$this->params['breadcrumbs'][] = $this->title;

$btnRefress = Html::a("<i class='glyphicon glyphicon-refresh'></i> "
    .Yii::t('app', 'Refrescar'), ['view', 'id'=>$model->id],['class' => 'btn btn-default']);

    $columns = [
        ['class' => 'yii\grid\SerialColumn'],
        'reference' => [
            'label' => Yii::t('app', 'Referencia'),
            'value' => 'reference'
        ],
        'name_category' => [
            'label' => Yii::t('app', 'Category'),
            'value' => 'name_category',
            'visible' => ($admin == TRUE || $admin_pdv == TRUE)
        ],
        'description' => [
            'label' => Yii::t('app', 'Description'),
            'value' => 'description'
        ],
        'measured_unit' => [
            'label' => Yii::t('app', 'Measured Unit'),
            'value' => 'measured_unit'
        ],
        'cellar' => [
            'label' => Yii::t('app', 'Cellar'),
            'value' => 'cellar',
            'visible' => ($admin == TRUE)
        ],
        [
            'header'=>Yii::t('app', 'Available quantity (#)'),
            'value' => 'current_inventory',
            'visible' => (($admin == TRUE || $admin_pdv == TRUE) && $model->code_status != 'pendiente_cont_1'),
            'format'=>['decimal',2]
        ],
        'unit_price' => [
            'label' => Yii::t('app', 'Unit Price'),
            'value' => 'unit_price',
            'visible' => ($admin == TRUE || $admin_pdv == TRUE),
            'format'=>['decimal',3]
        ],
        'counting_1' => [
            'label' => Yii::t('app', 'Cantidad - Primer conteo (#)'),
            'value' => 'counting_1'
        ],
        'assemble_1' => [
            'label' => Yii::t('app', 'Cantidad - Primer ensamble (#)'),
            'value' => 'assemble_1'
        ],
        'counting_2' => [
            'label' => Yii::t('app', 'Cantidad - Segundo conteo (#)'),
            'value' => 'counting_2'
        ],
        'assemble_2' => [
            'label' => Yii::t('app', 'Cantidad - Segundo ensamble (#)'),
            'value' => 'assemble_2'
        ],
        [
            'attribute' => 'difference_amount',
            'label' => Yii::t('app', 'Difference Amount'),
            'visible' => ($admin == TRUE || $admin_pdv == TRUE)
        ],
        [
            'attribute' => 'difference_price',
            'label' => Yii::t('app', 'Difference Price'),
            'visible' => ($admin == TRUE || $admin_pdv == TRUE)
        ],[
            'attribute' => 'notes',
            'label' => Yii::t('app', 'Notes'),
            'visible' => ($admin == TRUE || $admin_pdv == TRUE)
        ],
        [
            'attribute' => 'reason',
            'value' => function($data) use ($reasons){
               if(empty($reasons[$data['reason']]) == false){
                    return $reasons[$data['reason']];
                }
            },
            'visible' => ($admin == TRUE || $admin_pdv == TRUE)
        ]
    ];
    //Listas sortable
    $gridColumns = $columns;
    \yii\widgets\Pjax::begin();
    $fullExportMenu = ExportMenu::widget([
        'dataProvider' => $details,
        'columns' => $columns,
        'columnSelectorOptions' => [
            'label' => Yii::t('app', 'Columnas'),
        ],
        'dropdownOptions' => [
            'label' => Yii::t('app', 'Exportar'),
            'class' => 'btn btn-default'
        ],
        //'fontAwesome' => true,
        'showConfirmAlert' => false,
        'target' => '_blank',
        'filename' => $this->title . '_' . date('Ymd'),
        'exportRequestParam' => 'exportar',
        'columnBatchToggleSettings' => [
            'label' => Yii::t('app', 'Todo')
        ],
        'exportConfig' => [
            ExportMenu::FORMAT_TEXT => false,
            ExportMenu::FORMAT_PDF => false,
            ExportMenu::FORMAT_HTML => false,
        ]
    ]);
    \yii\widgets\Pjax::end();
?>

<div class="inventory-view box box-solid">
    <div class="box-header with-border">
        <?= Html::a(Yii::t('app', 'Cancel'), ['index', 'id' => $model->id], ['class' => 'btn btn-default']) ?>

        <?php
            $validate_bt = false;
            $count_bt = false;
            $adjust = false;
            $validate_view = false;
            $count_view = false;
            $adjust_view = false;

            if($model->code_status == Inventory::STATUS_PENDIENTE_1 || $model->code_status == Inventory::STATUS_PENDIENTE_2){
                $validate_bt = true;
                $count_bt = true;
            }else if($model->code_status == Inventory::STATUS_DESCUADRE){
                $validate_bt = true;
            }else if($model->code_status == Inventory::STATUS_PENDIENTE_AJUSTE){
                $adjust = true;
            }

            if($admin == true):
                if($validate_bt == true):
                    echo Html::a(Yii::t('app', 'Validate Only'),
                        ['update', 'id' => $model->id, 'type' => 'validate'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $validate_view = true;
                endif;
                if($count_bt == true):
                    echo Html::a(Yii::t('app', 'Count Only'),
                        ['update', 'id' => $model->id, 'type' => 'count'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $count_view = true;
                endif;
                if($adjust == true):
                    echo Html::a(Yii::t('app', 'Adjust Only'),
                        ['update', 'id' => $model->id, 'type' => 'adjust'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $adjust_view = true;
                endif;
            endif;

            if($admin_pdv == true):
                if($validate_bt == true && $model->created_by == Yii::$app->user->identity->username && $validate_view == false):
                    echo Html::a(Yii::t('app', 'Validate Only'),
                        ['update', 'id' => $model->id, 'type' => 'validate'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $validate_view = true;
                endif;
            endif;

            if($planeador == true):
                if($adjust == true && $adjust_view == false):
                    echo Html::a(Yii::t('app', 'Adjust Only'),
                        ['update', 'id' => $model->id, 'type' => 'adjust'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $adjust_view = true;
                endif;
            endif;

            if($auxiliar == true):
                if($count_bt == true && $count_view == false):
                    echo Html::a(Yii::t('app', 'Count Only'),
                        ['update', 'id' => $model->id, 'type' => 'count'],
                        ['class' => 'btn btn-primary margin-r-5']);
                    $count_view = true;
                endif;
            endif;
        ?>
    </div>
    <div class="box-body">
          <?= DetailView::widget([
            'model' => $model,
            'attributes' => [
                [
                    'label' => Yii::t('app', 'Consecutive'),
                    'attribute'=>'consecutive',
                    'visible' => ($admin == TRUE || $admin_pdv == TRUE)
                ],
                [
                    'label' => 'Compañia',
                    'attribute'=>'id_inventory_type',
                    'filter' => ArrayHelper::map(InventoryType::find()
                        ->select(['id_cia', 'name_cia'])
                        ->groupBy('name_cia')
                        ->asArray()->all(), 'id_cia', 'name_cia'),
                    'value'=>function($data){
                        if(empty($data->inventoryType) == FALSE){
                            return $data->inventoryType->name_cia;
                        }
                    },
                    'visible' => ($admin == TRUE || $admin_pdv == TRUE)
                ],
                [
                    'label' => 'Grupo de Centro de Operaciones',
                    'attribute'=>'id_inventory_type',
                    'filter' => ArrayHelper::map(InventoryType::find()
                        ->select(['id_group_co', 'name_group_co'])
                        ->groupBy('name_group_co')
                        ->asArray()->all(), 'id_group_co', 'name_group_co'),
                    'value'=>function($data){
                        if(empty($data->inventoryType) == FALSE){
                            return $data->inventoryType->name_group_co;
                        }
                    },
                    'visible' => ($admin == TRUE || $admin_pdv == TRUE)
                ],
                [
                    'label' => 'Unidad de Negocio',
                    'attribute'=>'business_unit',
                    'value'=>function($data){
                        if(empty($data->business_unit) == FALSE){
                            return $data->business_unit;
                        }
                    },
                    'visible' => ($admin == TRUE || $admin_pdv == TRUE)
                ],
                'name_pdv',
                'code_name',
                [
                    'attribute'=>'id_inventory_type',
                    'filter' => Yii::$app->runAction('generic/get-inventory-type-drop', []),
                    'value'=>function($data){
                        if(empty($data->inventoryType) == FALSE){
                            return $data->inventoryType->name;
                        }
                    },
                ],
                'created_by',
                'created',
            ],
        ]) ?>
    </div>
</div>

<div class="inventory-view box box-solid">
    <div class="box-header with-border">
        <h4><?= Yii::t('app', 'Products') ?></h4>
    </div>
    <div class="box-body">
        <div class="inventorydetail-index">
            <?php if($admin == TRUE || $admin_pdv == TRUE):
                $layout = <<< HTML
                <div class="box-header with-border" >
                {$fullExportMenu} {toggleData} {$btnRefress}
                </div>
                    {summary}
                <div class="clearfix"></div>
                    {items}
                {pager}
HTML;
            ?>
            <div class="box-body table-responsive tableFixHead">
                <small>
                    <?php \yii\widgets\Pjax::begin(); ?>
                    <?= GridView::widget([
                        'dataProvider' => $details,
                        'responsive' => false,
                        'responsiveWrap' => false,
                        'resizableColumns'=>false,
                        'floatHeader'=>false,
                        'floatHeaderOptions'=>['scrollingTop'=>'0'],
                        'layout' => $layout,
                        'toolbar'=> [
                            '{export}',
                            '{toggleData}',
                        ],
                        'export'=>[
                            'fontAwesome'=>true
                        ],
                        'exportConfig' => [
                            GridView::EXCEL => true,
                            GridView::PDF => true
                        ],
                        'columns' => $columns,
                    ]); ?>
                    <?php \yii\widgets\Pjax::end(); ?>
                </small>
            </div>
            <?php else: ?>
            <div class="box-body table-responsive tableFixHead">
                <small>
                    <?php \yii\widgets\Pjax::begin(); ?>
                    <?= GridView::widget([
                        'dataProvider' => $details,
                        'responsive' => true,
                        'responsiveWrap' => false,
                        'resizableColumns'=>true,
                        'floatHeader'=>false,
                        'floatHeaderOptions'=>['scrollingTop'=>'0'],
                        'columns' => $columns
                    ]); ?>
                    <?php \yii\widgets\Pjax::end(); ?>
                </small>
            </div>
            <?php endif; ?>
        </div>
    </div>
</div>