<?php

use yii\helpers\Url;
use yii\helpers\Html;
use yii\widgets\ActiveForm;
use kartik\select2\Select2;
use yii\web\JsExpression;

/* @var $this yii\web\View */
/* @var $form yii\widgets\ActiveForm */

$DragJS = <<<EOF
/* initialize the external events
-----------------------------------------------------------------*/
$('#external-events .fc-event').each(function() {
    // store data so the calendar knows to render an event upon drop
    $(this).data('event', {
        title: $.trim($(this).text()), // use the element's text as the event title
        stick: true // maintain when user navigates (see docs on the renderEvent method)
    });
    // make the event draggable using jQuery UI
    $(this).draggable({
        zIndex: 999,
        revert: true,      // will cause the event to go back to its
        revertDuration: 0  //  original position after the drag
    });
});
EOF;
$this->registerJs($DragJS);

$JSCode = <<<EOF
function(start, end) {
    $("#planning-code_pdv").val($("#pdv").val());
    $("#planning-name_pdv").val($("#pdv option:selected").text());
    $("#dialog-calendar").dialog("open");
    $("#planning-date").val(start.format("YYYY-MM-DD"));  
}
EOF;

$JSDropEvent = <<<EOF
function(date) {
    alert("Dropped on " + date.format());
    if ($('#drop-remove').is(':checked')) {
        // if so, remove the element from the "Draggable Events" list
        $(this).remove();
    }
}
EOF;

$JSEventClick = <<<EOF
function(calEvent, jsEvent, view) {
    $("#id-remove").val(calEvent.id);
    $("#dialog-remove").dialog("open");
}
EOF;

?>

<div id="dialog-remove" class="ui-widget display-none">
    <h4 class="text-center font-weight-bold"><?php echo Yii::t('app', 'Are you sure you want to delete this event?'); ?></h4>
    <input type="hidden" id="id-remove">
</div>

<div id="dialog-calendar" class="ui-widget dialog display-none">
    <div id="content-dialog">
        <?php if(empty($type_list) == FALSE && empty($business_units) == FALSE): ?>
            <?php $form = ActiveForm::begin(); ?>

            <label class="font-weight-bold margin-t-15"><?php echo Yii::t('app', 'Inventory Type'); ?></label>

            <?php echo $form->field($model, 'id_inventory_type')->dropDownList($type_list)->label(false); ?>

            <label class="font-weight-bold"><?php echo Yii::t('app', 'Business Unit'); ?></label>

            <?php echo $form->field($model, 'id_business_unit')->dropDownList($business_units)->label(false); ?>

            <?php echo $form->field($model, 'code_pdv')->hiddenInput()->label(false)->error(false); ?>

            <?php echo $form->field($model, 'name_pdv')->hiddenInput()->label(false)->error(false); ?>

            <?= $form->field($model, 'date')->hiddenInput()->label(false)->error(false); ?>

            <?php ActiveForm::end(); ?>

            <div id="error-calendar" class="alert alert-danger display-none" role="alert"></div>
        <?php else: ?>
            <h4 class="text-center font-weight-bold"><?php echo Yii::t('app', 'There are no types of Inventories the system'); ?></h4>
        <?php endif; ?>
    </div>
</div>

<div class="site-index">
    <?= yii2fullcalendar\yii2fullcalendar::widget(array(
        'id' => 'calendar',
        'events' => $events,
        'clientOptions' => [
            'selectable' => true,
            'selectHelper' => true,
            'droppable' => true,
            'editable' => true,
            'eventClick' => new JsExpression($JSEventClick),
            'drop' => new JsExpression($JSDropEvent),
            'select' => new JsExpression($JSCode),
            'start' => date('Y-'.$month.'-d')
        ],
    ));
    ?>
</div>

<script>
    function readloagDialog(){
        <?php if(empty($type_list) == TRUE): ?>
        $("#dialog-calendar").dialog({
            autoOpen: false,
            height: 'auto',
            width: 350,
            modal: true,
            cache: false,
            buttons: {
                "<?php echo Yii::t('app', 'Close') ?>": function() {
                    $('#error-calendar').hide();
                    $('#error-calendar').text('');
                    $(this).dialog( "close" );
                }
            }
        });
        <?php else: ?>
        $("#dialog-calendar").dialog({
            autoOpen: false,
            height: 'auto',
            width: 350,
            modal: true,
            cache: false,
            buttons: {
                "<?php echo Yii::t('app', 'Save') ?>": function() {
                    $('#error-calendar').hide();
                    $('#error-calendar').text('');
                    if($("#planning-date").val() != '' && $("#planning-id_inventory_type").val() != '' && $("#planning-id_business_unit").val() != '') {
                        let date = $("#planning-date").val();
                        let id_inventory_type = $("#planning-id_inventory_type").val();
                        let id_business_unit = $("#planning-id_business_unit").val();
                        let code_pdv = $("#planning-code_pdv").val();
                        var eventData;

                        $.ajax({
                            url: '<?php echo Url::to(['planning/create']); ?>',
                            type: "GET",
                            data: {
                                "date": date,
                                "id_inventory_type": id_inventory_type,
                                "id_business_unit": id_business_unit,
                                "code_pdv": code_pdv
                            },
                            success: function(result){
                                if(result.status == 'ERROR'){
                                    $('#error-calendar').show();
                                    $('#error-calendar').text(result.message);
                                    return false;
                                }else if(result.status == 'OK'){
                                    $("#planning-date").val('');
                                    $("#dialog-calendar").dialog("close");
                                    eventData = {
                                        id: result.id,
                                        title: result.name,
                                        start: result.start
                                    };
                                    $('#calendar').fullCalendar('renderEvent', eventData, true);
                                    return false;
                                }
                            },
                            error: function (request, status, error) {
                                $("#planning-date").val('');
                                $("#dialog-calendar").dialog("close");
                                return false;
                            }
                        });
                    }
                    $('#calendar').fullCalendar('unselect');
                },
                "<?php echo Yii::t('app', 'Close') ?>": function() {
                    $('#error-calendar').hide();
                    $('#error-calendar').text('');
                    $(this).dialog( "close" );
                }
            }
        });
        <?php endif; ?>

        return true;
    }


    $(".ui-dialog-titlebar-close").on('click', function () {
        $('#error-calendar').hide();
        $('#error-calendar').text('');
    });

    $("#dialog-remove").dialog({
        autoOpen: false,
        height: 'auto',
        width: 350,
        modal: true,
        buttons: {
            "<?php echo Yii::t('app', 'Delete') ?>": function() {
                if($("#id-remove").val() != ''){
                    $.ajax({
                        url: '<?php echo Url::to(['planning/delete']); ?>',
                        type: "POST",
                        data: {"id": $("#id-remove").val()},
                        success: function(result){
                            $('#calendar').fullCalendar('removeEvents', $("#id-remove").val());
                            $("#dialog-remove").dialog( "close" );
                            return false;
                        }
                    });
                }
            },
            "<?php echo Yii::t('app', 'Close') ?>": function() {
                $(this).dialog("close");
                $("#id-remove").val('');
            }
        }
    });

    $(document).ready(function () {
        readloagDialog();
    });
</script>