<?php

use yii\helpers\Html;
use kartik\grid\GridView;
use kartik\export\ExportMenu;
use yii\helpers\ArrayHelper;
use app\models\InventoryType;
use app\models\Usrpdv;

/* @var $this yii\web\View */
/* @var $searchModel app\models\InventorySearch */
/* @var $model app\models\Inventory */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = Yii::t('app', 'Inventories');
$this->params['breadcrumbs'][] = $this->title;
$roles = \Yii::$app->authManager->getRolesByUser(Yii::$app->user->identity->getId());
$admin = array_key_exists ('Administrador', $roles);
$admin_pdv = array_key_exists ('Administrador de PVD', $roles);
$auxiliar = array_key_exists ('Auxiliar de Inventario', $roles);
$planeador = array_key_exists ('Planeador de Inventarios', $roles);

$btnRefress = Html::a("<i class='glyphicon glyphicon-refresh'></i> "
    .Yii::t('app', 'Refrescar'), ['index'], ['class' => 'btn btn-default']);
$redirect = null;
$view_list = '';
$columns = [
    [
        'label' => Yii::t('app', 'Consecutive'),
        'attribute'=>'consecutive',
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    [
        'label' => 'Compañia',
        'attribute'=>'id_cia',
        'filter' => ArrayHelper::map(InventoryType::find()
            ->select(['id_cia', 'name_cia'])
            ->groupBy('name_cia')->asArray()->all(), 'id_cia', 'name_cia'),
        'value'=>function($data){
            if(empty($data->inventoryType) == false){
                return $data->inventoryType->name_cia;
            }
        },
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    [
        'label' => 'Grupo de Centro de Operaciones',
        'attribute'=>'id_group',
        'filter' => ArrayHelper::map(Usrpdv::find()
            ->where(['id_user' => Yii::$app->user->identity->getId()])
            ->groupBy('id_group_co')->all(), 'id_group_co', 'name_group_co'),
        'value'=>function($data){
            if(empty($data->inventoryType) == false){
                return $data->inventoryType->name_group_co;
            }
        },
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    [
        'label' => 'Unidad de Negocio',
        'attribute'=>'business_unit',
        'value'=>function($data){
            if(empty($data->business_unit) == false){
                return $data->business_unit;
            }
        },
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    'name_pdv',
    [
        'attribute'=>'id_inventory_type',
        'filter' => Yii::$app->runAction('generic/get-inventory-type-drop', []),
        'value'=>function($data){
            if(empty($data->inventoryType) == false){
                return $data->inventoryType->name;
            }
        },
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    [
        'attribute'=>'code_name',
        'filter' => Yii::$app->runAction('generic/get-inventory-status-drop', []),
        'value'=>function($data){
            if(empty($data->code_name) == false){
                return $data->code_name;
            }
        },
        'visible' => ($admin == true || $admin_pdv == true)
    ],
    'created_by',
    'created',
];


$validate_bt = false;
$count_bt = false;
$adjust = false;
$validate_view = false;
$count_view = false;
$adjust_view = false;

//Listas sortable
$gridColumns = $columns;
$gridColumns[] = [
        'class' => 'yii\grid\ActionColumn',
        'template' => '{view}{validate}{count}{delete}',
        'visibleButtons' => [
            'delete' => function ($data) use ($admin, $admin_pdv){
                if($admin == true || $admin_pdv  == true){
                    if($admin == false && $data->created_by != \Yii::$app->user->identity->username){
                        return false;
                    }
                    return true;
                }
            }
        ]
    ];

$fullExportMenu = ExportMenu::widget([
    'dataProvider' => $dataProvider,
    'columns' => $columns,
    'columnSelectorOptions' => [
        'label' => Yii::t('app', 'Columnas'),
    ],
    'dropdownOptions' => [
        'label' => Yii::t('app', 'Exportar'),
        'class' => 'btn btn-default'
    ],
    //'fontAwesome' => true,
    'showConfirmAlert' => false,
    'target' => '_blank',
    'filename' => $this->title . '_' . date('Ymd'),
    'exportRequestParam' => 'exportar',
    'columnBatchToggleSettings' => [
        'label' => Yii::t('app', 'Todo')
    ],
    'exportConfig' => [
        ExportMenu::FORMAT_TEXT => false,
        ExportMenu::FORMAT_PDF => false,
        ExportMenu::FORMAT_HTML => false,
    ]
]);
?>

<div id="dialog-type-inventory" class="ui-widget display-none">
    <?php if(empty($planning) == true): ?>
        <div id="error-pdv" class="alert alert-danger margin-t-15" role="alert">
            <?php echo Yii::t('app', 'There is no registered Plan for today, please contact the person in charge of the Planning of your Point of Sale.'); ?>
        </div>
    <?php else: ?>
        <h4 class="text-center font-weight-bold"><?php echo Yii::t('app', 'Create Inventory'); ?></h4>

        <label class="text-center margin-t-15"><?php echo Yii::t('app', 'Please select PDV'); ?></label>
        <?php echo Html::dropDownList('', [], ArrayHelper::map($planning, 'id', 'name_pdv'), ['class' => 'form-control', 'id' => 'id-planning']); ?>
    <?php endif; ?>
    <input type="hidden" id="id-remove">
</div>

<div id="div-type-inventory" class="inventory-index box box-solid">
    <?php if($admin == true || $admin_pdv == true): ?>
        <div class="box-header with-border">
            <p>
                <?php echo Html::a(Yii::t('app', 'Create Inventory'), null, ['class' => 'btn btn-success view-list-inventory']) ?>
            </p>
        </div>
    <?php endif; ?>

    <div class="box-body">
        <?php
        if($auxiliar == false && $planeador == false):

        $layout = <<< HTML
        <div class="box-header with-border" >
        {$fullExportMenu} {toggleData} {$btnRefress}
        </div>
            {summary}
        <div class="clearfix"></div>
            {items}
        {pager}
HTML;
        ?>
            <small>
                <?= GridView::widget([
                    'dataProvider' => $dataProvider,
                    'filterModel' => $searchModel,
                    'columns' => $gridColumns,
                    'responsive' => true,
                    'responsiveWrap' => false,
                    'resizableColumns'=>true,
                    'floatHeader'=>false,
                    'floatHeaderOptions'=>['scrollingTop'=>'0'],
                    'layout' => $layout,
                    'toolbar'=> [
                        '{export}',
                        '{toggleData}',
                    ],
                    'export'=>[
                        'fontAwesome'=>true
                    ],
                    'exportConfig' => [
                        GridView::EXCEL => true,
                        GridView::PDF => true
                    ]
                ]); ?>
            </small>

        <?php else: ?>
            <small>
                <?= GridView::widget([
                    'dataProvider' => $dataProvider,
                    'filterModel' => $searchModel,
                    'columns' => $gridColumns,
                    'responsive' => true,
                    'responsiveWrap' => false,
                    'resizableColumns'=>true,
                    'floatHeader'=>false,
                    'floatHeaderOptions'=>['scrollingTop'=>'0']
                ]); ?>
            </small>
        <?php endif; ?>
    </div>
</div>

<?php echo $this->render(
    '_main.php', [
        'planning' => $planning
    ]
) ?>


